@@ -1,15 +1,16 @@
-/* Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -248,30 +249,6 @@ class Event_reader {
     return go_to((m_ptr - m_buffer) + bytes);
   }
 
-  /**
-    Reads a basic type - bool, char, int, long, double, etc - from the buffer,
-    moves the cursor forward the number of bytes returned by sizeof(T)) and
-    returns the read value.
-
-    @retval value the T read from the cursor position.
-    @retval 0 if the cursor was out of buffer boundaries.
-  */
-  template <class T>
-  T read() {
-    PRINT_READER_STATUS("Event_reader::read");
-    if (!can_read(sizeof(T))) {
-      set_error("Cannot read from out of buffer bounds");
-      BAPI_PRINT("debug", ("Event_reader::tread(): "
-                           "sizeof()= %zu",
-                           sizeof(T)));
-      return 0;
-    }
-    T value = 0;
-    value = (T) * (m_ptr);
-    m_ptr = m_ptr + sizeof(T);
-    return value;
-  }
-
   /**
     Copies a basic type - bool, char, int, long, double, etc - from the buffer,
     moves the cursor forward the number of bytes returned by sizeof(T)) and
@@ -310,11 +287,11 @@ class Event_reader {
               to allocate to the new string..
   */
   template <typename T>
-  T read_and_letoh(unsigned char bytes = sizeof(T)) {
-    PRINT_READER_STATUS("Event_reader::read_and_letoh");
+  T read(unsigned char bytes = sizeof(T)) {
+    PRINT_READER_STATUS("Event_reader::read");
     if (!can_read(bytes)) {
       set_error("Cannot read from out of buffer bounds");
-      BAPI_PRINT("debug", ("Event_reader::read_and_letoh(): "
+      BAPI_PRINT("debug", ("Event_reader::read(): "
                            "sizeof()= %zu, bytes= %u",
                            sizeof(T), bytes));
       return 0;
@@ -322,7 +299,7 @@ class Event_reader {
     T value = 0;
     ::memcpy((char *)&value, m_ptr, bytes);
     m_ptr = m_ptr + bytes;
-    return letoh(value);
+    return (bytes > 1) ? letoh(value) : value;
   }
 
   /**
@@ -485,7 +462,7 @@ class Event_reader {
   const char *m_error;
 
   /**
-    Wrapper to le16toh to be used by read_and_letoh function.
+    Wrapper to le16toh to be used by read function.
 
     @param[in] value the value to be converted.
 
@@ -494,7 +471,7 @@ class Event_reader {
   uint16_t letoh(uint16_t value) { return le16toh(value); }
 
   /**
-    Wrapper to le32toh to be used by read_and_letoh function.
+    Wrapper to le32toh to be used by read function.
 
     @param[in] value the value to be converted.
 
@@ -503,7 +480,7 @@ class Event_reader {
   int32_t letoh(int32_t value) { return le32toh(value); }
 
   /**
-    Wrapper to le32toh to be used by read_and_letoh function.
+    Wrapper to le32toh to be used by read function.
 
     @param[in] value the value to be converted.
 
@@ -512,7 +489,7 @@ class Event_reader {
   uint32_t letoh(uint32_t value) { return le32toh(value); }
 
   /**
-    Wrapper to le64toh to be used by read_and_letoh function.
+    Wrapper to le64toh to be used by read function.
 
     @param[in] value the value to be converted.
 
@@ -521,7 +498,7 @@ class Event_reader {
   int64_t letoh(int64_t value) { return le64toh(value); }
 
   /**
-    Wrapper to le64toh to be used by read_and_letoh function.
+    Wrapper to le64toh to be used by read function.
 
     @param[in] value the value to be converted.
 
