@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -123,7 +124,9 @@ Gcs_xcom_control::Gcs_xcom_control(
     Gcs_xcom_engine *gcs_engine,
     Gcs_xcom_state_exchange_interface *state_exchange,
     Gcs_xcom_view_change_control_interface *view_control, bool boot,
-    My_xp_socket_util *socket_util)
+    My_xp_socket_util *socket_util,
+    std::unique_ptr<Network_provider_operations_interface>
+        comms_operation_interface)
     : m_gid(nullptr),
       m_gid_hash(0),
       m_xcom_proxy(xcom_proxy),
@@ -139,6 +142,7 @@ Gcs_xcom_control::Gcs_xcom_control(
       m_suspicions_manager(new Gcs_suspicions_manager(xcom_proxy, this)),
       m_suspicions_processing_thread(),
       m_sock_probe_interface(nullptr),
+      m_comms_operation_interface(std::move(comms_operation_interface)),
       m_xcom_running(false),
       m_leave_view_requested(false),
       m_leave_view_delivered(false),
@@ -287,17 +291,18 @@ enum_gcs_error Gcs_xcom_control::do_join(bool retry) {
     return GCS_NOK;
   }
 
-  while (ret == GCS_NOK) {
+  while (ret == GCS_NOK && !m_view_control->is_finalized()) {
     ret = retry_do_join();
 
     retry_join_count--;
     if (retry && m_join_attempts != 0 && ret == GCS_NOK &&
         retry_join_count >= 1) {
-      MYSQL_GCS_LOG_DEBUG(
-          "Sleeping for %u seconds before retrying to join the group. There "
-          "are "
-          " %u more attempt(s) before giving up.",
-          m_join_sleep_time, retry_join_count);
+      MYSQL_GCS_LOG_INFO("Sleeping for "
+                         << m_join_sleep_time
+                         << " seconds before retrying to join the group. There "
+                            "are "
+                         << retry_join_count
+                         << " more attempt(s) before giving up.");
       My_xp_util::sleep_seconds(m_join_sleep_time);
     } else {
       break;
@@ -334,6 +339,35 @@ static bool skip_own_peer_address(
   return false;
 }
 
+static constexpr const char *get_signaling_error() {
+  return "The group communication engine could not set up its internal event "
+         "notification mechanism. This may be due to one or more invalid "
+         "configuration settings. Double-check your group replication local "
+         "address, firewall"
+#if !defined(_WIN32)
+         ", SE Linux"
+#endif
+         " and TLS configurations and try "
+         "restarting Group Replication on this server.";
+}
+
+static const std::string get_connection_test_error(const std::string &address,
+                                                   int port) {
+  std::stringstream retval;
+  retval << "The group communication engine failed to test connectivity to the "
+            "local group communication engine on "
+         << address << ":" << port
+         << ". This may be due to one or more invalid configuration settings. "
+            "Double-check your group replication local address, firewall";
+#if !defined(_WIN32)
+  retval << ", SE Linux";
+#endif
+  retval << " and TLS configurations and try restarting Group Replication on "
+            "this server.";
+
+  return retval.str();
+}
+
 enum_gcs_error Gcs_xcom_control::retry_do_join() {
   /* Used to initialize xcom */
   int local_port = m_local_node_address->get_member_port();
@@ -358,7 +392,6 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
     MYSQL_GCS_LOG_ERROR("Error initializing the group communication engine.")
     goto err;
   }
-
   // Wait for XCom comms to become ready
   m_xcom_proxy->xcom_wait_for_xcom_comms_status_change(comm_status);
 
@@ -380,8 +413,7 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
      * Tested by the TEST_F(XComControlTest,
      * JoinTestFailedToConnectToXComQueueSignallingMechanism) GCS smoke test.
      */
-    MYSQL_GCS_LOG_ERROR("Error connecting to the local group communication"
-                        << " engine instance.")
+    MYSQL_GCS_LOG_ERROR(get_signaling_error());
     goto err;
     /* purecov: end */
   }
@@ -400,12 +432,18 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
       m_local_node_address->get_member_ip(),
       m_local_node_address->get_member_port());
   if (!could_connect_to_local_xcom) {
-    MYSQL_GCS_LOG_ERROR("Error connecting to the local group communication"
-                        << " engine instance.")
+    MYSQL_GCS_LOG_ERROR(
+        get_connection_test_error(m_local_node_address->get_member_ip(),
+                                  m_local_node_address->get_member_port())
+            .c_str());
     goto err;
   }
 
   if (m_boot) {
+    MYSQL_GCS_LOG_INFO(
+        "Booting a group: "
+        << m_local_node_info->get_member_uuid().actual_value.c_str() << ":"
+        << local_port)
     MYSQL_GCS_LOG_TRACE(
         "::join():: I am the boot node. %d - %s. Calling xcom_client_boot.",
         local_port, m_local_node_info->get_member_uuid().actual_value.c_str())
@@ -458,8 +496,8 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
   }
 
   m_xcom_running = true;
-  MYSQL_GCS_LOG_DEBUG("The member has joined the group. Local port: %d",
-                      local_port);
+  MYSQL_GCS_LOG_INFO(
+      "The member has joined the group. Local port: " << local_port);
 
   m_suspicions_manager->set_groupid_hash(m_gid_hash);
   m_suspicions_manager->set_my_info(m_local_node_info);
@@ -504,6 +542,7 @@ bool Gcs_xcom_control::send_add_node_request(
   // Go through the seed list S_CONNECTION_ATTEMPTS times.
   for (int attempt_nr = 0;
        !add_node_accepted && attempt_nr < CONNECTION_ATTEMPTS; attempt_nr++) {
+    if (m_view_control->is_finalized()) break;
     add_node_accepted = try_send_add_node_request_to_seeds(my_addresses);
   }
 
@@ -516,7 +555,9 @@ bool Gcs_xcom_control::try_send_add_node_request_to_seeds(
 
   // Until the add_node is successfully sent, for each peer...
   for (auto it = m_initial_peers.begin();
-       !add_node_accepted && it != m_initial_peers.end(); it++) {
+       !m_view_control->is_finalized() && !add_node_accepted &&
+       it != m_initial_peers.end();
+       it++) {
     // ...try to connect to it and send it the add_node request.
     Gcs_xcom_node_address &peer = **it;
 
@@ -524,7 +565,13 @@ bool Gcs_xcom_control::try_send_add_node_request_to_seeds(
     connection_descriptor *con = nullptr;
     std::tie(connected, con) = connect_to_peer(peer, my_addresses);
 
-    if (connected) {
+    if (bool finalized = m_view_control->is_finalized();
+        !finalized && connected) {
+      MYSQL_GCS_LOG_INFO("Sucessfully connected to peer "
+                         << peer.get_member_ip().c_str() << ":"
+                         << peer.get_member_port()
+                         << ". Sending a request to be added to the group");
+
       MYSQL_GCS_LOG_TRACE(
           "::join():: Calling xcom_client_add_node %d_%s connected to "
           "%s:%d "
@@ -550,6 +597,8 @@ bool Gcs_xcom_control::try_send_add_node_request_to_seeds(
       */
       if (xcom_will_process) add_node_accepted = true;
     }
+
+    free_connection(con);
   }
 
   return add_node_accepted;
@@ -575,10 +624,11 @@ std::pair<bool, connection_descriptor *> Gcs_xcom_control::connect_to_peer(
       m_local_node_address->get_member_port(), addr.c_str(), port);
 
   con = m_xcom_proxy->xcom_client_open_connection(addr, port);
-  if (con == nullptr) {
+  if (con->fd == -1) {
     // Could not connect to the peer.
-    MYSQL_GCS_LOG_ERROR("Error on opening a connection to "
-                        << addr << ":" << port << " on local port: "
+    MYSQL_GCS_LOG_ERROR("Error on opening a connection to peer node "
+                        << addr << ":" << port
+                        << " when joining a group. My local port is: "
                         << m_local_node_address->get_member_port() << ".");
     goto end;
   }
@@ -748,8 +798,8 @@ connection_descriptor *Gcs_xcom_control::get_connection_to_node(
         "get_connection_to_node: xcom_client_open_connection to %s:%d", addr,
         port)
 
-    if ((con = m_xcom_proxy->xcom_client_open_connection(addr, port)) ==
-        nullptr) {
+    con = m_xcom_proxy->xcom_client_open_connection(addr, port);
+    if (con->fd == -1) {
       MYSQL_GCS_LOG_DEBUG(
           "get_connection_to_node: Error while opening a connection to %s:%d",
           addr, port)
@@ -813,17 +863,21 @@ void Gcs_xcom_control::do_remove_node_from_group() {
     delete current_view;
   }
 
-  if (!con) {
+  if (con->fd == -1) {
     // GET INITIAL PEERS
     MYSQL_GCS_LOG_DEBUG(
         "do_remove_node_from_group: (%d) Couldn't get a connection from view! "
         "Using initial peers...",
         local_port)
 
+    free(con);
+    con = nullptr;
+
     con = get_connection_to_node(&m_initial_peers);
   }
 
-  if (con && !m_leave_view_delivered && m_view_control->belongs_to_group()) {
+  if (con->fd != -1 && !m_leave_view_delivered &&
+      m_view_control->belongs_to_group()) {
     MYSQL_GCS_LOG_TRACE(
         "do_remove_node_from_group: (%d) got a connection! "
         "m_leave_view_delivered=%d belongs=%d",
@@ -841,10 +895,12 @@ void Gcs_xcom_control::do_remove_node_from_group() {
         local_port);
   }
 
-  if (con) {
-    xcom_close_client_connection(con);
+  if (con->fd != -1) {
+    m_xcom_proxy->xcom_client_close_connection(con);
   }
 
+  if (con != nullptr) free(con);
+
   /*
     Destroy this node's stored suspicions to avoid them from unnecessary
     processing by the manager.
@@ -1130,7 +1186,7 @@ bool Gcs_xcom_control::xcom_receive_local_view(synode_no const config_id,
       Identify which nodes are alive and which are considered faulty.
 
       Note that there may be new nodes that are marked as faulty because the
-      connections among their peers are still beeing established.
+      connections among their peers are still being established.
     */
     build_total_members(xcom_nodes, alive_members, failed_members);
 
@@ -1311,7 +1367,7 @@ bool Gcs_xcom_control::is_this_node_in(
 bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
                                                 synode_no message_id,
                                                 Gcs_xcom_nodes *xcom_nodes,
-                                                bool same_view,
+                                                bool do_not_deliver_to_client,
                                                 synode_no max_synode) {
   bool ret = false;
   bool free_built_members = false;
@@ -1341,13 +1397,13 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
     current_members = const_cast<std::vector<Gcs_member_identifier> *>(
         &current_view->get_members());
   MYSQL_GCS_LOG_TRACE("::xcom_receive_global_view():: My node_id is %d",
-                      xcom_nodes->get_node_no())
+                      xcom_nodes->get_node_no());
 
   /*
     Identify which nodes are alive and which are considered faulty.
 
     Note that there may be new nodes that are marked as faulty because the
-    connections among their peers are still beeing established.
+    connections among their peers are still being established.
   */
   build_total_members(xcom_nodes, alive_members, failed_members);
 
@@ -1452,10 +1508,10 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
 
   /*
     If nobody has joined or left the group, there is no need to install any
-    view. This fact is denoted by the same_view flag and the execution only
-    gets so far because we may have to kill some faulty members. Note that
-    the code could be optimized but we are focused on correctness for now
-    and any possible optimization will be handled in the future.
+    view. This fact is denoted by the do_not_deliver_to_client flag and the
+    execution only gets so far because we may have to kill some faulty members.
+    Note that the code could be optimized but we are focused on correctness for
+    now and any possible optimization will be handled in the future.
 
     Views that contain a node marked as faulty are also ignored. This is
     done because XCOM does not deliver messages to or accept messages from
@@ -1471,13 +1527,13 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
     If the execution has managed to pass these steps, the node is alive and
     it is time to start building the view.
   */
-  if (same_view || failed_members.size() != 0) {
+  if (do_not_deliver_to_client || failed_members.size() != 0) {
     MYSQL_GCS_LOG_TRACE(
         "(My node_id is %d) ::xcom_receive_global_view():: "
         "Discarding view because nothing has changed. "
-        "Same view flag is %d, number of failed nodes is %llu"
+        "Do not deliver to client flag is %d, number of failed nodes is %llu"
         ", number of joined nodes is %llu, number of left nodes is %llu",
-        xcom_nodes->get_node_no(), same_view,
+        xcom_nodes->get_node_no(), do_not_deliver_to_client,
         static_cast<long long unsigned>(failed_members.size()),
         static_cast<long long unsigned>(joined_members.size()),
         static_cast<long long unsigned>(left_members.size()));
@@ -1572,6 +1628,8 @@ end:
   return ret;
 }
 
+extern uint32_t get_my_xcom_id();
+
 void Gcs_xcom_control::process_control_message(
     Gcs_message *msg, Gcs_protocol_version maximum_supported_protocol_version,
     Gcs_protocol_version used_protocol_version) {
@@ -1588,8 +1646,9 @@ void Gcs_xcom_control::process_control_message(
           msg->get_message_data().get_payload_length()));
 
   MYSQL_GCS_LOG_TRACE(
-      "::process_control_message():: From: %s regarding view_id: %s in %s",
-      msg->get_origin().get_member_id().c_str(),
+      "xcom_id %x ::process_control_message():: From: %s regarding view_id: %s "
+      "in %s",
+      get_my_xcom_id(), msg->get_origin().get_member_id().c_str(),
       ms_info->get_view_id()->get_representation().c_str(),
       get_node_address()->get_member_address().c_str())
 
@@ -1601,18 +1660,21 @@ void Gcs_xcom_control::process_control_message(
   MYSQL_GCS_DEBUG_EXECUTE(
       synode_no configuration_id = ms_info->get_configuration_id();
       if (!m_view_control->is_view_changing()){MYSQL_GCS_LOG_DEBUG(
-          "There is no state exchange going on. Ignoring exchangeable data "
+          "xcom_id %x There is no state exchange going on. Ignoring "
+          "exchangeable data "
           "because its from a previous state exchange phase. Message is "
           "from group_id (%u), msg_no(%llu), node_no(%llu)",
-          configuration_id.group_id,
+          get_my_xcom_id(), configuration_id.group_id,
           static_cast<long long unsigned>(configuration_id.msgno),
           static_cast<long long unsigned>(
               configuration_id
-                  .node))} MYSQL_GCS_LOG_DEBUG("There is a state exchange "
+                  .node))} MYSQL_GCS_LOG_DEBUG("xcom_id %x There is a state "
+                                               "exchange "
                                                "going on. Message is "
                                                "from group_id (%u), "
                                                "msg_no(%llu), "
                                                "node_no(%llu)",
+                                               get_my_xcom_id(),
                                                configuration_id.group_id,
                                                static_cast<long long unsigned>(
                                                    configuration_id.msgno),
@@ -1812,6 +1874,10 @@ void Gcs_xcom_control::set_node_address(
     init_me() method.
   */
   m_local_node_info = new Gcs_xcom_node_information(address);
+
+  Network_configuration_parameters params;
+  params.port = xcom_node_address->get_member_port();
+  m_comms_operation_interface->configure_active_provider(params);
 }
 
 void Gcs_xcom_control::set_peer_nodes(
@@ -2089,7 +2155,7 @@ void Gcs_suspicions_manager::run_process_suspicions(bool lock) {
   }
 
   // List of nodes to remove
-  Gcs_xcom_nodes nodes_to_remove;
+  Gcs_xcom_nodes nodes_to_remove, nodes_to_remember_expel;
 
   bool force_remove = false;
 
@@ -2129,6 +2195,9 @@ void Gcs_suspicions_manager::run_process_suspicions(bool lock) {
       }
 
       nodes_to_remove.add_node(*susp_it);
+      if ((*susp_it).is_member()) {
+        nodes_to_remember_expel.add_node(*susp_it);
+      }
       m_suspicions.remove_node(*susp_it);
       /* purecov: end */
     } else {
@@ -2162,9 +2231,9 @@ void Gcs_suspicions_manager::run_process_suspicions(bool lock) {
           "process_suspicions: Expelling suspects that timed out!");
       bool const removed =
           m_proxy->xcom_remove_nodes(nodes_to_remove, m_gid_hash);
-      if (removed) {
+      if (removed && !nodes_to_remember_expel.empty()) {
         m_expels_in_progress.remember_expels_issued(m_config_id,
-                                                    nodes_to_remove);
+                                                    nodes_to_remember_expel);
       }
     } else if (force_remove) {
       assert(!m_is_killer_node);
