@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -320,7 +321,8 @@ void Client::handle_message(ngs::Message_request *request) {
           s->handle_message(*request);
         }
         break;
-      }  // Fall through.
+      }
+      [[fallthrough]];
 
     default:
       // invalid message at this time
@@ -420,7 +422,7 @@ void Client::on_client_addr() {
     } break;
 
     case xpl::Connection_namedpipe:
-    case xpl::Connection_unixsocket:  // fall through
+    case xpl::Connection_unixsocket:
       m_client_host = "localhost";
       return;
 
@@ -511,7 +513,7 @@ void Client::on_session_auth_success(xpl::iface::Session *) {
   }
 }
 
-void Client::on_session_close(xpl::iface::Session *s MY_ATTRIBUTE((unused))) {
+void Client::on_session_close(xpl::iface::Session *s [[maybe_unused]]) {
   log_debug("%s: Session %i removed", client_id(), s->session_id());
 
   // no more open sessions, disconnect
@@ -519,7 +521,7 @@ void Client::on_session_close(xpl::iface::Session *s MY_ATTRIBUTE((unused))) {
   remove_client_from_server();
 }
 
-void Client::on_session_reset(xpl::iface::Session *s MY_ATTRIBUTE((unused))) {
+void Client::on_session_reset(xpl::iface::Session *s [[maybe_unused]]) {
   log_debug("%s: Resetting session %i", client_id(), s->session_id());
 
   if (!create_session()) {
