@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -113,15 +114,6 @@ class basic_named_pipe_impl_base {
 
   executor_type get_executor() noexcept { return io_ctx_->get_executor(); }
 
-  stdx::expected<void, std::error_code> close() {
-    if (is_open()) {
-      CloseHandle(native_handle_);
-      native_handle_ = impl::named_pipe::kInvalidHandle;
-    }
-
-    return {};
-  }
-
   stdx::expected<size_t, std::error_code> cancel() {
     // TODO: once the io-context has support for OVERLAPPED IO on windows
     // a full implemenantation can be done.
@@ -174,16 +166,12 @@ class basic_named_pipe_impl : public basic_named_pipe_impl_base {
     if (this == std::addressof(rhs)) {
       return *this;
     }
-
-    close();
     __base::operator=(std::move(rhs));
 
     return *this;
   }
 
-  ~basic_named_pipe_impl() {
-    if (is_open()) close();
-  }
+  ~basic_named_pipe_impl() {}
 
   executor_type get_executor() noexcept { return __base::get_executor(); }
 
@@ -222,13 +210,31 @@ class basic_named_pipe_impl : public basic_named_pipe_impl_base {
           make_error_code(net::socket_errc::already_open));
     }
 
-    native_handle_ =
-        CreateFile(TEXT(ep.path().c_str()), GENERIC_READ | GENERIC_WRITE, 0,
-                   NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+    using clock_type = std::chrono::steady_clock;
+    using namespace std::chrono_literals;
+    const auto retry_step = 10ms;
+    const auto end_time = clock_type::now() + 1s;
+    do {
+      native_handle_ =
+          CreateFile(TEXT(ep.path().c_str()), GENERIC_READ | GENERIC_WRITE, 0,
+                     NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 
-    if (native_handle_ == impl::named_pipe::kInvalidHandle) {
-      return stdx::make_unexpected(win32::last_error_code());
-    }
+      if (native_handle_ == impl::named_pipe::kInvalidHandle) {
+        auto const error_code = win32::last_error_code();
+        // if the pipe is not ready, try waiting max 1 second for it to become
+        // available
+        const std::error_code ec_pipe_busy{ERROR_PIPE_BUSY,
+                                           std::system_category()};
+
+        if ((error_code == ec_pipe_busy) && clock_type::now() < end_time) {
+          std::this_thread::sleep_for(retry_step);
+          continue;
+        }
+
+        return stdx::make_unexpected(error_code);
+      } else
+        break;
+    } while (true);
 
     return {};
   }
@@ -269,9 +275,10 @@ class basic_named_pipe : private basic_named_pipe_impl<Protocol> {
 
   constexpr bool is_open() const noexcept { return __base::is_open(); }
 
-  stdx::expected<void, std::error_code> close() { return __base::close(); }
-
   native_handle_type native_handle() const { return __base::native_handle(); }
+  void native_handle(native_handle_type handle) {
+    __base::native_handle(handle);
+  }
 
   stdx::expected<void, std::error_code> native_non_blocking(bool v) {
     return __base::native_non_blocking(v);
@@ -379,7 +386,9 @@ class basic_named_pipe_socket : public basic_named_pipe<Protocol> {
   // NOLINTNEXTLINE(hicpp-noexcept-move,performance-noexcept-move-constructor)
   basic_named_pipe_socket &operator=(basic_named_pipe_socket &&) = default;
 
-  ~basic_named_pipe_socket() = default;
+  ~basic_named_pipe_socket() {
+    if (is_open()) close();
+  }
 
   constexpr bool is_open() const noexcept { return __base::is_open(); }
 
@@ -394,6 +403,15 @@ class basic_named_pipe_socket : public basic_named_pipe<Protocol> {
     return {};
   }
 
+  stdx::expected<void, std::error_code> close() {
+    if (is_open()) {
+      DisconnectNamedPipe(native_handle());
+      __base::native_handle(impl::named_pipe::kInvalidHandle);
+    }
+
+    return {};
+  }
+
   template <class ConstBufferSequence>
   stdx::expected<size_t, std::error_code> write_some(
       const ConstBufferSequence &buffers) {
@@ -455,7 +473,9 @@ class basic_named_pipe_acceptor : public basic_named_pipe_impl<Protocol> {
   // NOLINTNEXTLINE(hicpp-noexcept-move,performance-noexcept-move-constructor)
   basic_named_pipe_acceptor &operator=(basic_named_pipe_acceptor &&) = default;
 
-  ~basic_named_pipe_acceptor() = default;
+  ~basic_named_pipe_acceptor() {
+    if (is_open()) close();
+  }
 
   executor_type get_executor() noexcept { return __base::get_executor(); }
 
@@ -469,6 +489,15 @@ class basic_named_pipe_acceptor : public basic_named_pipe_impl<Protocol> {
 
   stdx::expected<void, std::error_code> open() { return __base::open(); }
 
+  stdx::expected<void, std::error_code> close() {
+    if (is_open()) {
+      CloseHandle(native_handle());
+      native_handle(impl::named_pipe::kInvalidHandle);
+    }
+
+    return {};
+  }
+
   stdx::expected<native_handle_type, std::error_code> release() {
     return __base::release();
   }
@@ -488,10 +517,14 @@ class basic_named_pipe_acceptor : public basic_named_pipe_impl<Protocol> {
    *
    * in accordance with net-ts' acceptors' bind().
    *
+   * @param ep    an endpoint this method binds to a socket
+   * @param flags flags passed to CreateNamedPipe() as is. Use for PIPE_NOWAIT.
+   *
    * @retval std::errc::invalid_argument if ep.path() is empty
    * @retval std::errc::invalid_argument if already bound.
    */
-  stdx::expected<void, std::error_code> bind(const endpoint_type &ep) {
+  stdx::expected<void, std::error_code> bind(const endpoint_type &ep,
+                                             int flags = 0) {
     if (ep.path().empty()) {
       return stdx::make_unexpected(
           make_error_code(std::errc::invalid_argument));
@@ -505,6 +538,25 @@ class basic_named_pipe_acceptor : public basic_named_pipe_impl<Protocol> {
 
     ep_ = ep;
 
+    const auto protocol = protocol_type();
+
+    if (native_non_blocking_ == 1) flags |= PIPE_NOWAIT;
+
+    if (!is_open()) {
+      auto handle = CreateNamedPipe(
+          TEXT(ep_.path().c_str()), PIPE_ACCESS_DUPLEX,
+          protocol.type() | protocol.protocol() | PIPE_REJECT_REMOTE_CLIENTS |
+              flags,
+          back_log_, 1024 * 16, 1024 * 16, NMPWAIT_USE_DEFAULT_WAIT, NULL);
+
+      if (handle == impl::named_pipe::kInvalidHandle) {
+        auto ec = win32::last_error_code();
+        return stdx::make_unexpected(ec);
+      }
+
+      native_handle(handle);
+    }
+
     return {};
   }
 
@@ -533,12 +585,10 @@ class basic_named_pipe_acceptor : public basic_named_pipe_impl<Protocol> {
    * - CreateNamedPipe() on the endpoint assigned by bind()
    * - ConnectNamedPipe() to accept the client connection.
    *
-   * @param flags passed to CreateNamedPipe() as is. Use for PIPE_NOWAIT.
-   *
    * @return a connected named pipe.
    * @retval std::errc::invalid_argument if no endpoint bound.
    */
-  stdx::expected<socket_type, std::error_code> accept(int flags = 0) {
+  stdx::expected<socket_type, std::error_code> accept() {
     // not bound.
     if (ep_.path().empty()) {
       return stdx::make_unexpected(
@@ -547,23 +597,6 @@ class basic_named_pipe_acceptor : public basic_named_pipe_impl<Protocol> {
 
     const auto protocol = protocol_type();
 
-    if (native_non_blocking_ == 1) flags |= PIPE_NOWAIT;
-
-    if (!is_open()) {
-      auto handle = CreateNamedPipe(
-          TEXT(ep_.path().c_str()), PIPE_ACCESS_DUPLEX,
-          protocol.type() | protocol.protocol() | flags |
-              PIPE_REJECT_REMOTE_CLIENTS,
-          back_log_, 1024 * 16, 1024 * 16, NMPWAIT_USE_DEFAULT_WAIT, NULL);
-
-      if (handle == impl::named_pipe::kInvalidHandle) {
-        auto ec = win32::last_error_code();
-        return stdx::make_unexpected(ec);
-      }
-
-      native_handle(handle);
-    }
-
     const bool connected = ConnectNamedPipe(native_handle(), NULL);
     if (!connected) {
       auto last_ec = win32::last_error_code();
@@ -574,18 +607,17 @@ class basic_named_pipe_acceptor : public basic_named_pipe_impl<Protocol> {
                                        std::system_category()};
 
       // ERROR_PIPE_CONNECTED is also a success
-      // ERROR_NO_DATA too, it just meands the pipe is already closed, but quite
+      // ERROR_NO_DATA too, it just means the pipe is already closed, but quite
       // likely readable.
       if (last_ec == ec_pipe_connected || last_ec == ec_no_data) {
-        return {stdx::in_place, get_executor().context(), protocol,
-                release().value()};
+        return {std::in_place, get_executor().context(), protocol,
+                native_handle()};
       }
 
       return stdx::make_unexpected(last_ec);
     }
 
-    return {stdx::in_place, get_executor().context(), protocol,
-            release().value()};
+    return {std::in_place, get_executor().context(), protocol, native_handle()};
   }
 
   stdx::expected<endpoint_type, std::error_code> local_endpoint() const {
