@@ -1,16 +1,17 @@
 /*
- Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+ Copyright (c) 2018, 2024, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is also distributed with certain software (including
+ This program is designed to work with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have included with MySQL.
+ separately licensed software that they have either included with
+ the program or referenced in the documentation.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,6 +26,7 @@
 #include <fstream>
 #include <stdexcept>
 #include <system_error>
+#include <thread>
 
 #ifdef _WIN32
 #include <windows.h>
@@ -32,16 +34,21 @@
 #include <unistd.h>
 #endif
 
+#include <gmock/gmock-matchers.h>
+#include <gtest/gtest.h>
+
+#include "config_builder.h"
 #include "dim.h"
-#include "gmock/gmock.h"
 #include "keyring/keyring_manager.h"
 #include "mock_server_testutils.h"
+#include "mysql/harness/string_utils.h"  // split_string
 #include "mysqlrouter/keyring_info.h"
+#include "mysqlrouter/utils.h"  // copy_file
+#include "process_manager.h"
 #include "random_generator.h"
 #include "router_component_test.h"
 #include "script_generator.h"
 #include "tcp_port_pool.h"
-#include "utils.h"
 
 /**
  * @file
@@ -64,11 +71,10 @@ MATCHER_P(FileContentNotEqual, master_key, "") {
   return file_content.str() != master_key;
 }
 
-class MasterKeyReaderWriterTest : public RouterComponentTest {
+class MasterKeyReaderWriterTest : public RouterComponentBootstrapTest {
  protected:
   void SetUp() override {
     RouterComponentTest::SetUp();
-    logging_folder = Path(tmp_dir_.name()).join("log").str();
 
     mysql_harness::DIM &dim = mysql_harness::DIM::instance();
     // RandomGenerator
@@ -87,25 +93,43 @@ class MasterKeyReaderWriterTest : public RouterComponentTest {
     }
   }
 
-  std::string get_metadata_cache_section(unsigned server_port) {
-    return "[metadata_cache:test]\n"
-           "router_id=1\n"
-           "bootstrap_server_addresses=mysql://localhost:" +
-           std::to_string(server_port) +
-           "\n"
-           "user=mysql_router1_user\n"
-           "metadata_cluster=test\n"
-           "ttl=500\n\n";
+  static std::pair<std::string, std::map<std::string, std::string>>
+  metadata_cache_section(uint16_t server_port) {
+    return {"metadata_cache:test",
+            {{"router_id", "1"},
+             {"bootstrap_server_addresses",
+              "mysql://localhost:" + std::to_string(server_port)},
+             {"user", "mysql_router1_user"},
+             {"metadata_cluster", "test"},
+             {"ttl", "500"}}};
   }
 
-  std::string get_metadata_cache_routing_section(const std::string &role,
-                                                 const std::string &strategy,
-                                                 unsigned router_port) {
-    return "[routing:test_default]\n"
-           "bind_port=" +
-           std::to_string(router_port) + "\n" +
-           "destinations=metadata-cache://test/default?role=" + role + "\n" +
-           "protocol=classic\n" + "routing_strategy=" + strategy + "\n";
+  static std::string get_metadata_cache_section(unsigned server_port) {
+    auto section = metadata_cache_section(server_port);
+
+    return mysql_harness::ConfigBuilder::build_section(section.first,
+                                                       section.second);
+  }
+
+  static std::pair<std::string, std::map<std::string, std::string>>
+  metadata_cache_routing_section(const std::string &role,
+                                 const std::string &strategy,
+                                 uint16_t router_port) {
+    return {"routing:test_default",
+            {{"bind_port", std::to_string(router_port)},
+             {"destinations", "metadata-cache://test/default?role=" + role},
+             {"protocol", "classic"},
+             {"routing_strategy", strategy}}};
+  }
+
+  static std::string get_metadata_cache_routing_section(
+      const std::string &role, const std::string &strategy,
+      uint16_t router_port) {
+    auto section = metadata_cache_routing_section(role, strategy, router_port);
+
+    return mysql_harness::ConfigBuilder::build_section(section.first,
+                                                       section.second) +
+           "\n";
   }
 
   KeyringInfo init_keyring() {
@@ -137,7 +161,6 @@ class MasterKeyReaderWriterTest : public RouterComponentTest {
     ScriptGenerator script_generator(ProcessManager::get_origin(),
                                      tmp_dir_.name());
     std::map<std::string, std::string> default_section = get_DEFAULT_defaults();
-    default_section["logging_folder"] = logging_folder;
     default_section["keyring_path"] =
         Path(tmp_dir_.name()).join("keyring").str();
 
@@ -164,7 +187,6 @@ class MasterKeyReaderWriterTest : public RouterComponentTest {
                                      tmp_dir_.name());
 
     auto default_section = get_DEFAULT_defaults();
-    default_section["logging_folder"] = logging_folder;
     default_section["keyring_path"] =
         Path(tmp_dir_.name()).join("keyring").str();
     default_section["master_key_reader"] =
@@ -174,17 +196,8 @@ class MasterKeyReaderWriterTest : public RouterComponentTest {
     return default_section;
   }
 
-  auto &launch_router(const std::vector<std::string> &params,
-                      int expected_exit_code = EXIT_SUCCESS) {
-    return ProcessManager::launch_router(
-        params, expected_exit_code, /*catch_stderr=*/true, /*with_sudo=*/false,
-        /*wait_for_notify_ready=*/-1s);
-  }
-
-  TcpPortPool port_pool_;
   TempDirectory tmp_dir_;
   TempDirectory bootstrap_dir_;
-  std::string logging_folder;
   std::string master_key_;
 };
 
@@ -204,22 +217,16 @@ TEST_F(MasterKeyReaderWriterTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router({
+  auto &router = launch_router_for_bootstrap({
       "--bootstrap=127.0.0.1:" + std::to_string(server_port),
-      "--report-host",
-      "dont.query.dns",
       "--directory=" + bootstrap_dir_.name(),
       "--force",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
   });
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping was successful
-  check_exit_code(router, EXIT_SUCCESS, 30000ms);
+  // check if the bootstrapping was successful
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS, 30000ms));
   EXPECT_TRUE(router.expect_output(
       "MySQL Router configured for the InnoDB Cluster 'mycluster'"))
       << router.get_full_output() << std::endl
@@ -257,22 +264,16 @@ TEST_F(MasterKeyReaderWriterTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router({
+  auto &router = launch_router_for_bootstrap({
       "--directory=" + bootstrap_dir_.name(),
       "--force",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
-      "--report-host",
-      "dont.query.dns",
       "--bootstrap=127.0.0.1:" + std::to_string(server_port),
   });
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping was successful
-  check_exit_code(router, EXIT_SUCCESS, 30000ms);
+  // check if the bootstrapping was successful
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS, 30000ms));
   EXPECT_TRUE(
       router.expect_output("MySQL Router configured for the "
                            "InnoDB Cluster 'mycluster'"))
@@ -307,8 +308,8 @@ TEST_F(MasterKeyReaderWriterTest,
 /**
  * @test
  *       verify that when --master-key-reader option is used, but specified
- * reader cannot be executed, then bootstrap fails and appropriate error message
- * is printed to standard output.
+ * reader cannot be executed, then bootstrap fails and appropriate error
+ * message is printed to standard output.
  */
 TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyReader) {
   auto server_port = port_pool_.get_next_available();
@@ -320,11 +321,9 @@ TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyReader) {
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router(
+  auto &router = launch_router_for_bootstrap(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
-          "--report-host",
-          "dont.query.dns",
           "--directory=" + bootstrap_dir_.name(),
           "--force",
           "--master-key-reader=" + script_generator.get_fake_reader_script(),
@@ -332,12 +331,8 @@ TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyReader) {
       },
       EXIT_FAILURE);
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router, EXIT_FAILURE);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
   EXPECT_TRUE(router.expect_output(
       "Error: Cannot fetch master key file using master key reader"))
       << router.get_full_output() << std::endl
@@ -360,11 +355,9 @@ TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyWriter) {
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router(
+  auto &router = launch_router_for_bootstrap(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
-          "--report-host",
-          "dont.query.dns",
           "--directory=" + bootstrap_dir_.name(),
           "--force",
           "--master-key-reader=" + script_generator.get_reader_script(),
@@ -372,12 +365,8 @@ TEST_F(MasterKeyReaderWriterTest, BootstrapFailsWhenCannotRunMasterKeyWriter) {
       },
       EXIT_FAILURE);
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router, EXIT_FAILURE);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
   EXPECT_TRUE(router.expect_output(
       "Error: Cannot write master key file using master key writer"))
       << router.get_full_output() << std::endl
@@ -405,31 +394,25 @@ TEST_F(MasterKeyReaderWriterTest, KeyringFileRestoredWhenBootstrapFails) {
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router(
+  auto &router = launch_router_for_bootstrap(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
           "--directory=" + bootstrap_dir_.name(),
           "--force",
           "--master-key-reader=" + script_generator.get_fake_reader_script(),
           "--master-key-writer=" + script_generator.get_fake_writer_script(),
-          "--report-host",
-          "dont.query.dns",
       },
       EXIT_FAILURE);
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router, EXIT_FAILURE);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
   ASSERT_THAT(keyring_path.str(), FileContentEqual("keyring file content"));
 }
 
 /**
  * @test
- *       verify that if --master-key-reader option is used and bootstrap fails,
- *       then original master key is restored.
+ *       verify that if --master-key-reader option is used and bootstrap
+ * fails, then original master key is restored.
  */
 TEST_F(MasterKeyReaderWriterTest, MasterKeyRestoredWhenBootstrapFails) {
   // create file that is used by master-key-reader and master-key-writer
@@ -441,7 +424,7 @@ TEST_F(MasterKeyReaderWriterTest, MasterKeyRestoredWhenBootstrapFails) {
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router(
+  auto &router = launch_router_for_bootstrap(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
           "--connect-timeout=1",
@@ -452,12 +435,8 @@ TEST_F(MasterKeyReaderWriterTest, MasterKeyRestoredWhenBootstrapFails) {
       },
       EXIT_FAILURE);
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router, EXIT_FAILURE);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
   ASSERT_THAT(master_key_path.str(), FileContentEqual(""));
 }
 
@@ -480,22 +459,16 @@ TEST_F(MasterKeyReaderWriterTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router({
+  auto &router = launch_router_for_bootstrap({
       "--bootstrap=127.0.0.1:" + std::to_string(server_port),
-      "--report-host",
-      "dont.query.dns",
       "--directory=" + bootstrap_dir_.name(),
       "--force",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
   });
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping was successful
-  check_exit_code(router, EXIT_SUCCESS, 30000ms);
+  // check if the bootstrapping was successful
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS, 30000ms));
   EXPECT_TRUE(
       router.expect_output("MySQL Router configured for the "
                            "InnoDB Cluster 'mycluster'"))
@@ -511,7 +484,7 @@ TEST_F(MasterKeyReaderWriterTest,
 /*
  * @test
  *       verify that if master key exists and is not empty and bootstrap pass,
- * then original master key is not overriden.
+ * then original master key is not overridden.
  */
 TEST_F(MasterKeyReaderWriterTest,
        DontWriteMasterKeyAtBootstrapIfMasterKeyAlreadyExists) {
@@ -526,22 +499,16 @@ TEST_F(MasterKeyReaderWriterTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router({
+  auto &router = launch_router_for_bootstrap({
       "--bootstrap=127.0.0.1:" + std::to_string(server_port),
-      "--report-host",
-      "dont.query.dns",
       "--directory=" + bootstrap_dir_.name(),
       "--force",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
   });
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router));
   ASSERT_THAT(Path(tmp_dir_.name()).join("master_key").str(),
               FileContentEqual("master key value"));
 }
@@ -555,71 +522,89 @@ TEST_F(MasterKeyReaderWriterTest,
 TEST_F(MasterKeyReaderWriterTest, ConnectToMetadataServerPass) {
   const auto server_port = port_pool_.get_next_available();
   const auto router_port = port_pool_.get_next_available();
-  std::string metadata_cache_section = get_metadata_cache_section(server_port);
-  std::string routing_section =
-      get_metadata_cache_routing_section("PRIMARY", "round-robin", router_port);
-
-  init_keyring();
 
   // launch server
   /*auto &server =*/launch_mysql_server_mock(
       get_data_dir().join("metadata_dynamic_nodes.js").str(), server_port);
 
-  auto default_section_map = get_default_section_map();
   // launch the router with metadata-cache configuration
   TempDirectory conf_dir("conf");
-  launch_router({
-      "-c",
-      create_config_file(conf_dir.name(),
-                         metadata_cache_section + routing_section,
-                         &default_section_map),
-  });
 
-  auto matcher = [&](const std::string &line) -> bool {
-    return line.find("Connected with metadata server running on") != line.npos;
-  };
+  auto writer = config_writer(conf_dir.name())
+                    .section(metadata_cache_section(server_port))
+                    .section(metadata_cache_routing_section(
+                        "PRIMARY", "round-robin", router_port));
 
-  EXPECT_TRUE(find_in_file(logging_folder + "/mysqlrouter.log", matcher,
-                           std::chrono::milliseconds(10000)));
+  // set master-key-reader/writer in DEFAULT section
+  auto &default_section = writer.sections().at("DEFAULT");
+
+  const auto keyring_info = init_keyring();
+
+  default_section["keyring_path"] = keyring_info.get_keyring_file();
+  default_section["master_key_reader"] = keyring_info.get_master_key_reader();
+  default_section["master_key_writer"] = keyring_info.get_master_key_writer();
+
+  auto &router =
+      router_spawner()
+          // ERROR: 1273 Syntax Error at: ROLLBACK ...
+          // it will never get READY.
+          .wait_for_sync_point(ProcessManager::Spawner::SyncPoint::RUNNING)
+          .spawn({"-c", writer.write()});
+
+  EXPECT_TRUE(wait_log_contains(
+      router, "Connected with metadata server running on", 10s));
 }
 
 /**
  * @test
  *       verify that when master key returned by master-key-reader is correct
- *       and then launching the router succeeds, then master-key is not written
- *       to log files.
+ *       and then launching the router succeeds, then master-key is not
+ * written to log files.
  *
  */
 TEST_F(MasterKeyReaderWriterTest,
        NoMasterKeyInLogsWhenConnectToMetadataServerPass) {
   const auto server_port = port_pool_.get_next_available();
   const auto router_port = port_pool_.get_next_available();
-  std::string metadata_cache_section = get_metadata_cache_section(server_port);
-  std::string routing_section =
-      get_metadata_cache_routing_section("PRIMARY", "round-robin", router_port);
-
-  init_keyring();
 
   // launch server
   auto &server = launch_mysql_server_mock(
       get_data_dir().join("metadata_dynamic_nodes.js").str(), server_port);
   ASSERT_NO_FATAL_FAILURE(check_port_ready(server, server_port, 10000ms));
 
-  auto default_section_map = get_default_section_map();
-  // launch the router with metadata-cache configuration
   TempDirectory conf_dir("conf");
-  auto &router = launch_router(
-      {"-c", create_config_file(conf_dir.name(),
-                                metadata_cache_section + routing_section,
-                                &default_section_map)});
 
-  auto matcher = [&, this](const std::string &line) -> bool {
-    return line.find(master_key_) != line.npos;
-  };
+  auto writer = config_writer(conf_dir.name())
+                    .section(metadata_cache_section(server_port))
+                    .section(metadata_cache_routing_section(
+                        "PRIMARY", "round-robin", router_port));
 
-  EXPECT_FALSE(find_in_file(logging_folder + "/mysqlrouter.log", matcher,
-                            std::chrono::milliseconds(1000)))
-      << router.get_full_output();
+  // set master-key-reader/writer in DEFAULT section
+  auto &default_section = writer.sections().at("DEFAULT");
+
+  const auto keyring_info = init_keyring();
+
+  default_section["keyring_path"] = keyring_info.get_keyring_file();
+  default_section["master_key_reader"] = keyring_info.get_master_key_reader();
+  default_section["master_key_writer"] = keyring_info.get_master_key_writer();
+
+  // launch the router with metadata-cache configuration
+
+  auto &router =
+      router_spawner()
+          // ERROR: 1273 Syntax Error at: ROLLBACK ...
+          // it will never get READY.
+          .wait_for_sync_point(ProcessManager::Spawner::SyncPoint::RUNNING)
+          .spawn({"-c", writer.write()});
+
+  // wait a bit for log to generate.
+  std::this_thread::sleep_for(1s);
+
+  auto log_lines =
+      mysql_harness::split_string(router.get_logfile_content(), '\n');
+  EXPECT_THAT(
+      log_lines,
+      ::testing::Not(::testing::Contains(::testing::HasSubstr(master_key_))));
 }
 
 /**
@@ -652,9 +637,9 @@ TEST_F(MasterKeyReaderWriterTest, CannotLaunchRouterWhenNoMasterKeyReader) {
                              metadata_cache_section + routing_section,
                              &default_section_map),
       },
-      EXIT_FAILURE);
+      EXIT_FAILURE, true, false, -1s);
 
-  check_exit_code(router, EXIT_FAILURE);
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
 }
 
 /**
@@ -685,14 +670,14 @@ TEST_F(MasterKeyReaderWriterTest, CannotLaunchRouterWhenMasterKeyIncorrect) {
       {"-c", create_config_file(conf_dir.name(),
                                 metadata_cache_section + routing_section,
                                 &incorrect_master_key_default_section_map)},
-      EXIT_FAILURE);
+      EXIT_FAILURE, true, false, -1s);
 
-  check_exit_code(router, EXIT_FAILURE);
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
 }
 /*
- * These tests are executed only for STANDALONE layout and are not executed for
- * Windows. Bootstrap for layouts different than STANDALONE use directories to
- * which tests don't have access (see install_layout.cmake).
+ * These tests are executed only for STANDALONE layout and are not executed
+ * for Windows. Bootstrap for layouts different than STANDALONE use
+ * directories to which tests don't have access (see install_layout.cmake).
  */
 Path g_origin_path;
 #ifndef SKIP_BOOTSTRAP_SYSTEM_DEPLOYMENT_TESTS
@@ -791,20 +776,14 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest, BootstrapPass) {
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router({
+  auto &router = launch_router_for_bootstrap({
       "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
-      "--report-host",
-      "dont.query.dns",
       "--master-key-reader=" + script_generator.get_reader_script(),
       "--master-key-writer=" + script_generator.get_writer_script(),
   });
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping was successful
-  check_exit_code(router, EXIT_SUCCESS);
+  // check if the bootstrapping was successful
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS));
 
   EXPECT_TRUE(
       router.expect_output("MySQL Router configured for the "
@@ -819,9 +798,9 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest, BootstrapPass) {
 
 /**
  * @test
- *       verify if bootstrap with --master-key-reader and with system deployment
- * layout, but specified reader cannot be executed, then bootstrap fails and
- * appropriate error message is printed to standard output.
+ *       verify if bootstrap with --master-key-reader and with system
+ * deployment layout, but specified reader cannot be executed, then bootstrap
+ * fails and appropriate error message is printed to standard output.
  */
 TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
        BootstrapFailsWhenCannotRunMasterKeyReader) {
@@ -832,22 +811,16 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router(
+  auto &router = launch_router_for_bootstrap(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
-          "--report-host",
-          "dont.query.dns",
           "--master-key-reader=" + script_generator.get_fake_reader_script(),
           "--master-key-writer=" + script_generator.get_writer_script(),
       },
       EXIT_FAILURE);
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router, EXIT_FAILURE);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
 
   EXPECT_TRUE(router.expect_output(
       "Error: Cannot fetch master key file using master key reader"))
@@ -870,22 +843,16 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router(
+  auto &router = launch_router_for_bootstrap(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
-          "--report-host",
-          "dont.query.dns",
           "--master-key-reader=" + script_generator.get_reader_script(),
           "--master-key-writer=" + script_generator.get_fake_writer_script(),
       },
       EXIT_FAILURE);
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router, EXIT_FAILURE);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
 
   EXPECT_TRUE(router.expect_output(
       "Error: Cannot write master key file using master key writer"))
@@ -916,31 +883,25 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router(
+  auto &router = launch_router_for_bootstrap(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
           "--connect-timeout=1",
           "--master-key-reader=" + script_generator.get_fake_reader_script(),
           "--master-key-writer=" + script_generator.get_fake_writer_script(),
-          "--report-host",
-          "dont.query.dns",
       },
       EXIT_FAILURE);
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router, EXIT_FAILURE);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
 
   ASSERT_THAT(keyring_path.str(), FileContentEqual("keyring file content"));
 }
 
 /**
  * @test
- *       verify bootstrap with --master-key-reader and system deployment layout
- *       and bootstrap fails, then original master key is restored.
+ *       verify bootstrap with --master-key-reader and system deployment
+ * layout and bootstrap fails, then original master key is restored.
  */
 TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
        MasterKeyRestoredWhenBootstrapFails) {
@@ -953,7 +914,7 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
                                    tmp_dir_.name());
 
   // launch the router in bootstrap mode
-  auto &router = launch_router(
+  auto &router = launch_router_for_bootstrap(
       {
           "--bootstrap=127.0.0.1:" + std::to_string(server_port),
           "--connect-timeout=1",
@@ -962,12 +923,8 @@ TEST_F(MasterKeyReaderWriterSystemDeploymentTest,
       },
       EXIT_FAILURE);
 
-  // add login hook
-  router.register_response("Please enter MySQL password for root: ",
-                           "fake-pass\n");
-
-  // check if the bootstraping failed
-  check_exit_code(router, EXIT_FAILURE);
+  // check if the bootstrapping failed
+  ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_FAILURE));
 
   ASSERT_THAT(master_key_path.str(), FileContentEqual(""));
 }
