@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2019, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,8 +27,6 @@
 #include <thread>
 
 #ifdef RAPIDJSON_NO_SIZETYPEDEFINE
-// if we build within the server, it will set RAPIDJSON_NO_SIZETYPEDEFINE
-// globally and require to include my_rapidjson_size_t.h
 #include "my_rapidjson_size_t.h"
 #endif
 
@@ -40,11 +39,12 @@
 #include "config_builder.h"
 #include "dim.h"
 #include "mysql/harness/utility/string.h"  // ::join
-#include "mysql_session.h"
+#include "mysqlrouter/mysql_session.h"
 #include "rest_api_testutils.h"
 #include "router_component_test.h"
+#include "router_component_testutils.h"
 #include "tcp_port_pool.h"
-#include "temp_dir.h"
+#include "test/temp_directory.h"
 
 #include "mysqlrouter/rest_client.h"
 
@@ -89,7 +89,7 @@ class RestMetadataCacheApiWithoutClusterTest
     : public RestApiTestBase,
       public ::testing::WithParamInterface<RestApiTestParams> {};
 
-// precondition to these tests is that we can start a router agianst a
+// precondition to these tests is that we can start a router against a
 // metadata-cluster which has no nodes. But with Bug#28352482 (no empty
 // bootstrap_server_addresses) fixed we can't bring the metadata into that state
 // anymore. We just won't start.
@@ -112,19 +112,21 @@ TEST_P(RestMetadataCacheApiWithoutClusterTest, DISABLED_ensure_openapi) {
   config_sections.push_back(mysql_harness::ConfigBuilder::build_section(
       "metadata_cache:" + metadata_cache_section_name,
       {
+          {"router_id", "1"},
           {"user", keyring_username},
-          {"ttl", "0.2"},
+          {"ttl", "0.1"},
       }));
 
   std::string conf_file{create_config_file(
-      conf_dir_.name(), mysql_harness::join(config_sections, "\n"),
+      conf_dir_.name(), mysql_harness::join(config_sections, ""),
       &default_section_)};
   ProcessWrapper &http_server{launch_router({"-c", conf_file})};
 
   g_refresh_failed = 0;
   g_time_last_refresh_failed = "";
 
-  fetch_and_validate_schema_and_resource(GetParam(), http_server);
+  ASSERT_NO_FATAL_FAILURE(
+      fetch_and_validate_schema_and_resource(GetParam(), http_server));
 
   // this part is relevant only for Get OK, otherwise let's avoid useless sleep
   if (GetParam().status_code == HttpMethod::Get &&
@@ -134,7 +136,8 @@ TEST_P(RestMetadataCacheApiWithoutClusterTest, DISABLED_ensure_openapi) {
 
     // check the resources again, we want to compare them against the previous
     // ones
-    fetch_and_validate_schema_and_resource(GetParam(), http_server);
+    ASSERT_NO_FATAL_FAILURE(
+        fetch_and_validate_schema_and_resource(GetParam(), http_server));
   }
 }
 
@@ -271,7 +274,7 @@ static const RestApiTestParams rest_api_params_without_cluster[]{
           [](const JsonValue *value) -> void {
             ASSERT_NE(value, nullptr);
             ASSERT_TRUE(value->IsInt());
-            ASSERT_EQ(value->GetInt(), 200);
+            ASSERT_EQ(value->GetInt(), 100);
           }},
      },
      kMetadataSwaggerPaths},
@@ -292,6 +295,7 @@ class RestMetadataCacheApiTest
       public ::testing::WithParamInterface<RestApiTestParams> {
  protected:
   const uint16_t metadata_server_port_{port_pool_.get_next_available()};
+  const uint16_t metadata_server_http_port_{port_pool_.get_next_available()};
 };
 
 /**
@@ -360,7 +364,7 @@ TEST_P(RestMetadataCacheApiTest, ensure_openapi) {
 
   /*auto &md_server =*/ProcessManager::launch_mysql_server_mock(
       get_data_dir().join("metadata_1_node_repeat.js").str(),
-      metadata_server_port_, EXIT_SUCCESS, false);
+      metadata_server_port_, EXIT_SUCCESS, false, metadata_server_http_port_);
 
   const std::string userfile = create_password_file();
 
@@ -375,16 +379,20 @@ TEST_P(RestMetadataCacheApiTest, ensure_openapi) {
   config_sections.push_back(mysql_harness::ConfigBuilder::build_section(
       "metadata_cache:" + metadata_cache_section_name,
       {
+          {"router_id", "1"},
           {"user", keyring_username},
           // name of the cluster in the mock's metadata
           {"metadata_cluster", "test"},
-          {"ttl", "0.2"},
-          {"bootstrap_server_addresses",
-           "mysql://127.0.0.1:" + std::to_string(metadata_server_port_)},
+          {"ttl", "0.1"},
       }));
 
+  const std::string state_file = create_state_file(
+      get_test_temp_dir_name(),
+      create_state_file_content("uuid", "", {metadata_server_port_}, 0));
+  default_section_["dynamic_state"] = state_file;
+
   std::string conf_file{create_config_file(
-      conf_dir_.name(), mysql_harness::join(config_sections, "\n"),
+      conf_dir_.name(), mysql_harness::join(config_sections, ""),
       &default_section_)};
 
   auto &router_proc{launch_router({"-c", conf_file})};
@@ -402,18 +410,19 @@ TEST_P(RestMetadataCacheApiTest, ensure_openapi) {
                                   metadata_cache_section_name + "/status"));
   }
 
-  EXPECT_NO_FATAL_FAILURE(
+  ASSERT_NO_FATAL_FAILURE(
       fetch_and_validate_schema_and_resource(GetParam(), router_proc));
 
   // this part is relevant only for Get OK, otherwise let's avoid useless sleep
   if (GetParam().methods == HttpMethod::Get &&
       GetParam().status_code == HttpStatusCode::Ok) {
-    // sleep ~2*TTL to make the counters and timestamps change
-    std::this_thread::sleep_for(500ms);
+    // wait a few metadata refresh cycles for the counters and timestamps change
+    ASSERT_TRUE(
+        wait_for_transaction_count_increase(metadata_server_http_port_, 4));
 
     // check the resources again, we want to compare them against the previous
     // ones
-    EXPECT_NO_FATAL_FAILURE(
+    ASSERT_NO_FATAL_FAILURE(
         fetch_and_validate_schema_and_resource(GetParam(), router_proc));
   }
 }
@@ -558,19 +567,19 @@ static const RestApiTestParams rest_api_valid_methods[]{
           [](const JsonValue *value) -> void {
             ASSERT_NE(value, nullptr);
             ASSERT_TRUE(value->IsString());
-            ASSERT_STREQ(value->GetString(), "test");
+            ASSERT_STREQ(value->GetString(), "");
           }},
          {"/groupReplicationId",
           [](const JsonValue *value) -> void {
             ASSERT_NE(value, nullptr);
             ASSERT_TRUE(value->IsString());
-            ASSERT_STREQ(value->GetString(), "");
+            ASSERT_STREQ(value->GetString(), "uuid");
           }},
          {"/timeRefreshInMs",
           [](const JsonValue *value) -> void {
             ASSERT_NE(value, nullptr);
             ASSERT_TRUE(value->IsUint64());
-            ASSERT_EQ(value->GetUint64(), 200);
+            ASSERT_EQ(value->GetUint64(), 100);
           }},
          {"/nodes",
           [](const JsonValue *value) -> void {
@@ -795,36 +804,36 @@ static const RestApiTestParams rest_api_invalid_methods_params[]{
     {"metadata_list_invalid_methods",
      std::string(rest_api_basepath) + "/metadata/", "/metadata",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
-         HttpMethod::Connect,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::kProblemJsonMethodNotAllowed, kMetadataSwaggerPaths},
+     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
+     kMetadataSwaggerPaths},
 
     {"metadata_status_invalid_methods",
      std::string(rest_api_basepath) + "/metadata/" +
          metadata_cache_section_name + "/status",
      "/metadata/{metadataName}/status",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
-         HttpMethod::Connect,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::kProblemJsonMethodNotAllowed, kMetadataSwaggerPaths},
+     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
+     kMetadataSwaggerPaths},
 
     {"metadata_config_invalid_methods",
      std::string(rest_api_basepath) + "/metadata/" +
          metadata_cache_section_name + "/config",
      "/metadata/{metadataName}/config",
      HttpMethod::Post | HttpMethod::Delete | HttpMethod::Patch |
-         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options |
-         HttpMethod::Connect,
+         HttpMethod::Head | HttpMethod::Trace | HttpMethod::Options,
      HttpStatusCode::MethodNotAllowed, kContentTypeJsonProblem,
      kRestApiUsername, kRestApiPassword,
      /*request_authentication =*/true,
-     RestApiComponentTest::kProblemJsonMethodNotAllowed, kMetadataSwaggerPaths},
+     RestApiComponentTest::get_json_method_not_allowed_verifiers(),
+     kMetadataSwaggerPaths},
 };
 
 INSTANTIATE_TEST_SUITE_P(
@@ -855,10 +864,10 @@ TEST_F(RestMetadataCacheApiTest, metadata_cache_api_no_auth) {
 
   check_exit_code(router, EXIT_FAILURE, 10s);
 
-  const std::string router_output = router.get_full_logfile();
+  const std::string router_output = router.get_logfile_content();
   EXPECT_THAT(router_output,
               ::testing::HasSubstr(
-                  "plugin 'rest_metadata_cache' init failed: option "
+                  "  init 'rest_metadata_cache' failed: option "
                   "require_realm in [rest_metadata_cache] is required"))
       << router_output;
 }
@@ -880,13 +889,12 @@ TEST_F(RestMetadataCacheApiTest, invalid_realm) {
 
   check_exit_code(router, EXIT_FAILURE, 10s);
 
-  const std::string router_output = router.get_full_logfile();
+  const std::string router_output = router.get_logfile_content();
   EXPECT_THAT(
       router_output,
       ::testing::HasSubstr(
           "Configuration error: The option 'require_realm=invalidrealm' "
-          "in [rest_metadata_cache] does not match any http_auth_realm."))
-      << router_output;
+          "in [rest_metadata_cache] does not match any http_auth_realm."));
 }
 
 /**
@@ -946,7 +954,7 @@ TEST_F(RestMetadataCacheApiTest, rest_metadata_cache_section_twice) {
       mysql_harness::ConfigBuilder::build_section("rest_metadata_cache", {}));
 
   const std::string conf_file{create_config_file(
-      conf_dir_.name(), mysql_harness::join(config_sections, "\n"))};
+      conf_dir_.name(), mysql_harness::join(config_sections, ""))};
   auto &router =
       launch_router({"-c", conf_file}, EXIT_FAILURE, true, false, -1s);
 
@@ -978,13 +986,11 @@ TEST_F(RestMetadataCacheApiTest, rest_metadata_cache_section_has_key) {
 
   check_exit_code(router, EXIT_FAILURE, 10s);
 
-  const std::string router_output = router.get_full_logfile();
-  EXPECT_THAT(
-      router_output,
-      ::testing::HasSubstr(
-          "plugin 'rest_metadata_cache' init failed: [rest_metadata_cache] "
-          "section does not expect a key, found 'A'"))
-      << router_output;
+  const std::string router_output = router.get_logfile_content();
+  EXPECT_THAT(router_output,
+              ::testing::HasSubstr(
+                  "  init 'rest_metadata_cache' failed: [rest_metadata_cache] "
+                  "section does not expect a key, found 'A'"));
 }
 
 int main(int argc, char *argv[]) {
