@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,9 +40,12 @@
 
 #include "my_hostname.h"  // HOSTNAME_LENGTH
 #include "my_inttypes.h"
-#include "mysql_com.h"  // USERNAME_LENGTH
+#include "mysql_com.h"           // USERNAME_LENGTH
+#include "sql/auth/auth_acls.h"  // Access_bitmask
 #include "template_utils.h"
 
+#include <openssl/rsa.h>
+
 /* Forward Declarations */
 class Alter_info;
 class Field_iterator_table_ref;
@@ -61,7 +65,7 @@ class Security_context;
 class ACL_USER;
 struct TABLE;
 struct MEM_ROOT;
-struct TABLE_LIST;
+class Table_ref;
 enum class role_enum;
 enum class Consumer_type;
 class LEX_GRANT_AS;
@@ -69,6 +73,7 @@ class LEX_GRANT_AS;
 namespace consts {
 extern const std::string mysql;
 extern const std::string system_user;
+extern const std::string connection_admin;
 }  // namespace consts
 
 /** user, host tuple which reference either acl_cache or g_default_roles */
@@ -101,9 +106,9 @@ enum ACL_internal_access_result {
 */
 class ACL_internal_table_access {
  public:
-  ACL_internal_table_access() {}
+  ACL_internal_table_access() = default;
 
-  virtual ~ACL_internal_table_access() {}
+  virtual ~ACL_internal_table_access() = default;
 
   /**
     Check access to an internal table.
@@ -111,6 +116,8 @@ class ACL_internal_table_access {
     to save_priv.
     @param want_access the privileges requested
     @param [in, out] save_priv the privileges granted
+    @param any_combination_will_do true if it's enough to have any privilege
+    for any combination of the table columns.
     @retval ACL_INTERNAL_ACCESS_GRANTED All the requested privileges
       are granted, and saved in save_priv.
     @retval ACL_INTERNAL_ACCESS_DENIED At least one of the requested
@@ -120,8 +127,9 @@ class ACL_internal_table_access {
       privilege. Requested privileges that are granted, if any, are saved
       in save_priv.
   */
-  virtual ACL_internal_access_result check(ulong want_access,
-                                           ulong *save_priv) const = 0;
+  virtual ACL_internal_access_result check(
+      Access_bitmask want_access, Access_bitmask *save_priv,
+      bool any_combination_will_do) const = 0;
 };
 
 /**
@@ -136,14 +144,16 @@ class ACL_internal_table_access {
 */
 class ACL_internal_schema_access {
  public:
-  ACL_internal_schema_access() {}
+  ACL_internal_schema_access() = default;
 
-  virtual ~ACL_internal_schema_access() {}
+  virtual ~ACL_internal_schema_access() = default;
 
   /**
     Check access to an internal schema.
     @param want_access the privileges requested
     @param [in, out] save_priv the privileges granted
+    @param any_combination_will_do true if it's enough to have any privilege
+    for any combination of the table columns.
     @retval ACL_INTERNAL_ACCESS_GRANTED All the requested privileges
       are granted, and saved in save_priv.
     @retval ACL_INTERNAL_ACCESS_DENIED At least one of the requested
@@ -153,8 +163,9 @@ class ACL_internal_schema_access {
       privilege. Requested privileges that are granted, if any, are saved
       in save_priv.
   */
-  virtual ACL_internal_access_result check(ulong want_access,
-                                           ulong *save_priv) const = 0;
+  virtual ACL_internal_access_result check(
+      Access_bitmask want_access, Access_bitmask *save_priv,
+      bool any_combination_will_do) const = 0;
 
   /**
     Search for per table ACL access rules by table name.
@@ -181,12 +192,13 @@ class ACL_internal_schema_registry {
 */
 class IS_internal_schema_access : public ACL_internal_schema_access {
  public:
-  IS_internal_schema_access() {}
+  IS_internal_schema_access() = default;
 
-  ~IS_internal_schema_access() override {}
+  ~IS_internal_schema_access() override = default;
 
-  ACL_internal_access_result check(ulong want_access,
-                                   ulong *save_priv) const override;
+  ACL_internal_access_result check(Access_bitmask want_access,
+                                   Access_bitmask *save_priv,
+                                   bool any_combination_will_do) const override;
 
   const ACL_internal_table_access *lookup(const char *name) const override;
 };
@@ -418,7 +430,7 @@ class User_table_schema {
   // Added in 8.0.14
   virtual uint user_attributes_idx() = 0;
 
-  virtual ~User_table_schema() {}
+  virtual ~User_table_schema() = default;
 };
 
 /*
@@ -684,7 +696,7 @@ class User_table_schema_factory {
   }
 
   virtual bool is_old_user_table_schema(TABLE *table);
-  virtual ~User_table_schema_factory() {}
+  virtual ~User_table_schema_factory() = default;
 };
 
 extern bool mysql_user_table_is_in_short_password_format;
@@ -709,7 +721,7 @@ bool acl_check_host(THD *thd, const char *host, const char *ip);
 
 /*
   User Attributes are the once which are defined during CREATE/ALTER/GRANT
-  statement. These attributes are divided into following catagories.
+  statement. These attributes are divided into following categories.
 */
 
 #define NONE_ATTR 0L
@@ -753,15 +765,17 @@ bool check_engine_type_for_acl_table(THD *thd, bool mdl_locked);
 bool grant_init(bool skip_grant_tables);
 void grant_free(void);
 bool reload_acl_caches(THD *thd, bool mdl_locked);
-ulong acl_get(THD *thd, const char *host, const char *ip, const char *user,
-              const char *db, bool db_is_pattern);
+Access_bitmask acl_get(THD *thd, const char *host, const char *ip,
+                       const char *user, const char *db, bool db_is_pattern);
 bool is_acl_user(THD *thd, const char *host, const char *user);
 bool acl_getroot(THD *thd, Security_context *sctx, const char *user,
                  const char *host, const char *ip, const char *db);
 bool check_acl_tables_intact(THD *thd, bool mdl_locked);
-bool check_acl_tables_intact(THD *thd, TABLE_LIST *tables);
+bool check_acl_tables_intact(THD *thd, Table_ref *tables);
 void notify_flush_event(THD *thd);
 bool wildcard_db_grant_exists();
+void append_auth_id_string(const THD *thd, const char *user, size_t user_len,
+                           const char *host, size_t host_len, String *str);
 
 /* sql_authorization */
 bool skip_grant_tables();
@@ -769,35 +783,38 @@ bool mysql_set_active_role_none(THD *thd);
 bool mysql_set_role_default(THD *thd);
 bool mysql_set_active_role_all(THD *thd, const List<LEX_USER> *except_users);
 bool mysql_set_active_role(THD *thd, const List<LEX_USER> *role_list);
-bool mysql_grant(THD *thd, const char *db, List<LEX_USER> &list, ulong rights,
-                 bool revoke_grant, bool is_proxy,
+bool mysql_grant(THD *thd, const char *db, List<LEX_USER> &list,
+                 Access_bitmask rights, bool revoke_grant, bool is_proxy,
                  const List<LEX_CSTRING> &dynamic_privilege,
                  bool grant_all_current_privileges, LEX_GRANT_AS *grant_as);
-bool mysql_routine_grant(THD *thd, TABLE_LIST *table, bool is_proc,
-                         List<LEX_USER> &user_list, ulong rights, bool revoke,
-                         bool write_to_binlog);
-int mysql_table_grant(THD *thd, TABLE_LIST *table, List<LEX_USER> &user_list,
-                      List<LEX_COLUMN> &column_list, ulong rights, bool revoke);
-bool check_grant(THD *thd, ulong want_access, TABLE_LIST *tables,
+bool mysql_routine_grant(THD *thd, Table_ref *table, bool is_proc,
+                         List<LEX_USER> &user_list, Access_bitmask rights,
+                         bool revoke, bool write_to_binlog);
+int mysql_table_grant(THD *thd, Table_ref *table, List<LEX_USER> &user_list,
+                      List<LEX_COLUMN> &column_list,
+                      Access_bitmask Access_bitmask, bool revoke);
+bool check_grant(THD *thd, Access_bitmask want_access, Table_ref *tables,
                  bool any_combination_will_do, uint number, bool no_errors);
 bool check_grant_column(THD *thd, GRANT_INFO *grant, const char *db_name,
                         const char *table_name, const char *name, size_t length,
-                        Security_context *sctx, ulong want_privilege);
-bool check_column_grant_in_table_ref(THD *thd, TABLE_LIST *table_ref,
+                        Security_context *sctx, Access_bitmask want_privilege);
+bool check_column_grant_in_table_ref(THD *thd, Table_ref *table_ref,
                                      const char *name, size_t length,
-                                     ulong want_privilege);
-bool check_grant_all_columns(THD *thd, ulong want_access,
+                                     Access_bitmask want_privilege);
+bool check_grant_all_columns(THD *thd, Access_bitmask want_access,
                              Field_iterator_table_ref *fields);
-bool check_grant_routine(THD *thd, ulong want_access, TABLE_LIST *procs,
+bool check_grant_routine(THD *thd, Access_bitmask want_access, Table_ref *procs,
                          bool is_proc, bool no_error);
-bool check_grant_db(THD *thd, const char *db);
+bool check_grant_db(THD *thd, const char *db,
+                    const bool check_table_grant = false);
 bool acl_check_proxy_grant_access(THD *thd, const char *host, const char *user,
                                   bool with_grant);
-void get_privilege_desc(char *to, uint max_length, ulong access);
+void get_privilege_desc(char *to, uint max_length, Access_bitmask access);
 void get_mqh(THD *thd, const char *user, const char *host, USER_CONN *uc);
-ulong get_table_grant(THD *thd, TABLE_LIST *table);
-ulong get_column_grant(THD *thd, GRANT_INFO *grant, const char *db_name,
-                       const char *table_name, const char *field_name);
+Access_bitmask get_table_grant(THD *thd, Table_ref *table);
+Access_bitmask get_column_grant(THD *thd, GRANT_INFO *grant,
+                                const char *db_name, const char *table_name,
+                                const char *field_name);
 bool mysql_show_grants(THD *, LEX_USER *, const List_of_auth_id_refs &, bool,
                        bool);
 bool mysql_show_create_user(THD *thd, LEX_USER *user, bool are_both_users_same);
@@ -808,40 +825,43 @@ bool sp_grant_privileges(THD *thd, const char *sp_db, const char *sp_name,
                          bool is_proc);
 void fill_effective_table_privileges(THD *thd, GRANT_INFO *grant,
                                      const char *db, const char *table);
-int fill_schema_user_privileges(THD *thd, TABLE_LIST *tables, Item *cond);
-int fill_schema_schema_privileges(THD *thd, TABLE_LIST *tables, Item *cond);
-int fill_schema_table_privileges(THD *thd, TABLE_LIST *tables, Item *cond);
-int fill_schema_column_privileges(THD *thd, TABLE_LIST *tables, Item *cond);
+int fill_schema_user_privileges(THD *thd, Table_ref *tables, Item *cond);
+int fill_schema_schema_privileges(THD *thd, Table_ref *tables, Item *cond);
+int fill_schema_table_privileges(THD *thd, Table_ref *tables, Item *cond);
+int fill_schema_column_privileges(THD *thd, Table_ref *tables, Item *cond);
 const ACL_internal_schema_access *get_cached_schema_access(
     GRANT_INTERNAL_INFO *grant_internal_info, const char *schema_name);
 
-bool lock_tables_precheck(THD *thd, TABLE_LIST *tables);
-bool create_table_precheck(THD *thd, TABLE_LIST *tables,
-                           TABLE_LIST *create_table);
+bool lock_tables_precheck(THD *thd, Table_ref *tables);
+bool create_table_precheck(THD *thd, Table_ref *tables,
+                           Table_ref *create_table);
 bool check_fk_parent_table_access(THD *thd, HA_CREATE_INFO *create_info,
                                   Alter_info *alter_info);
-bool check_lock_view_underlying_table_access(THD *thd, TABLE_LIST *tbl,
+bool check_lock_view_underlying_table_access(THD *thd, Table_ref *tbl,
                                              bool *fake_lock_tables_acl);
 bool check_readonly(THD *thd, bool err_if_readonly);
 void err_readonly(THD *thd);
 
 bool is_secure_transport(int vio_type);
 
-bool check_one_table_access(THD *thd, ulong privilege, TABLE_LIST *tables);
-bool check_single_table_access(THD *thd, ulong privilege, TABLE_LIST *tables,
-                               bool no_errors);
-bool check_routine_access(THD *thd, ulong want_access, const char *db,
+bool check_one_table_access(THD *thd, Access_bitmask privilege,
+                            Table_ref *tables);
+bool check_single_table_access(THD *thd, Access_bitmask privilege,
+                               Table_ref *tables, bool no_errors);
+bool check_routine_access(THD *thd, Access_bitmask want_access, const char *db,
                           char *name, bool is_proc, bool no_errors);
-bool check_some_access(THD *thd, ulong want_access, TABLE_LIST *table);
+bool check_some_access(THD *thd, Access_bitmask want_access, Table_ref *table);
 bool has_full_view_routine_access(THD *thd, const char *db,
                                   const char *definer_user,
                                   const char *definer_host);
 bool has_partial_view_routine_access(THD *thd, const char *db,
                                      const char *routine_name, bool is_proc);
-bool check_access(THD *thd, ulong want_access, const char *db, ulong *save_priv,
+bool check_access(THD *thd, Access_bitmask want_access, const char *db,
+                  Access_bitmask *save_priv,
                   GRANT_INTERNAL_INFO *grant_internal_info,
                   bool dont_check_global_grants, bool no_errors);
-bool check_table_access(THD *thd, ulong requirements, TABLE_LIST *tables,
+bool check_table_access(THD *thd, Access_bitmask requirements,
+                        Table_ref *tables,
                         bool any_combination_of_privileges_will_do, uint number,
                         bool no_errors);
 bool check_table_encryption_admin_access(THD *thd);
@@ -851,7 +871,8 @@ bool mysql_revoke_role(THD *thd, const List<LEX_USER> *users,
                        const List<LEX_USER> *roles);
 void get_default_roles(const Auth_id_ref &user, List_of_auth_id_refs &list);
 
-bool is_granted_table_access(THD *thd, ulong required_acl, TABLE_LIST *table);
+bool is_granted_table_access(THD *thd, Access_bitmask required_acl,
+                             Table_ref *table);
 
 bool mysql_alter_or_clear_default_roles(THD *thd, role_enum role_type,
                                         const List<LEX_USER> *users,
@@ -869,7 +890,7 @@ bool is_granted_role(LEX_CSTRING user, LEX_CSTRING host, LEX_CSTRING role,
                      LEX_CSTRING role_host);
 bool is_mandatory_role(LEX_CSTRING role, LEX_CSTRING role_host,
                        bool *is_mandatory);
-bool check_global_access(THD *thd, ulong want_access);
+bool check_global_access(THD *thd, Access_bitmask want_access);
 
 /* sql_user_table */
 void commit_and_close_mysql_tables(THD *thd);
@@ -997,7 +1018,7 @@ class Default_local_authid : public Create_authid<Default_local_authid> {
 };
 
 /**
-  Grant the privilege temporarily to the in-memory global privleges map.
+  Grant the privilege temporarily to the in-memory global privileges map.
   This class is not thread safe.
  */
 class Grant_temporary_dynamic_privileges
@@ -1026,7 +1047,7 @@ class Drop_temporary_dynamic_privileges {
 class Grant_temporary_static_privileges
     : public Grant_privileges<Grant_temporary_static_privileges> {
  public:
-  Grant_temporary_static_privileges(const THD *thd, const ulong privs);
+  Grant_temporary_static_privileges(const THD *thd, const Access_bitmask privs);
   bool precheck(Security_context *sctx);
   bool grant_privileges(Security_context *sctx);
 
@@ -1035,12 +1056,14 @@ class Grant_temporary_static_privileges
   const THD *m_thd;
 
   /** Privileges */
-  const ulong m_privs;
+  const Access_bitmask m_privs;
 };
 
 bool operator==(const LEX_CSTRING &a, const LEX_CSTRING &b);
 bool is_partial_revoke_exists(THD *thd);
 void set_system_user_flag(THD *thd, bool check_for_main_security_ctx = false);
+void set_connection_admin_flag(THD *thd,
+                               bool check_for_main_security_ctx = false);
 
 /**
   Storage container for default auth ids. Default roles are only weakly
@@ -1097,11 +1120,32 @@ using Role_id = Auth_id;
 static constexpr int USER_HOST_BUFF_SIZE =
     HOSTNAME_LENGTH + USERNAME_LENGTH + 2;
 
+struct random_password_info {
+  std::string user;
+  std::string host;
+  std::string password;
+  unsigned int authentication_factor;
+};
+
 void generate_random_password(std::string *password, uint32_t);
-typedef std::list<std::vector<std::string>> Userhostpassword_list;
+typedef std::list<random_password_info> Userhostpassword_list;
 bool send_password_result_set(THD *thd,
                               const Userhostpassword_list &generated_passwords);
 bool lock_and_get_mandatory_roles(std::vector<Role_id> *mandatory_roles);
 bool mysql_alter_user_comment(THD *thd, const List<LEX_USER> *users,
                               const std::string &json_blob, bool expect_text);
+
+/* helper method to check if sandbox mode should be turned off or not */
+bool turn_off_sandbox_mode(THD *thd, LEX_USER *user);
+
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+bool decrypt_RSA_private_key(uchar *pkt, int cipher_length,
+                             unsigned char *plain_text, size_t plain_text_len,
+                             EVP_PKEY *private_key);
+#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+bool decrypt_RSA_private_key(uchar *pkt, int cipher_length,
+                             unsigned char *plain_text, size_t plain_text_len,
+                             RSA *private_key);
+#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+
 #endif /* AUTH_COMMON_INCLUDED */
