@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -62,7 +63,7 @@
 #include "sql/sql_view.h"  // mysql_register_view
 #include "sql/strfunc.h"
 #include "sql/system_variables.h"
-#include "sql/table.h"  // TABLE_LIST
+#include "sql/table.h"  // Table_ref
 #include "sql/thd_raii.h"
 #include "sql/transaction.h"
 #include "thr_lock.h"
@@ -106,7 +107,7 @@ class View_metadata_updater_context {
     m_thd->set_open_tables_state(&m_open_tables_state_backup);
 
     // Restore lex.
-    m_thd->lex->cleanup(m_thd, true);
+    m_thd->lex->cleanup(true);
     m_thd->lex->destroy();
     lex_end(m_thd->lex);
     delete static_cast<st_lex_local *>(m_thd->lex);
@@ -205,14 +206,14 @@ class View_metadata_updater_error_handler final
 };
 
 Uncommitted_tables_guard::~Uncommitted_tables_guard() {
-  for (const TABLE_LIST *table : m_uncommitted_tables) {
+  for (const Table_ref *table : m_uncommitted_tables) {
     tdc_remove_table(m_thd, TDC_RT_REMOVE_ALL, table->get_db_name(),
                      table->get_table_name(), false);
   }
 }
 
 /**
-  Prepare TABLE_LIST object for views referencing Base Table/ View/ Stored
+  Prepare Table_ref object for views referencing Base Table/ View/ Stored
   routine "db.tbl_or_sf_name".
 
   @tparam     T               Type of object (View_table/View_routine) to fetch
@@ -220,7 +221,13 @@ Uncommitted_tables_guard::~Uncommitted_tables_guard() {
   @param      thd             Current thread.
   @param      db              Database name.
   @param      tbl_or_sf_name  Base table/ View/ Stored function name.
-  @param[out] views           TABLE_LIST objects for views.
+  @param      skip_same_db    Indicates whether it is OK to skip
+                              views belonging to the same database
+                              as table (as they will be dropped anyway).
+  @param      mem_root        Memory root for allocation of temporary
+                              objects which will be cleared after
+                              processing this table/view/routine.
+  @param[out] views           Table_ref objects for views.
 
   @retval     false           Success.
   @retval     true            Failure.
@@ -230,7 +237,8 @@ Uncommitted_tables_guard::~Uncommitted_tables_guard() {
 template <typename T>
 static bool prepare_view_tables_list(THD *thd, const char *db,
                                      const char *tbl_or_sf_name,
-                                     std::vector<TABLE_LIST *> *views) {
+                                     bool skip_same_db, MEM_ROOT *mem_root,
+                                     std::vector<Table_ref *> *views) {
   DBUG_TRACE;
   std::vector<dd::Object_id> view_ids;
   std::set<dd::Object_id> prepared_view_ids;
@@ -246,7 +254,7 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
     dd::String_type schema_name;
     // Get schema name and view name from the object id of the view.
     {
-      dd::View *view = nullptr;
+      std::unique_ptr<dd::View> view;
       // We need to use READ_UNCOMMITTED here as the view could be changed
       // by the same statement (e.g. RENAME TABLE).
       if (thd->dd_client()->acquire_uncached_uncommitted(view_ids.at(idx),
@@ -254,7 +262,7 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
         return true;
       if (!view) continue;
 
-      dd::Schema *schema = nullptr;
+      std::unique_ptr<dd::Schema> schema;
       if (thd->dd_client()->acquire_uncached_uncommitted(view->schema_id(),
                                                          &schema))
         return true;
@@ -263,16 +271,31 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
       schema_name = schema->name();
     }
 
-    // If TABLE_LIST object is already prepared for view name then skip it.
-    if (prepared_view_ids.find(view_ids.at(idx)) == prepared_view_ids.end()) {
-      // Prepare TABLE_LIST object for the view and push_back
+    if (skip_same_db) {
+      /*
+        DROP DATABASE acquires exclusive metadata lock on database being
+        dropped. So we have guarantee that dependent view belonging to
+        the same database won't be moved into other database using
+        RENAME TABLE or doing DROP/CREATE. Hence it is safe to skip
+        view belonging to the database being dropped even if we don't
+        have any lock on it yet.
+      */
+      assert(thd->mdl_context.owns_equal_or_stronger_lock(MDL_key::SCHEMA, db,
+                                                          "", MDL_EXCLUSIVE));
+      if (my_strcasecmp(table_alias_charset, db, schema_name.c_str()) == 0)
+        continue;
+    }
 
-      char *db_name = strmake_root(thd->mem_root, schema_name.c_str(),
-                                   schema_name.length());
+    // If Table_ref object is already prepared for view name then skip it.
+    if (prepared_view_ids.find(view_ids.at(idx)) == prepared_view_ids.end()) {
+      // Prepare Table_ref object for the view and push_back
+
+      char *db_name =
+          strmake_root(mem_root, schema_name.c_str(), schema_name.length());
       if (db_name == nullptr) return true;
 
       char *vw_name =
-          strmake_root(thd->mem_root, view_name.c_str(), view_name.length());
+          strmake_root(mem_root, view_name.c_str(), view_name.length());
       if (vw_name == nullptr) return true;
 
       /*
@@ -288,9 +311,9 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
           my_casedn_str(system_charset_info, vw_name);
       }
 
-      auto vw = new (thd->mem_root)
-          TABLE_LIST(db_name, schema_name.length(), vw_name, view_name.length(),
-                     TL_IGNORE, MDL_EXCLUSIVE);
+      auto vw = new (mem_root)
+          Table_ref(db_name, schema_name.length(), vw_name, view_name.length(),
+                    TL_IGNORE, MDL_EXCLUSIVE);
       if (vw == nullptr) return true;
 
       views->push_back(vw);
@@ -318,10 +341,17 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
   @param          thd                 Current thread.
   @param          db                  Database name.
   @param          tbl_or_sf_name      Base table/ View/ Stored function name.
-  @param          views_list          TABLE_LIST objects of the referencing
+  @param          views_list          Table_ref objects of the referencing
                                       views.
+  @param          skip_same_db        Indicates whether it is OK to skip
+                                      views belonging to the same database
+                                      as table (as they will be dropped
+                                      anyway).
   @param          commit_dd_changes   Indicates whether changes to DD need
                                       to be committed.
+  @param          mem_root            Memory root for allocation of temporary
+                                      objects which will be cleared after
+                                      processing referenced table/view/routine.
 
   @retval     false           Success.
   @retval     true            Failure.
@@ -331,15 +361,16 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
 template <typename T>
 static bool mark_all_views_invalid(THD *thd, const char *db,
                                    const char *tbl_or_sf_name,
-                                   std::vector<TABLE_LIST *> *views_list,
-                                   bool commit_dd_changes) {
+                                   const std::vector<Table_ref *> *views_list,
+                                   bool skip_same_db, bool commit_dd_changes,
+                                   MEM_ROOT *mem_root) {
   DBUG_TRACE;
   assert(!views_list->empty());
 
   // Acquire lock on all the views.
   MDL_request_list mdl_requests;
   for (auto view : *views_list) {
-    MDL_request *schema_request = new (thd->mem_root) MDL_request;
+    MDL_request *schema_request = new (mem_root) MDL_request;
     ;
     MDL_REQUEST_INIT(schema_request, MDL_key::SCHEMA, view->db, "",
                      MDL_INTENTION_EXCLUSIVE, MDL_STATEMENT);
@@ -361,8 +392,9 @@ static bool mark_all_views_invalid(THD *thd, const char *db,
     considered for state update.
     Hence preparing updated list of view tables after acquiring the lock.
   */
-  std::vector<TABLE_LIST *> updated_views_list;
-  if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, &updated_views_list))
+  std::vector<Table_ref *> updated_views_list;
+  if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, skip_same_db,
+                                  mem_root, &updated_views_list))
     return true;
   if (updated_views_list.empty()) return false;
 
@@ -392,11 +424,11 @@ static bool mark_all_views_invalid(THD *thd, const char *db,
   Helper method to
     - Mark view as invalid if DDL operation leaves the view in
       invalid state.
-    - Open all the views from the TABLE_LIST vector and
+    - Open all the views from the Table_ref vector and
       recreates the view metadata.
 
   @param          thd                 Thread handle.
-  @param          views               TABLE_LIST objects of the views.
+  @param          views               Table_ref objects of the views.
   @param          commit_dd_changes   Indicates whether changes to DD need
                                       to be committed.
   @param[in,out]  uncommitted_tables  Helper class to store list of views
@@ -410,7 +442,7 @@ static bool mark_all_views_invalid(THD *thd, const char *db,
 */
 
 static bool open_views_and_update_metadata(
-    THD *thd, const std::vector<TABLE_LIST *> *views, bool commit_dd_changes,
+    THD *thd, const std::vector<Table_ref *> *views, bool commit_dd_changes,
     Uncommitted_tables_guard *uncommitted_tables) {
   DBUG_TRACE;
 
@@ -542,9 +574,12 @@ static bool open_views_and_update_metadata(
     */
     if (!commit_dd_changes) uncommitted_tables->add_table(view);
 
-    // Prepare view query from the Item-tree built from the original query.
+    // Prepare view query from the Item-tree built using original query.
+    const CHARSET_INFO *view_client_cs;
+    resolve_charset(view->view_client_cs_name.str, system_charset_info,
+                    &view_client_cs);
     char view_query_buff[4096];
-    String view_query(view_query_buff, sizeof(view_query_buff), thd->charset());
+    String view_query(view_query_buff, sizeof(view_query_buff), view_client_cs);
     view_query.length(0);
 
     if (thd->lex->unit->is_mergeable() &&
@@ -554,7 +589,10 @@ static bool open_views_and_update_metadata(
         order->used_alias = false;  /// @see Item::print_for_order()
     }
     Sql_mode_parse_guard parse_guard(thd);
-    thd->lex->unit->print(thd, &view_query, QT_TO_ARGUMENT_CHARSET);
+    thd->lex->unit->print(
+        thd, &view_query,
+        static_cast<enum_query_type>(QT_TO_ARGUMENT_CHARSET |
+                                     QT_HIDE_ROLLUP_FUNCTIONS));
     if (lex_string_strmake(thd->mem_root, &view->select_stmt, view_query.ptr(),
                            view_query.length()))
       return true;
@@ -684,13 +722,12 @@ static bool update_view_metadata(THD *thd, const char *db,
                                  Uncommitted_tables_guard *uncommitted_tables) {
   if (is_view_metadata_update_needed(thd, db, tbl_or_sf_name)) {
     // Prepare list of all views referencing the db.table_name.
-    std::vector<TABLE_LIST *> views;
-    if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, &views))
+    std::vector<Table_ref *> views;
+    if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, false,
+                                    thd->mem_root, &views))
       return true;
     if (views.empty()) return false;
 
-    DEBUG_SYNC(thd, "after_preparing_view_tables_list");
-
     bool is_drop_operation = (thd->lex->sql_command == SQLCOM_DROP_TABLE ||
                               thd->lex->sql_command == SQLCOM_DROP_VIEW ||
                               thd->lex->sql_command == SQLCOM_DROP_FUNCTION ||
@@ -700,8 +737,8 @@ static bool update_view_metadata(THD *thd, const char *db,
     // If operation is drop operation then view referencing it becomes invalid.
     // Hence mark all view as invalid.
     if (is_drop_operation)
-      return mark_all_views_invalid<T>(thd, db, tbl_or_sf_name, &views,
-                                       commit_dd_changes);
+      return mark_all_views_invalid<T>(thd, db, tbl_or_sf_name, &views, false,
+                                       commit_dd_changes, thd->mem_root);
 
     /*
        Open views and update views metadata.
@@ -744,7 +781,7 @@ static bool update_referencing_views_metadata(
 }
 
 bool update_referencing_views_metadata(
-    THD *thd, const TABLE_LIST *table, const char *new_db,
+    THD *thd, const Table_ref *table, const char *new_db,
     const char *new_table_name, bool commit_dd_changes,
     Uncommitted_tables_guard *uncommitted_tables) {
   DBUG_TRACE;
@@ -757,7 +794,7 @@ bool update_referencing_views_metadata(
 }
 
 bool update_referencing_views_metadata(
-    THD *thd, const TABLE_LIST *table, bool commit_dd_changes,
+    THD *thd, const Table_ref *table, bool commit_dd_changes,
     Uncommitted_tables_guard *uncommitted_tables) {
   return update_referencing_views_metadata(
       thd, table, nullptr, nullptr, commit_dd_changes, uncommitted_tables);
@@ -780,7 +817,7 @@ bool update_referencing_views_metadata(THD *thd, const sp_name *spname) {
   assert(spname);
 
   /*
-    Updates to view metatdata for DDL on stored routines does not include
+    Updates to view metadata for DDL on stored routines does not include
     any changes to non-atomic SE. Hence transaction is not committed in
     the update_view_metadata().
   */
@@ -790,6 +827,74 @@ bool update_referencing_views_metadata(THD *thd, const sp_name *spname) {
   return error;
 }
 
+/**
+  Helper method to mark referencing views as invalid.
+
+  @tparam         T                   Type of object (View_table/View_routine)
+                                      to fetch referencing view names.
+  @param          thd                 Current thread.
+  @param          db                  Database name.
+  @param          tbl_or_sf_name      Base table/ View/ Stored function name.
+  @param          skip_same_db        Indicates whether it is OK to skip
+                                      views belonging to the same database
+                                      as table (as they will be dropped
+                                      anyway).
+  @param          commit_dd_changes   Indicates whether changes to DD need
+                                      to be committed.
+  @param          mem_root            Memory root for allocation of temporary
+                                      objects which will be cleared after
+                                      each call to this function.
+
+  @retval     false           Success.
+  @retval     true            Failure.
+
+*/
+
+template <typename T>
+static bool mark_referencing_views_invalid(THD *thd, const char *db,
+                                           const char *tbl_or_sf_name,
+                                           bool skip_same_db,
+                                           bool commit_dd_changes,
+                                           MEM_ROOT *mem_root) {
+  std::vector<Table_ref *> views;
+  if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, skip_same_db,
+                                  mem_root, &views))
+    return true;
+
+  if (views.empty()) return false;
+
+  DEBUG_SYNC(thd, "after_preparing_view_tables_list");
+
+  return mark_all_views_invalid<T>(thd, db, tbl_or_sf_name, &views,
+                                   skip_same_db, commit_dd_changes, mem_root);
+}
+
+bool mark_referencing_views_invalid(THD *thd, const Table_ref *table,
+                                    bool skip_same_db, bool commit_dd_changes,
+                                    MEM_ROOT *mem_root) {
+  DBUG_TRACE;
+
+  const char *db_name = table->get_db_name();
+  const char *table_name = table->get_table_name();
+
+  // Don't mark referencing view invalid if we drop dictionary table.
+  if (dd::get_dictionary()->is_dd_table_name(db_name, table_name)) return false;
+
+  return mark_referencing_views_invalid<dd::View_table>(
+      thd, db_name, table_name, skip_same_db, commit_dd_changes, mem_root);
+}
+
+bool mark_referencing_views_invalid(THD *thd, const sp_name *spname,
+                                    MEM_ROOT *mem_root) {
+  /*
+    DROP DATABASE always drops routines atomically so we don't need to commit
+    transaction here. Also since DROP DATABASE drops routines views are dropped
+    there is no point in using skip-views-in-the-same-db optimization here.
+  */
+  return mark_referencing_views_invalid<dd::View_routine>(
+      thd, spname->m_db.str, spname->m_name.str, false, false, mem_root);
+}
+
 std::string push_view_warning_or_error(THD *thd, const char *db,
                                        const char *view_name) {
   // Report error for "SHOW FIELDS/DESCRIBE" operations.
