@@ -1,15 +1,16 @@
-/* Copyright (c) 2005, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,9 +40,9 @@
 #include "my_macros.h"
 #include "my_sys.h"
 #include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/components/services/bits/psi_memory_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
-#include "mysql/components/services/psi_memory_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
@@ -147,14 +148,14 @@ static bool load_events_from_db(THD *thd, Event_queue *event_queue);
     1   s > t
 */
 
-int sortcmp_lex_string(LEX_CSTRING s, LEX_CSTRING t, CHARSET_INFO *cs) {
+int sortcmp_lex_string(LEX_CSTRING s, LEX_CSTRING t, const CHARSET_INFO *cs) {
   return cs->coll->strnncollsp(cs, pointer_cast<const uchar *>(s.str), s.length,
                                pointer_cast<const uchar *>(t.str), t.length);
 }
 
 /*
   Reconstructs interval expression from interval type and expression
-  value that is in form of a value of the smalles entity:
+  value that is in form of a value of the smallest entity:
   For
     YEAR_MONTH - expression is in months
     DAY_MINUTE - expression is in minutes
@@ -563,6 +564,13 @@ bool Events::update_event(THD *thd, Event_parse_data *parse_data,
       trans_commit_stmt(thd) || trans_commit(thd))
     goto err_with_rollback;
 
+  if (new_dbname != nullptr) {
+    /* RENAME: Drop the old event instrumentation. */
+    MYSQL_DROP_SP(to_uint(enum_sp_type::EVENT), parse_data->dbname.str,
+                  parse_data->dbname.length, parse_data->name.str,
+                  parse_data->name.length);
+  }
+
   // Update element in event queue.
   if (event_queue && new_element != nullptr) {
     /*
@@ -714,6 +722,13 @@ bool Events::lock_schema_events(THD *thd, const dd::Schema &schema) {
     schema_name = schema_name_buf;
   }
 
+  /*
+    Ensure that we don't hold memory used by MDL_requests after locks have
+    been acquired. This reduces memory usage in cases when we have DROP
+    DATABASE that needs to drop lots of different objects.
+  */
+  MEM_ROOT mdl_reqs_root(key_memory_rm_db_mdl_reqs_root, MEM_ROOT_BLOCK_SIZE);
+
   MDL_request_list mdl_requests;
   for (std::vector<dd::String_type>::const_iterator name = event_names.begin();
        name != event_names.end(); ++name) {
@@ -721,7 +736,7 @@ bool Events::lock_schema_events(THD *thd, const dd::Schema &schema) {
     dd::Event::create_mdl_key(dd::String_type(schema_name), *name, &mdl_key);
 
     // Add MDL_request for routine to mdl_requests list.
-    MDL_request *mdl_request = new (thd->mem_root) MDL_request;
+    MDL_request *mdl_request = new (&mdl_reqs_root) MDL_request;
     MDL_REQUEST_INIT_BY_KEY(mdl_request, &mdl_key, MDL_EXCLUSIVE,
                             MDL_TRANSACTION);
     mdl_requests.push_front(mdl_request);
@@ -809,14 +824,14 @@ static bool send_show_create_event(THD *thd, Event_timed *et,
                          system_charset_info);
   protocol->store_string(show_str.c_ptr(), show_str.length(),
                          et->m_creation_ctx->get_client_cs());
-  protocol->store_string(et->m_creation_ctx->get_client_cs()->csname,
-                         strlen(et->m_creation_ctx->get_client_cs()->csname),
-                         system_charset_info);
-  protocol->store_string(et->m_creation_ctx->get_connection_cl()->name,
-                         strlen(et->m_creation_ctx->get_connection_cl()->name),
-                         system_charset_info);
-  protocol->store_string(et->m_creation_ctx->get_db_cl()->name,
-                         strlen(et->m_creation_ctx->get_db_cl()->name),
+  const char *csname = et->m_creation_ctx->get_client_cs()->csname;
+  protocol->store_string(csname, strlen(csname), system_charset_info);
+  protocol->store_string(
+      et->m_creation_ctx->get_connection_cl()->m_coll_name,
+      strlen(et->m_creation_ctx->get_connection_cl()->m_coll_name),
+      system_charset_info);
+  protocol->store_string(et->m_creation_ctx->get_db_cl()->m_coll_name,
+                         strlen(et->m_creation_ctx->get_db_cl()->m_coll_name),
                          system_charset_info);
 
   if (protocol->end_row()) return true;
@@ -1009,9 +1024,10 @@ static PSI_cond_info all_events_conds[] = {
 PSI_thread_key key_thread_event_scheduler, key_thread_event_worker;
 
 static PSI_thread_info all_events_threads[] = {
-    {&key_thread_event_scheduler, "event_scheduler", PSI_FLAG_SINGLETON, 0,
-     PSI_DOCUMENT_ME},
-    {&key_thread_event_worker, "event_worker", 0, 0, PSI_DOCUMENT_ME}};
+    {&key_thread_event_scheduler, "event_scheduler", "evt_sched",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
+    {&key_thread_event_worker, "event_worker", "evt_wkr", 0, 0,
+     PSI_DOCUMENT_ME}};
 #endif /* HAVE_PSI_INTERFACE */
 
 PSI_stage_info stage_waiting_on_empty_queue = {0, "Waiting on empty queue", 0,
@@ -1153,7 +1169,7 @@ static bool load_events_from_db(THD *thd, Event_queue *event_queue) {
     if (lower_case_table_names == 2) {
       // Lower case table names == 2 is tested on OSX.
       my_stpcpy(name_buf, converted_schema_name);
-      my_casedn_str(&my_charset_utf8_tolower_ci, name_buf);
+      my_casedn_str(&my_charset_utf8mb3_tolower_ci, name_buf);
       converted_schema_name = name_buf;
     }
 
