@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +42,7 @@
 #include "sql/current_thd.h"
 #include "sql/field.h"
 #include "sql/handler.h"
-#include "sql/mysqld.h"  // key_master_info_run_lock
+#include "sql/mysqld.h"  // key_source_info_run_lock
 #include "sql/rpl_filter.h"
 #include "sql/rpl_info.h"
 #include "sql/rpl_info_dummy.h"         // Rpl_info_dummy
@@ -50,9 +51,9 @@
 #include "sql/rpl_info_table_access.h"  // Rpl_info_table_access
 #include "sql/rpl_mi.h"                 // Master_info
 #include "sql/rpl_msr.h"                // channel_map
-#include "sql/rpl_rli.h"                // Relay_log_info
-#include "sql/rpl_rli_pdb.h"            // Slave_worker
-#include "sql/rpl_slave.h"
+#include "sql/rpl_replica.h"
+#include "sql/rpl_rli.h"      // Relay_log_info
+#include "sql/rpl_rli_pdb.h"  // Slave_worker
 #include "sql/sql_class.h"
 #include "sql/sql_const.h"
 #include "sql/system_variables.h"
@@ -61,7 +62,7 @@
 #include "thr_lock.h"
 
 /*
-  Defines meta information on diferent repositories.
+  Defines meta information on different repositories.
 */
 Rpl_info_factory::struct_table_data Rpl_info_factory::rli_table_data;
 Rpl_info_factory::struct_file_data Rpl_info_factory::rli_file_data;
@@ -77,7 +78,7 @@ Rpl_info_factory::struct_table_data Rpl_info_factory::worker_table_data;
   @param[in]  channel   the channel for which mi is to be created
   @param[in]  to_decide_repo     the flag is set to true if mi repositories
                                  are allowed to convert. For details,
-                                 see init_slave()
+                                 see init_replica()
 
   The execution fails if a user requests a type but a different type
   already exists in the system. This is done to avoid that a user
@@ -94,18 +95,18 @@ Master_info *Rpl_info_factory::create_mi(uint mi_option, const char *channel,
   Rpl_info_handler *handler_dest = nullptr;
   uint instances = 1;
   const char *msg =
-      "Failed to allocate memory for the master info "
+      "Failed to allocate memory for the connection metadata "
       "structure";
 
   DBUG_TRACE;
 
   if (!(mi = new Master_info(
 #ifdef HAVE_PSI_INTERFACE
-            &key_master_info_run_lock, &key_master_info_data_lock,
-            &key_master_info_sleep_lock, &key_master_info_thd_lock,
-            &key_master_info_rotate_lock, &key_master_info_data_cond,
-            &key_master_info_start_cond, &key_master_info_stop_cond,
-            &key_master_info_sleep_cond, &key_master_info_rotate_cond,
+            &key_source_info_run_lock, &key_source_info_data_lock,
+            &key_source_info_sleep_lock, &key_source_info_thd_lock,
+            &key_source_info_rotate_lock, &key_source_info_data_cond,
+            &key_source_info_start_cond, &key_source_info_stop_cond,
+            &key_source_info_sleep_cond, &key_source_info_rotate_cond,
 #endif
             instances, channel)))
     goto err;
@@ -143,7 +144,7 @@ err:
     mi->channel_wrlock();
     delete mi;
   }
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_MASTER_INFO, msg);
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_CONNECTION_METADATA, msg);
   return nullptr;
 }
 
@@ -178,7 +179,8 @@ err:
   delete handler_dest;
   handler_dest = nullptr;
 
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CHANGING_MASTER_INFO_REPO_TYPE, *msg);
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CHANGING_CONNECTION_METADATA_REPO_TYPE,
+         *msg);
   return true;
 }
 
@@ -210,10 +212,10 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
   Rpl_info_handler *handler_dest = nullptr;
   uint instances = 1;
   uint worker_repository = INVALID_INFO_REPOSITORY;
-  uint worker_instances = 1;
+  std::string scan_msg;
   const char *msg = nullptr;
   const char *msg_alloc =
-      "Failed to allocate memory for the relay log info "
+      "Failed to allocate memory for the applier metadata "
       "structure";
   Rpl_filter *rpl_filter = nullptr;
 
@@ -226,10 +228,11 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
     found.
   */
   if (rli_option != INFO_REPOSITORY_DUMMY &&
-      scan_repositories(&worker_instances, &worker_repository,
-                        worker_table_data, worker_file_data, &msg))
+      scan_and_check_repositories(worker_repository, worker_table_data,
+                                  worker_file_data, scan_msg)) {
+    msg = scan_msg.c_str();
     goto err;
-
+  }
   if (!(rli = new Relay_log_info(
             is_slave_recovery,
 #ifdef HAVE_PSI_INTERFACE
@@ -254,7 +257,7 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
       worker_repository != rli_option) {
     opt_rli_repository_id = rli_option = worker_repository;
     LogErr(WARNING_LEVEL,
-           ER_RPL_CHANGING_RELAY_LOG_INFO_REPO_TYPE_FAILED_DUE_TO_GAPS);
+           ER_RPL_CHANGING_APPLIER_METADATA_REPO_TYPE_FAILED_DUE_TO_GAPS);
     std::swap(handler_src, handler_dest);
   }
 
@@ -263,7 +266,7 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
       goto err;
   } else {
     if (channel != nullptr) {
-      /* Here dest code should be TABLE type repo. See, init_slave() */
+      /* Here dest code should be TABLE type repo. See, init_replica() */
       if (handler_dest->get_rpl_info_type() != INFO_REPOSITORY_TABLE) {
         LogErr(ERROR_LEVEL, ER_RPL_REPO_SHOULD_BE_TABLE);
         goto err;
@@ -285,7 +288,7 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
   /* Set filters here to guarantee that any rli object has a valid filter */
   rpl_filter = rpl_channel_filters.get_channel_filter(channel);
   if (rpl_filter == nullptr) {
-    LogErr(ERROR_LEVEL, ER_RPL_SLAVE_FILTER_CREATE_FAILED, channel);
+    LogErr(ERROR_LEVEL, ER_RPL_REPLICA_FILTER_CREATE_FAILED, channel);
     msg = msg_alloc;
     goto err;
   }
@@ -305,7 +308,7 @@ err:
     rli->set_rpl_info_handler(nullptr);
     delete rli;
   }
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_RELAY_LOG_INFO, msg);
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_APPLIER_METADATA, msg);
   return nullptr;
 }
 
@@ -341,7 +344,7 @@ err:
   delete handler_dest;
   handler_dest = nullptr;
 
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CHANGING_RELAY_LOG_INFO_REPO_TYPE, *msg);
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CHANGING_APPLIER_METADATA_REPO_TYPE, *msg);
   return true;
 }
 
@@ -358,7 +361,12 @@ bool Rpl_info_factory::reset_workers(Relay_log_info *rli) {
 
   DBUG_TRACE;
 
-  if (rli->recovery_parallel_workers == 0) return false;
+  /*
+    Skip the optimization check if the last value of the number of workers
+    might not have been persisted
+  */
+  if (rli->recovery_parallel_workers == 0 && !rli->mi->is_gtid_only_mode())
+    return false;
 
   if (Rpl_info_file::do_reset_info(
           Slave_worker::get_number_worker_fields(), worker_file_data.pattern,
@@ -373,17 +381,18 @@ bool Rpl_info_factory::reset_workers(Relay_log_info *rli) {
 
   error = false;
 
-  DBUG_EXECUTE_IF("mts_debug_reset_workers_fails", error = true;);
+  DBUG_EXECUTE_IF("mta_debug_reset_workers_fails", error = true;);
 
 err:
   if (error)
     LogErr(ERROR_LEVEL,
-           ER_RPL_FAILED_TO_DELETE_FROM_SLAVE_WORKERS_INFO_REPOSITORY);
+           ER_RPL_FAILED_TO_DELETE_FROM_REPLICA_WORKERS_INFO_REPOSITORY);
   rli->recovery_parallel_workers = 0;
   rli->clear_mts_recovery_groups();
-  if (rli->flush_info(true)) {
+  if (rli->flush_info(Relay_log_info::RLI_FLUSH_IGNORE_SYNC_OPT)) {
     error = true;
-    LogErr(ERROR_LEVEL, ER_RPL_FAILED_TO_RESET_STATE_IN_SLAVE_INFO_REPOSITORY);
+    LogErr(ERROR_LEVEL,
+           ER_RPL_FAILED_TO_RESET_STATE_IN_REPLICA_INFO_REPOSITORY);
   }
   return error;
 }
@@ -437,8 +446,8 @@ Slave_worker *Rpl_info_factory::create_worker(uint rli_option, uint worker_id,
                         &handler_src, &handler_dest, &msg))
     goto err;
   /*
-    Preparing the being set up handler with search keys early.
-    The file repo type handler can't be manupulated this way and it does
+    Preparing the handler being set up with search keys early.
+    The file repo type handler can't be manipulated this way and it does
     not have to.
   */
   if (handler_dest->get_rpl_info_type() == INFO_REPOSITORY_TABLE)
@@ -452,15 +461,15 @@ Slave_worker *Rpl_info_factory::create_worker(uint rli_option, uint worker_id,
   if (decide_repository(worker, rli_option, &handler_src, &handler_dest, &msg))
     goto err;
 
-  if (DBUG_EVALUATE_IF("mts_worker_thread_init_fails", 1, 0) ||
+  if (DBUG_EVALUATE_IF("mta_worker_thread_init_fails", 1, 0) ||
       worker->rli_init_info(is_gaps_collecting_phase)) {
-    DBUG_EXECUTE_IF("enable_mts_worker_failure_init", {
-      DBUG_SET("-d,mts_worker_thread_init_fails");
-      DBUG_SET("-d,enable_mts_worker_failure_init");
+    DBUG_EXECUTE_IF("enable_mta_worker_failure_init", {
+      DBUG_SET("-d,mta_worker_thread_init_fails");
+      DBUG_SET("-d,enable_mta_worker_failure_init");
     });
-    DBUG_EXECUTE_IF("enable_mts_wokrer_failure_in_recovery_finalize", {
-      DBUG_SET("-d,mts_worker_thread_init_fails");
-      DBUG_SET("-d,enable_mts_wokrer_failure_in_recovery_finalize");
+    DBUG_EXECUTE_IF("enable_mta_wokrer_failure_in_recovery_finalize", {
+      DBUG_SET("-d,mta_worker_thread_init_fails");
+      DBUG_SET("-d,enable_mta_wokrer_failure_in_recovery_finalize");
     });
     msg = "Failed to initialize the worker info structure";
     goto err;
@@ -483,7 +492,7 @@ err:
     worker->set_rpl_info_handler(nullptr);
     delete worker;
   }
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_RELAY_LOG_INFO, msg);
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_APPLIER_METADATA, msg);
   return nullptr;
 }
 
@@ -498,7 +507,7 @@ static void build_worker_info_name(char *to, const char *path,
 }
 
 /**
-  Initializes startup information on diferent repositories.
+  Initializes startup information on different repositories.
 */
 void Rpl_info_factory::init_repository_metadata() {
   /* Needed for the file names and paths for worker info files. */
@@ -570,7 +579,7 @@ void Rpl_info_factory::init_repository_metadata() {
                            be used, i.e., destination repository.
   @param[out] handler_src  Source repository from where information is
                            copied into the destination repository.
-  @param[out] handler_dest Destination repository to where informaiton is
+  @param[out] handler_dest Destination repository to where information is
                            copied.
   @param[out] msg          Error message if something goes wrong.
 
@@ -745,7 +754,7 @@ enum_return_check Rpl_info_factory::check_src_repository(
 
   @param  handler_src  Source repository from where information is
                        copied into the destination repository.
-  @param  handler_dest Destination repository to where informaiton is
+  @param  handler_dest Destination repository to where information is
                        copied.
   @param  err_src      Possible error status of the source repo check
   @param  err_dst      Possible error status of the destination repo check
@@ -785,7 +794,7 @@ bool Rpl_info_factory::check_error_repository(Rpl_info_handler *handler_src,
   @param[in]  info         Either master info or relay log info.
   @param[out] handler_src  Source repository from where information is
                            copied into the destination repository.
-  @param[out] handler_dest Destination repository to where informaiton is
+  @param[out] handler_dest Destination repository to where information is
                            copied.
   @param[out] msg          Error message if something goes wrong.
 
@@ -823,7 +832,7 @@ bool Rpl_info_factory::init_repositories(Rpl_info *info,
                            be used, i.e., destination repository.
   @param[out] handler_src  Source repository from where information is
                            copied into the destination repository.
-  @param[out] handler_dest Destination repository to where informaiton is
+  @param[out] handler_dest Destination repository to where information is
                            copied.
   @param[out] msg          Error message if something goes wrong.
 
@@ -837,7 +846,7 @@ bool Rpl_info_factory::init_repositories(const struct_table_data &table_data,
                                          Rpl_info_handler **handler_dest,
                                          const char **msg) {
   bool error = true;
-  *msg = "Failed to allocate memory for master info repositories";
+  *msg = "Failed to allocate memory for connection metadata repositories";
 
   DBUG_TRACE;
 
@@ -884,54 +893,85 @@ err:
   return error;
 }
 
-bool Rpl_info_factory::scan_repositories(uint *found_instances,
-                                         uint *found_rep_option,
-                                         const struct_table_data &table_data,
-                                         const struct_file_data &file_data,
-                                         const char **msg) {
-  bool error = false;
+bool Rpl_info_factory::scan_and_count_repositories(
+    ulonglong &found_instances, uint &found_rep_option,
+    const struct_table_data &table_data, const struct_file_data &file_data,
+    std::string &msg) {
   uint file_instances = 0;
-  uint table_instances = 0;
-  assert(found_rep_option != nullptr);
+  ulonglong table_instances = 0;
 
   DBUG_TRACE;
 
   if (Rpl_info_table::do_count_info(
           table_data.n_fields, table_data.schema, table_data.name,
           &table_data.nullable_fields, &table_instances)) {
-    error = true;
-    goto err;
+    return true;
   }
 
   if (Rpl_info_file::do_count_info(
           file_data.n_fields, file_data.pattern, file_data.name_indexed,
           &file_data.nullable_fields, &file_instances)) {
-    error = true;
-    goto err;
+    return true;
   }
 
   if (file_instances != 0 && table_instances != 0) {
-    error = true;
-    *msg =
+    msg.assign(
         "Multiple repository instances found with data in "
         "them. Unable to decide which is the correct one to "
-        "choose";
-    goto err;
+        "choose");
+    return true;
   }
 
   if (table_instances != 0) {
-    *found_instances = table_instances;
-    *found_rep_option = INFO_REPOSITORY_TABLE;
+    found_instances = table_instances;
+    found_rep_option = INFO_REPOSITORY_TABLE;
   } else if (file_instances != 0) {
-    *found_instances = file_instances;
-    *found_rep_option = INFO_REPOSITORY_FILE;
+    found_instances = file_instances;
+    found_rep_option = INFO_REPOSITORY_FILE;
   } else {
-    *found_instances = 0;
-    *found_rep_option = INVALID_INFO_REPOSITORY;
+    found_instances = 0;
+    found_rep_option = INVALID_INFO_REPOSITORY;
   }
 
-err:
-  return error;
+  return false;
+}
+
+bool Rpl_info_factory::scan_and_check_repositories(
+    uint &found_rep_option, const struct_table_data &table_data,
+    const struct_file_data &file_data, std::string &msg) {
+  uint file_instances = 0;
+
+  DBUG_TRACE;
+
+  auto [error, table_in_use] = Rpl_info_table::table_in_use(
+      table_data.n_fields, table_data.schema, table_data.name,
+      &table_data.nullable_fields);
+
+  if (error) return true;
+
+  if (Rpl_info_file::do_count_info(
+          file_data.n_fields, file_data.pattern, file_data.name_indexed,
+          &file_data.nullable_fields, &file_instances)) {
+    return true;
+  }
+
+  if (file_instances != 0 && table_in_use) {
+    msg.assign(
+        "Multiple repository instances found with data in "
+        "them. Unable to decide which is the correct one to "
+        "choose");
+    return true;
+  }
+
+  if (table_in_use) {
+    found_rep_option = INFO_REPOSITORY_TABLE;
+  } else if (file_instances != 0) {
+    found_rep_option = INFO_REPOSITORY_FILE;
+  } else {
+    found_rep_option = INVALID_INFO_REPOSITORY;
+  }
+
+  return false;
 }
 
 bool Rpl_info_factory::configure_channel_replication_filters(
@@ -953,7 +993,7 @@ bool Rpl_info_factory::configure_channel_replication_filters(
       when it is being configured.
     */
     if (rli->rpl_filter->copy_global_replication_filters()) {
-      LogErr(ERROR_LEVEL, ER_RPL_SLAVE_GLOBAL_FILTERS_COPY_FAILED,
+      LogErr(ERROR_LEVEL, ER_RPL_REPLICA_GLOBAL_FILTERS_COPY_FAILED,
              channel_name);
       return true;
     }
@@ -964,7 +1004,7 @@ bool Rpl_info_factory::configure_channel_replication_filters(
       warning.
     */
     if (!rli->rpl_filter->is_empty()) {
-      LogErr(WARNING_LEVEL, ER_RPL_SLAVE_RESET_FILTER_OPTIONS, channel_name);
+      LogErr(WARNING_LEVEL, ER_RPL_REPLICA_RESET_FILTER_OPTIONS, channel_name);
       rli->rpl_filter->reset();
     }
   }
@@ -972,7 +1012,7 @@ bool Rpl_info_factory::configure_channel_replication_filters(
 }
 
 /**
-  This function should be called from init_slave() only.
+  This function should be called from init_replica() only.
 
   During the server start, read all the slave repositories
   on disk (either in FILE or TABLE form) and create corresponding
@@ -1024,10 +1064,10 @@ bool Rpl_info_factory::configure_channel_replication_filters(
 
  From the table it also follows that conversion of repositories
  is possible *ONLY* in the case of A) i.e for ex: if B) type repository
- (i.e a named slave channel) was found during server starup but the user
+ (i.e a named slave channel) was found during server startup but the user
  repository option is INFO_REPOSITORY_FILE, then we exit the function.
 
- @note: only for type A) i.e default channel, it is permissable to
+ @note: only for type A) i.e default channel, it is permissible to
         have different repo types for Master_info and Relay_log_info
         (Ex: FILE for mi and TABLE for rli)
 
@@ -1071,14 +1111,14 @@ bool Rpl_info_factory::create_slave_info_objects(
   DBUG_TRACE;
 
   Master_info *mi = nullptr;
-  const char *msg = nullptr;
+  std::string msg;
   bool error = false, channel_error;
   bool default_channel_existed_previously = false;
 
   std::vector<std::string> channel_list;
 
   /* Number of instances of Master_info repository */
-  uint mi_instances = 0;
+  ulonglong mi_instances = 0;
 
   /* At this point, the repository in invalid or unknown */
   uint mi_repository = INVALID_INFO_REPOSITORY;
@@ -1086,9 +1126,9 @@ bool Rpl_info_factory::create_slave_info_objects(
   /*
     Number of instances of Relay_log_info_repository.
     (Number of Slave worker objects that will be created by the Coordinator
-    (when slave_parallel_workers>0) at a later stage and not here).
+    (when replica_parallel_workers>0) at a later stage and not here).
   */
-  uint rli_instances = 0;
+  ulonglong rli_instances = 0;
 
   /* At this point, the repository is invalid or unknown */
   uint rli_repository = INVALID_INFO_REPOSITORY;
@@ -1101,12 +1141,12 @@ bool Rpl_info_factory::create_slave_info_objects(
   Rpl_info_factory::init_repository_metadata();
 
   /* Count the number of Master_info and Relay_log_info repositories */
-  if (scan_repositories(&mi_instances, &mi_repository, mi_table_data,
-                        mi_file_data, &msg) ||
-      scan_repositories(&rli_instances, &rli_repository, rli_table_data,
-                        rli_file_data, &msg)) {
+  if (scan_and_count_repositories(mi_instances, mi_repository, mi_table_data,
+                                  mi_file_data, msg) ||
+      scan_and_count_repositories(rli_instances, rli_repository, rli_table_data,
+                                  rli_file_data, msg)) {
     /* msg will contain the reason of failure */
-    LogErr(ERROR_LEVEL, ER_RPL_SLAVE_GENERIC_MESSAGE, msg);
+    LogErr(ERROR_LEVEL, ER_RPL_REPLICA_GENERIC_MESSAGE, msg.c_str());
     error = true;
     goto end;
   }
@@ -1116,7 +1156,7 @@ bool Rpl_info_factory::create_slave_info_objects(
                                          mi_repository,
                                          pchannel_map->get_default_channel(),
                                          &default_channel_existed_previously)) {
-    LogErr(ERROR_LEVEL, ER_RPL_SLAVE_COULD_NOT_CREATE_CHANNEL_LIST);
+    LogErr(ERROR_LEVEL, ER_RPL_REPLICA_COULD_NOT_CREATE_CHANNEL_LIST);
     error = true;
     goto end;
   }
@@ -1161,14 +1201,18 @@ bool Rpl_info_factory::create_slave_info_objects(
     if (!channel_error &&
         (!is_default_channel || default_channel_existed_previously)) {
       bool ignore_if_no_info = (channel_list.size() == 1) ? true : false;
-      channel_error =
-          load_mi_and_rli_from_repositories(mi, ignore_if_no_info, thread_mask);
+      channel_error = load_mi_and_rli_from_repositories(
+          mi, ignore_if_no_info, thread_mask, false, true);
     }
 
     if (!channel_error) {
       error = configure_channel_replication_filters(mi->rli, cname);
+      invalidate_repository_position(mi);
+      // With GTID ONLY the worker info is not needed
+      if (mi->is_gtid_only_mode()) Rpl_info_factory::reset_workers(mi->rli);
     } else {
-      LogErr(ERROR_LEVEL, ER_RPL_SLAVE_FAILED_TO_INIT_A_MASTER_INFO_STRUCTURE,
+      LogErr(ERROR_LEVEL,
+             ER_RPL_REPLICA_FAILED_TO_INIT_A_CONNECTION_METADATA_STRUCTURE,
              cname);
     }
     error = error || channel_error;
@@ -1233,9 +1277,9 @@ Master_info *Rpl_info_factory::create_mi_and_rli_objects(
 }
 
 /**
-   Make a list of all the channels if existed on the previos slave run.
+   Make a list of any channels that may have existed on the previous slave run.
 
-   @param[out]  channel_list    the names of all channels that exists
+   @param[out]  channel_list    the names of all channels that exist
                                 on this slave.
 
    @param[in]   mi_instances    number of master_info repositories
@@ -1254,9 +1298,9 @@ Master_info *Rpl_info_factory::create_mi_and_rli_objects(
 */
 
 bool Rpl_info_factory::load_channel_names_from_repository(
-    std::vector<std::string> &channel_list,
-    uint mi_instances MY_ATTRIBUTE((unused)), uint mi_repository,
-    const char *default_channel, bool *default_channel_existed_previously) {
+    std::vector<std::string> &channel_list, uint mi_instances [[maybe_unused]],
+    uint mi_repository, const char *default_channel,
+    bool *default_channel_existed_previously) {
   DBUG_TRACE;
 
   *default_channel_existed_previously = false;
@@ -1286,7 +1330,7 @@ bool Rpl_info_factory::load_channel_names_from_repository(
 }
 
 /**
-  In a multisourced slave, during init_slave(), the repositories
+  In a multisourced slave, during init_replica(), the repositories
   are read to initialize the slave info objects. To initialize
   the slave info objects, we need the number of channels the slave
   was connected to previously. The following function, finds the
@@ -1354,7 +1398,8 @@ bool Rpl_info_factory::load_channel_names_from_table(
 
   /* Ensure that the table pk (Channel_name) is at the correct position */
   if (info->verify_table_primary_key_fields(table)) {
-    LogErr(ERROR_LEVEL, ER_RPL_SLAVE_FAILED_TO_CREATE_CHANNEL_FROM_MASTER_INFO);
+    LogErr(ERROR_LEVEL,
+           ER_RPL_REPLICA_FAILED_TO_CREATE_CHANNEL_FROM_CONNECTION_METADATA);
     error = -1;
     goto err;
   }
@@ -1395,3 +1440,10 @@ err:
   delete info;
   return error != HA_ERR_END_OF_FILE && error != 0;
 }
+
+void Rpl_info_factory::invalidate_repository_position(Master_info *mi) {
+  if (mi->is_gtid_only_mode()) {
+    mi->set_receiver_position_info_invalid(true);
+    mi->rli->set_applier_source_position_info_invalid(true);
+  }
+}
