@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,15 +22,20 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "sql/rpl_io_monitor.h"
+#include <mysql/components/my_service.h>
+#include <mysql/components/services/group_replication_status_service.h>
 #include "mysql/components/services/log_builtins.h"
 
-#include "sql/json_dom.h"
+#include "sql-common/json_dom.h"
+#include "sql/changestreams/apply/replication_thread_status.h"
 #include "sql/mysqld.h"
 #include "sql/mysqld_thd_manager.h"  // Global_THD_manager
 #include "sql/protocol_classic.h"
 #include "sql/rpl_async_conn_failover.h"  // reset_pos
-#include "sql/rpl_msr.h"                  /* Multisource replication */
-#include "sql/rpl_slave.h"
+#include "sql/rpl_async_conn_failover_configuration_propagation.h"
+#include "sql/rpl_group_replication.h"
+#include "sql/rpl_msr.h" /* Multisource replication */
+#include "sql/rpl_replica.h"
 #include "sql/rpl_sys_key_access.h"
 #include "sql/rpl_sys_table_access.h"
 #include "sql/sql_class.h"  // THD
@@ -53,8 +59,6 @@
 static bool restart_io_thread(THD *thd, const std::string &channel_name,
                               bool force_sender_with_highest_weight);
 
-bool Source_IO_monitor::m_monitor_thd_initiated = false;
-
 /*
   The SQL_QUERIES array contains three queries. The enum_sql_query_tag index/tag
   is used to get each query. There are for following purpose:
@@ -163,9 +167,7 @@ std::string Source_IO_monitor::get_query(enum_sql_query_tag qtag) {
   return query;
 }
 
-Source_IO_monitor::Source_IO_monitor() { init_mutex(); }
-
-void Source_IO_monitor::init_mutex() {
+Source_IO_monitor::Source_IO_monitor() {
 #ifdef HAVE_PSI_INTERFACE
   mysql_mutex_init(key_monitor_info_run_lock, &m_run_lock, MY_MUTEX_INIT_FAST);
   mysql_cond_init(key_monitor_info_run_cond, &m_run_cond);
@@ -173,15 +175,12 @@ void Source_IO_monitor::init_mutex() {
   mysql_mutex_init(nullptr, &m_run_lock, MY_MUTEX_INIT_FAST);
   mysql_cond_init(nullptr, &m_run_cond);
 #endif
-  m_monitor_thd_initiated = true;
 }
 
-void Source_IO_monitor::cleanup_mutex() {
-  if (m_monitor_thd_initiated) {
-    mysql_mutex_destroy(&m_run_lock);
-    mysql_cond_destroy(&m_run_cond);
-    m_monitor_thd_initiated = false;
-  }
+Source_IO_monitor::~Source_IO_monitor() {
+  terminate_monitoring_process();
+  mysql_mutex_destroy(&m_run_lock);
+  mysql_cond_destroy(&m_run_cond);
 }
 
 bool Source_IO_monitor::is_monitor_killed(THD *thd, Master_info *) {
@@ -194,7 +193,6 @@ bool Source_IO_monitor::is_monitor_killed(THD *thd, Master_info *) {
 bool Source_IO_monitor::launch_monitoring_process(PSI_thread_key thread_key) {
   DBUG_TRACE;
 
-  if (!m_monitor_thd_initiated) init_mutex();
   mysql_mutex_lock(&m_run_lock);
 
   // Callers should ensure the process is terminated
@@ -206,7 +204,7 @@ bool Source_IO_monitor::launch_monitoring_process(PSI_thread_key thread_key) {
 
   if (mysql_thread_create(thread_key, &m_th, &connection_attrib,
                           launch_handler_thread, (void *)this)) {
-    my_error(ER_SLAVE_THREAD, MYF(0));
+    my_error(ER_REPLICA_THREAD, MYF(0));
     mysql_mutex_unlock(&m_run_lock);
     return true;
   }
@@ -224,7 +222,7 @@ bool Source_IO_monitor::launch_monitoring_process(PSI_thread_key thread_key) {
 
 void Source_IO_monitor::source_monitor_handler() {
   THD *thd{nullptr};  // needs to be first for thread_stack
-  thd = new THD;      // note that contructor of THD uses DBUG_ !
+  thd = new THD;      // note that constructor of THD uses DBUG_ !
   m_monitor_thd = thd;
   struct timespec waittime;
 
@@ -240,8 +238,8 @@ void Source_IO_monitor::source_monitor_handler() {
 #endif
   thd->thread_stack = (char *)&thd;  // remember where our stack is
 
-  if (init_slave_thread(thd, SLAVE_THD_IO)) {
-    my_error(ER_SLAVE_FATAL_ERROR, MYF(0),
+  if (init_replica_thread(thd, SLAVE_THD_IO)) {
+    my_error(ER_REPLICA_FATAL_ERROR, MYF(0),
              "Failed during Replica IO Monitor thread initialization ");
     goto err;
   }
@@ -257,7 +255,8 @@ void Source_IO_monitor::source_monitor_handler() {
   mysql_cond_broadcast(&m_run_cond);
   mysql_mutex_unlock(&m_run_lock);
 
-  while (!is_monitor_killed(thd, nullptr)) {
+  while (!is_monitor_killed(thd, nullptr) &&
+         !is_group_replication_member_secondary()) {
     sync_senders_details(thd);
 
     THD_STAGE_INFO(thd, stage_rpl_failover_wait_before_next_fetch);
@@ -301,6 +300,7 @@ std::tuple<bool, std::string> Source_IO_monitor::delete_rows(
     std::tuple<std::string, std::string, uint> conn_detail) {
   bool err_val{false};
   std::string err_msg{};
+
   Rpl_sys_table_access::for_each_in_tuple(
       conn_detail, [&](const auto &n, const auto &x) {
         if (table_op.store_field(table->field[n], x)) {
@@ -320,6 +320,7 @@ std::tuple<bool, std::string> Source_IO_monitor::write_rows(
     RPL_FAILOVER_SOURCE_TUPLE conn_detail) {
   bool err_val{false};
   std::string err_msg{};
+
   Rpl_sys_table_access::for_each_in_tuple(
       conn_detail, [&](const auto &n, const auto &x) {
         if (table_op.store_field(table->field[n], x)) {
@@ -431,7 +432,7 @@ int Source_IO_monitor::connect_senders(THD *thd,
     uint mi_port = mi->port;
     const std::string mi_network_namespace(mi->network_namespace_str());
 
-    THD_STAGE_INFO(thd, stage_connecting_to_master);
+    THD_STAGE_INFO(thd, stage_connecting_to_source);
     Mysql_connection *conn =
         new Mysql_connection(thd, mi, host, port, mi_network_namespace);
     if (!conn->is_connected()) {
@@ -472,7 +473,7 @@ int Source_IO_monitor::connect_senders(THD *thd,
     }
 
     /*
-      3.4. Store gathered memebership details to
+      3.4. Store gathered membership details to
            replication_asynchronous_connection_failover table.
     */
     THD_STAGE_INFO(thd, stage_rpl_failover_updating_source_member_details);
@@ -746,13 +747,47 @@ int Source_IO_monitor::save_group_members(
     }
   }
 
-  if (table_op.close(err_val)) {
+  /* Increment member action configuration version. */
+  if (table_op.increment_version()) {
+    LogErr(ERROR_LEVEL, ER_RPL_INCREMENTING_MEMBER_ACTION_VERSION, db.c_str(),
+           table_name.c_str());
+    return 1;
+  }
+
+  /*
+    Send replication_asynchronous_connection_failover data to group replication
+    group members.
+  */
+  if (rpl_acf_configuration_handler->send_failover_data(table_op)) {
     return 1;
   }
 
   return 0;
 }
 
+bool Source_IO_monitor::has_primary_lost_contact_with_majority() {
+  bool primary_lost_contact_with_majority = false;
+  my_h_service gr_status_service_handler = nullptr;
+
+  srv_registry->acquire("group_replication_status_service_v1",
+                        &gr_status_service_handler);
+  if (nullptr != gr_status_service_handler) {
+    SERVICE_TYPE(group_replication_status_service_v1) *gr_status_service =
+        reinterpret_cast<SERVICE_TYPE(group_replication_status_service_v1) *>(
+            gr_status_service_handler);
+
+    if (gr_status_service
+            ->is_group_in_single_primary_mode_and_im_the_primary() &&
+        !gr_status_service->is_member_online_with_group_majority()) {
+      primary_lost_contact_with_majority = true;
+    }
+
+    srv_registry->release(gr_status_service_handler);
+  }
+
+  return primary_lost_contact_with_majority;
+}
+
 std::tuple<int, bool, bool, std::tuple<std::string, std::string, uint>>
 Source_IO_monitor::get_online_members(
     THD *thd, Master_info *mi, const Mysql_connection *conn,
@@ -896,8 +931,24 @@ Source_IO_monitor::get_online_members(
 }
 
 int Source_IO_monitor::sync_senders_details(THD *thd) {
-  std::vector<std::string> channels;
+  bool primary_lost_contact_with_majority =
+      has_primary_lost_contact_with_majority();
 
+  if (primary_lost_contact_with_majority) {
+    /* Log the warning only once per majority loss. */
+    if (!m_primary_lost_contact_with_majority_warning_logged) {
+      m_primary_lost_contact_with_majority_warning_logged = true;
+      LogErr(WARNING_LEVEL, ER_GRP_RPL_FAILOVER_PRIMARY_WITHOUT_MAJORITY);
+    }
+    return 0;
+  } else {
+    if (m_primary_lost_contact_with_majority_warning_logged) {
+      m_primary_lost_contact_with_majority_warning_logged = false;
+      LogErr(WARNING_LEVEL, ER_GRP_RPL_FAILOVER_PRIMARY_BACK_TO_MAJORITY);
+    }
+  }
+
+  std::vector<std::string> channels;
   channel_map.rdlock();
   for (mi_map::iterator it = channel_map.begin(); it != channel_map.end();
        it++) {
@@ -976,8 +1027,6 @@ Source_IO_monitor::get_senders_details(const std::string &channel_name) {
 }
 
 int Source_IO_monitor::terminate_monitoring_process() {
-  if (!m_monitor_thd_initiated) return 0;
-
   mysql_mutex_lock(&m_run_lock);
 
   if (m_monitor_thd_state.is_thread_dead()) {
@@ -988,7 +1037,7 @@ int Source_IO_monitor::terminate_monitoring_process() {
   // Awake up possible stuck conditions
   mysql_cond_broadcast(&m_run_cond);
 
-  ulong stop_wait_timeout = rpl_stop_slave_timeout;
+  ulong stop_wait_timeout = rpl_stop_replica_timeout;
   while (m_monitor_thd_state.is_thread_alive()) {
     DBUG_PRINT("sleep",
                ("Waiting for the Monitoring IO process thread to finish"));
@@ -1022,7 +1071,6 @@ int Source_IO_monitor::terminate_monitoring_process() {
   assert(m_monitor_thd_state.is_thread_dead());
 
   mysql_mutex_unlock(&m_run_lock);
-  cleanup_mutex();
   return 0;
 }
 
@@ -1036,9 +1084,8 @@ bool Source_IO_monitor::is_monitoring_process_running() {
   return m_monitor_thd_state.is_thread_alive();
 }
 
-Source_IO_monitor &Source_IO_monitor::get_instance() {
-  static Source_IO_monitor shared_instance;
-  return shared_instance;
+Source_IO_monitor *Source_IO_monitor::get_instance() {
+  return rpl_source_io_monitor;
 }
 
 static bool restart_io_thread(THD *thd, const std::string &channel_name,
@@ -1054,7 +1101,8 @@ static bool restart_io_thread(THD *thd, const std::string &channel_name,
   }
 
   if (Async_conn_failover_manager::do_auto_conn_failover(
-          mi, force_sender_with_highest_weight)) {
+          mi, force_sender_with_highest_weight) !=
+      Async_conn_failover_manager::DoAutoConnFailoverError::no_error) {
     LogErr(WARNING_LEVEL, ER_RPL_REPLICA_MONITOR_IO_THREAD_RECONNECT_CHANNEL,
            "choosing the source for", channel_name.c_str());
     channel_map.unlock();
@@ -1078,7 +1126,7 @@ static bool restart_io_thread(THD *thd, const std::string &channel_name,
   thread_mask |= SLAVE_IO;
   thd->set_skip_readonly_check();
 
-  if (terminate_slave_threads(mi, thread_mask, rpl_stop_slave_timeout,
+  if (terminate_slave_threads(mi, thread_mask, rpl_stop_replica_timeout,
                               false /*need_lock_term=false*/)) {
     LogErr(WARNING_LEVEL, ER_RPL_REPLICA_MONITOR_IO_THREAD_RECONNECT_CHANNEL,
            "stopping", channel_name.c_str());
