@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2002, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2002, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,6 +26,7 @@
 #include "sql/sp.h"
 
 #include <string.h>
+
 #include <algorithm>
 #include <atomic>
 #include <memory>
@@ -43,9 +45,9 @@
 #include "my_sqlcommand.h"
 #include "my_sys.h"
 #include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/components/services/bits/psi_statement_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
-#include "mysql/components/services/psi_statement_bits.h"
 #include "mysql/psi/mysql_sp.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
@@ -103,14 +105,12 @@ class sp_rcontext;
 /* Used in error handling only */
 #define SP_TYPE_STRING(type) \
   (type == enum_sp_type::FUNCTION ? "FUNCTION" : "PROCEDURE")
-static bool create_string(THD *thd, String *buf, enum_sp_type sp_type,
-                          const char *db, size_t dblen, const char *name,
-                          size_t namelen, const char *params, size_t paramslen,
-                          const char *returns, size_t returnslen,
-                          const char *body, size_t bodylen,
-                          st_sp_chistics *chistics,
-                          const LEX_CSTRING &definer_user,
-                          const LEX_CSTRING &definer_host, sql_mode_t sql_mode);
+static bool create_string(
+    THD *thd, String *buf, enum_sp_type sp_type, const char *db, size_t dblen,
+    const char *name, size_t namelen, const char *params, size_t paramslen,
+    const char *returns, size_t returnslen, const char *body, size_t bodylen,
+    st_sp_chistics *chistics, const LEX_CSTRING &definer_user,
+    const LEX_CSTRING &definer_host, sql_mode_t sql_mode, bool if_not_exists);
 
 /**************************************************************************
   Fetch stored routines and events creation_ctx for upgrade.
@@ -521,7 +521,7 @@ enum_sp_return_code db_load_routine(
 
   if (!create_string(thd, &defstr, type, nullptr, 0, sp_name, sp_name_len,
                      params, strlen(params), returns, strlen(returns), body,
-                     strlen(body), sp_chistics, user, host, sql_mode)) {
+                     strlen(body), sp_chistics, user, host, sql_mode, false)) {
     ret = SP_INTERNAL_ERROR;
     goto end;
   }
@@ -590,19 +590,22 @@ end:
 }
 
 /**
-  Precheck for create routine statement.
+  Method to check if routine with same name already exists.
 
-  @param  thd      Thread context.
-  @param  sp       Stored routine object to store.
+  @param      thd              Thread context.
+  @param      sp               Stored routine object to store.
+  @param      if_not_exists    True if 'IF NOT EXISTS' clause was specified.
+  @param[out] already_exists   Set to true if routine already exists.
 
-  @retval  false   Success.
-  @retval  true    Error.
+  @retval     false            Success.
+  @retval     true             Error.
 */
+static bool check_routine_already_exists(THD *thd, sp_head *sp,
+                                         bool if_not_exists,
+                                         bool &already_exists) {
+  assert(!already_exists);
 
-static bool create_routine_precheck(THD *thd, sp_head *sp) {
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
-
-  // Check if routine with same name exists.
   bool error;
   const dd::Routine *sr;
   if (sp->m_type == enum_sp_type::FUNCTION)
@@ -615,12 +618,35 @@ static bool create_routine_precheck(THD *thd, sp_head *sp) {
     // Error is reported by DD API framework.
     return true;
   }
-  if (sr != nullptr) {
-    my_error(ER_SP_ALREADY_EXISTS, MYF(0), SP_TYPE_STRING(sp->m_type),
-             sp->m_name.str);
-    return true;
+  if (sr == nullptr) {
+    // Routine with same name does not exist.
+    return false;
   }
 
+  already_exists = true;
+  if (if_not_exists) {
+    push_warning_printf(thd, Sql_condition::SL_NOTE, ER_SP_ALREADY_EXISTS,
+                        ER_THD(thd, ER_SP_ALREADY_EXISTS),
+                        SP_TYPE_STRING(sp->m_type), sp->m_name.str);
+    return false;
+  }
+
+  my_error(ER_SP_ALREADY_EXISTS, MYF(0), SP_TYPE_STRING(sp->m_type),
+           sp->m_name.str);
+  return true;
+}
+
+/**
+  Precheck for create routine statement.
+
+  @param   thd     Thread context.
+  @param   sp      Stored routine object to store.
+
+  @retval  false   Success.
+  @retval  true    Error.
+*/
+
+static bool create_routine_precheck(THD *thd, sp_head *sp) {
   /*
     Check if stored function creation is allowed only to the users having SUPER
     privileges.
@@ -659,15 +685,29 @@ static bool create_routine_precheck(THD *thd, sp_head *sp) {
   }
 
   // Validate body definition to avoid invalid UTF8 characters.
-  if (is_invalid_string(sp->m_body_utf8, system_charset_info)) return true;
+  std::string invalid_sub_str;
+  if (is_invalid_string(sp->m_body_utf8, system_charset_info,
+                        invalid_sub_str)) {
+    // Provide contextual information
+    my_error(ER_DEFINITION_CONTAINS_INVALID_STRING, MYF(0), "stored routine",
+             sp->m_db.str, sp->m_name.str, system_charset_info->csname,
+             invalid_sub_str.c_str());
+    return true;
+  }
 
   // Validate routine comment.
   if (sp->m_chistics->comment.length) {
     // validate comment string to avoid invalid utf8 characters.
     if (is_invalid_string(LEX_CSTRING{sp->m_chistics->comment.str,
                                       sp->m_chistics->comment.length},
-                          system_charset_info))
+                          system_charset_info, invalid_sub_str)) {
+      // Provide contextual information
+      my_error(ER_COMMENT_CONTAINS_INVALID_STRING, MYF(0), "stored routine",
+               (std::string(sp->m_db.str) + "." + std::string(sp->m_name.str))
+                   .c_str(),
+               system_charset_info->csname, invalid_sub_str.c_str());
       return true;
+    }
 
     // Check comment string length.
     if (check_string_char_length(
@@ -682,6 +722,53 @@ static bool create_routine_precheck(THD *thd, sp_head *sp) {
   return false;
 }
 
+/**
+  Method to log create routine event to binlog.
+
+  @param      thd              Thread context.
+  @param      sp               Stored routine object to store.
+  @param      definer          Definer of the routine.
+  @param      if_not_exists    True if 'IF NOT EXISTS' clause was specified.
+  @param      already_exists   True if routine already exists.
+
+  @retval false success
+  @retval true  error
+*/
+static bool sp_binlog_create_routine_stmt(THD *thd, sp_head *sp,
+                                          const LEX_USER *definer,
+                                          bool if_not_exists,
+                                          bool already_exists) {
+  String log_query;
+  log_query.set_charset(system_charset_info);
+
+  String retstr(64);
+  retstr.set_charset(system_charset_info);
+  if (sp->m_type == enum_sp_type::FUNCTION) sp->returns_type(thd, &retstr);
+
+  if (!create_string(thd, &log_query, sp->m_type,
+                     (sp->m_explicit_name ? sp->m_db.str : nullptr),
+                     (sp->m_explicit_name ? sp->m_db.length : 0),
+                     sp->m_name.str, sp->m_name.length, sp->m_params.str,
+                     sp->m_params.length, retstr.c_ptr(), retstr.length(),
+                     sp->m_body.str, sp->m_body.length, sp->m_chistics,
+                     definer->user, definer->host, thd->variables.sql_mode,
+                     if_not_exists))
+    return true;
+
+  thd->add_to_binlog_accessed_dbs(sp->m_db.str);
+
+  /*
+    This statement will be replicated as a statement, even when using
+    row-based replication.
+  */
+  Save_and_Restore_binlog_format_state binlog_format_state(thd);
+  if (write_bin_log(thd, true, log_query.c_ptr(), log_query.length(),
+                    !already_exists))
+    return true;
+
+  return false;
+}
+
 /**
   Creates a stored routine.
 
@@ -696,21 +783,25 @@ static bool create_routine_precheck(THD *thd, sp_head *sp) {
     In case of crash, there won't be any discrepancy between
     the data-dictionary table and the binary log.
 
-  @param thd     Thread context.
-  @param sp      Stored routine object to store.
-  @param definer Definer of the SP.
+  @param       thd                Thread context.
+  @param       sp                 Stored routine object to store.
+  @param       definer            Definer of the SP.
+  @param       if_not_exists      True if 'IF NOT EXISTS' clause was specified.
+  @param[out]  sp_already_exists  Set to true if routine already exists.
 
-  @retval false success
-  @retval true  error
+  @retval      false              Success.
+  @retval      true               Error.
 */
 
-bool sp_create_routine(THD *thd, sp_head *sp, const LEX_USER *definer) {
+bool sp_create_routine(THD *thd, sp_head *sp, const LEX_USER *definer,
+                       bool if_not_exists, bool &sp_already_exists) {
   DBUG_TRACE;
   DBUG_PRINT("enter", ("type: %d  name: %.*s", static_cast<int>(sp->m_type),
                        static_cast<int>(sp->m_name.length), sp->m_name.str));
 
   assert(sp->m_type == enum_sp_type::PROCEDURE ||
          sp->m_type == enum_sp_type::FUNCTION);
+  assert(!sp_already_exists);
 
   /* Grab an exclusive MDL lock. */
   MDL_key::enum_mdl_namespace mdl_type = (sp->m_type == enum_sp_type::FUNCTION)
@@ -723,6 +814,28 @@ bool sp_create_routine(THD *thd, sp_head *sp, const LEX_USER *definer) {
   }
   DEBUG_SYNC(thd, "after_acquiring_mdl_lock_on_routine");
 
+  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::Schema *schema = nullptr;
+
+  // Check whether routine with same name already exists.
+  if (check_routine_already_exists(thd, sp, if_not_exists, sp_already_exists)) {
+    /* If this happens, an error should have been reported. */
+    return true;
+  }
+  if (sp_already_exists) {
+    assert(if_not_exists);
+    /*
+      Routine with same name exists, warning is already reported. Log
+      create routine event to binlog.
+    */
+    if (mysql_bin_log.is_open() &&
+        sp_binlog_create_routine_stmt(thd, sp, definer, if_not_exists,
+                                      sp_already_exists))
+      goto err_with_rollback;
+
+    return false;
+  }
+
   if (create_routine_precheck(thd, sp)) {
     /* If this happens, an error should have been reported. */
     return true;
@@ -732,8 +845,6 @@ bool sp_create_routine(THD *thd, sp_head *sp, const LEX_USER *definer) {
                   DBUG_SET("+d,fail_while_acquiring_dd_object"););
 
   // Check that a database with this name exists.
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
-  const dd::Schema *schema = nullptr;
   if (thd->dd_client()->acquire(sp->m_db.str, &schema)) {
     DBUG_EXECUTE_IF("fail_while_acquiring_routine_schema_obj",
                     DBUG_SET("-d,fail_while_acquiring_dd_object"););
@@ -760,34 +871,9 @@ bool sp_create_routine(THD *thd, sp_head *sp, const LEX_USER *definer) {
   }
 
   // Log stored routine create event.
-  if (mysql_bin_log.is_open()) {
-    String log_query;
-    log_query.set_charset(system_charset_info);
-
-    String retstr(64);
-    retstr.set_charset(system_charset_info);
-    if (sp->m_type == enum_sp_type::FUNCTION) sp->returns_type(thd, &retstr);
-
-    if (!create_string(thd, &log_query, sp->m_type,
-                       (sp->m_explicit_name ? sp->m_db.str : nullptr),
-                       (sp->m_explicit_name ? sp->m_db.length : 0),
-                       sp->m_name.str, sp->m_name.length, sp->m_params.str,
-                       sp->m_params.length, retstr.c_ptr(), retstr.length(),
-                       sp->m_body.str, sp->m_body.length, sp->m_chistics,
-                       definer->user, definer->host, thd->variables.sql_mode))
-      goto err_report_with_rollback;
-
-    thd->add_to_binlog_accessed_dbs(sp->m_db.str);
-
-    /*
-      This statement will be replicated as a statement, even when using
-      row-based replication.
-    */
-    Save_and_Restore_binlog_format_state binlog_format_state(thd);
-
-    if (write_bin_log(thd, true, log_query.c_ptr(), log_query.length(), true))
-      goto err_report_with_rollback;
-  }
+  if (mysql_bin_log.is_open() &&
+      sp_binlog_create_routine_stmt(thd, sp, definer, if_not_exists, false))
+    goto err_report_with_rollback;
 
   // Commit changes to the data-dictionary and binary log.
   if (DBUG_EVALUATE_IF("simulate_create_routine_failure", true, false) ||
@@ -1027,7 +1113,18 @@ bool sp_update_routine(THD *thd, enum_sp_type type, sp_name *name,
   // Validate routine comment.
   if (chistics->comment.str) {
     // validate comment string to invalid utf8 characters.
-    if (is_invalid_string(chistics->comment, system_charset_info)) return true;
+    std::string invalid_sub_str;
+    if (is_invalid_string(
+            LEX_CSTRING{chistics->comment.str, chistics->comment.length},
+            system_charset_info, invalid_sub_str)) {
+      // Provide contextual information
+      my_error(
+          ER_COMMENT_CONTAINS_INVALID_STRING, MYF(0), "stored routine",
+          (std::string(name->m_db.str) + "." + std::string(name->m_name.str))
+              .c_str(),
+          system_charset_info->csname, invalid_sub_str.c_str());
+      return true;
+    }
 
     // Check comment string length.
     if (check_string_char_length(
@@ -1083,13 +1180,14 @@ err_report_with_rollback:
 bool lock_db_routines(THD *thd, const dd::Schema &schema) {
   DBUG_TRACE;
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  // Vectors for names of stored functions and procedures of the schema.
+  std::vector<dd::String_type> func_names, proc_names;
 
-  // Vector for the stored routines of the schema.
-  std::vector<const dd::Routine *> routines;
-
-  // Fetch stored routines of the schema.
-  if (thd->dd_client()->fetch_schema_components(&schema, &routines))
+  // Fetch names of stored functions and procedures of the schema.
+  if (thd->dd_client()->fetch_schema_component_names<dd::Function>(
+          &schema, &func_names) ||
+      thd->dd_client()->fetch_schema_component_names<dd::Procedure>(
+          &schema, &proc_names))
     return true;
 
   /*
@@ -1103,24 +1201,31 @@ bool lock_db_routines(THD *thd, const dd::Schema &schema) {
     my_casedn_str(system_charset_info, schema_name_buf);
     schema_name = schema_name_buf;
   }
+  const dd::String_type schema_name_str(schema_name);
+
+  /*
+    Ensure that we don't hold memory used by MDL_requests after locks have
+    been acquired. This reduces memory usage in cases when we have DROP
+    DATABASE that needs to drop lots of different objects.
+  */
+  MEM_ROOT mdl_reqs_root(key_memory_rm_db_mdl_reqs_root, MEM_ROOT_BLOCK_SIZE);
 
   MDL_request_list mdl_requests;
-  for (const dd::Routine *routine : routines) {
-    MDL_key mdl_key;
 
-    if (is_dd_routine_type_function(routine))
-      dd::Function::create_mdl_key(dd::String_type(schema_name),
-                                   routine->name(), &mdl_key);
-    else
-      dd::Procedure::create_mdl_key(dd::String_type(schema_name),
-                                    routine->name(), &mdl_key);
+  auto add_requests_for_names = [&](dd::Routine::enum_routine_type type,
+                                    const std::vector<dd::String_type> &names) {
+    for (const dd::String_type &name : names) {
+      MDL_key mdl_key;
+      dd::Routine::create_mdl_key(type, schema_name_str, name, &mdl_key);
+      MDL_request *mdl_request = new (&mdl_reqs_root) MDL_request;
+      MDL_REQUEST_INIT_BY_KEY(mdl_request, &mdl_key, MDL_EXCLUSIVE,
+                              MDL_TRANSACTION);
+      mdl_requests.push_front(mdl_request);
+    }
+  };
 
-    // Add MDL_request for routine to mdl_requests list.
-    MDL_request *mdl_request = new (thd->mem_root) MDL_request;
-    MDL_REQUEST_INIT_BY_KEY(mdl_request, &mdl_key, MDL_EXCLUSIVE,
-                            MDL_TRANSACTION);
-    mdl_requests.push_front(mdl_request);
-  }
+  add_requests_for_names(dd::Routine::RT_FUNCTION, func_names);
+  add_requests_for_names(dd::Routine::RT_PROCEDURE, proc_names);
 
   return thd->mdl_context.acquire_locks(&mdl_requests,
                                         thd->variables.lock_wait_timeout);
@@ -1132,64 +1237,89 @@ bool lock_db_routines(THD *thd, const dd::Schema &schema) {
   @param   thd         Thread context.
   @param   schema      Schema object.
 
-  @retval  SP_OK       Success
-  @retval  non-SP_OK   Error (Other constants are used to indicate errors)
+  @retval  false       Success
+  @retval  true        Error
 */
 
-enum_sp_return_code sp_drop_db_routines(THD *thd, const dd::Schema &schema) {
+bool sp_drop_db_routines(THD *thd, const dd::Schema &schema) {
   DBUG_TRACE;
 
   bool is_routine_dropped = false;
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  // Vectors for names of stored functions and procedures of the schema.
+  std::vector<dd::String_type> func_names, proc_names;
 
-  // Vector for the stored routines of the schema.
-  std::vector<const dd::Routine *> routines;
+  // Fetch names of stored functions and procedures of the schema.
+  if (thd->dd_client()->fetch_schema_component_names<dd::Function>(
+          &schema, &func_names) ||
+      thd->dd_client()->fetch_schema_component_names<dd::Procedure>(
+          &schema, &proc_names))
+    return true;
 
-  // Fetch stored routines of the schema.
-  if (thd->dd_client()->fetch_schema_components(&schema, &routines))
-    return SP_INTERNAL_ERROR;
+  MEM_ROOT foreach_fn_root(key_memory_rm_table_foreach_root,
+                           MEM_ROOT_BLOCK_SIZE);
 
-  enum_sp_return_code ret_code = SP_OK;
-  for (const dd::Routine *routine : routines) {
-    sp_name name(
-        {schema.name().c_str(), schema.name().length()},
-        {const_cast<char *>(routine->name().c_str()), routine->name().length()},
-        false);
-    enum_sp_type type = is_dd_routine_type_function(routine)
-                            ? enum_sp_type::FUNCTION
-                            : enum_sp_type::PROCEDURE;
+  auto drop_routines_by_names = [&](enum_sp_type type,
+                                    const std::vector<dd::String_type> &names) {
+    for (const dd::String_type &name : names) {
+      dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+      const dd::Routine *routine = nullptr;
 
-    DBUG_EXECUTE_IF("fail_drop_db_routines", {
-      my_error(ER_SP_DROP_FAILED, MYF(0), "ROUTINE", "");
-      return SP_DROP_FAILED;
-    });
+      if (type == enum_sp_type::FUNCTION) {
+        if (thd->dd_client()->acquire<dd::Function>(schema.name().c_str(),
+                                                    name.c_str(), &routine))
+          return true;
+      } else {
+        if (thd->dd_client()->acquire<dd::Procedure>(schema.name().c_str(),
+                                                     name.c_str(), &routine))
+          return true;
+      }
 
-    if (thd->dd_client()->drop(routine)) {
-      ret_code = SP_DROP_FAILED;
-      my_error(ER_SP_DROP_FAILED, MYF(0),
-               is_dd_routine_type_function(routine) ? "FUNCTION" : "PROCEDURE",
-               routine->name().c_str());
-      break;
-    }
+      DBUG_EXECUTE_IF("fail_drop_db_routines", {
+        my_error(ER_SP_DROP_FAILED, MYF(0), "ROUTINE", "");
+        return true;
+      });
 
-    if (type == enum_sp_type::FUNCTION &&
-        update_referencing_views_metadata(thd, &name))
-      return SP_INTERNAL_ERROR;
+      if (routine == nullptr || thd->dd_client()->drop(routine)) {
+        assert(routine != nullptr);
+        my_error(ER_SP_DROP_FAILED, MYF(0),
+                 type == enum_sp_type::FUNCTION ? "FUNCTION" : "PROCEDURE",
+                 name.c_str());
+        return true;
+      }
 
-    is_routine_dropped = true;
+      if (type == enum_sp_type::FUNCTION) {
+        sp_name fn_name({schema.name().c_str(), schema.name().length()},
+                        {const_cast<char *>(name.c_str()), name.length()},
+                        false);
+
+        if (mark_referencing_views_invalid(thd, &fn_name, &foreach_fn_root))
+          return true;
+
+        foreach_fn_root.ClearForReuse();
+      }
+
+      is_routine_dropped = true;
 
 #ifdef HAVE_PSI_SP_INTERFACE
-    /* Drop statistics for this stored routine from performance schema. */
-    MYSQL_DROP_SP(to_uint(type), schema.name().c_str(), schema.name().length(),
-                  routine->name().c_str(), routine->name().length());
+      /* Drop statistics for this stored routine from performance schema. */
+      MYSQL_DROP_SP(to_uint(type), schema.name().c_str(),
+                    schema.name().length(), name.c_str(), name.length());
 #endif
+    }
+
+    return false;
+  };
+
+  if (drop_routines_by_names(enum_sp_type::FUNCTION, func_names) ||
+      drop_routines_by_names(enum_sp_type::PROCEDURE, proc_names)) {
+    return true;
   }
 
   // Invalidate the sp cache.
   if (is_routine_dropped) sp_cache_invalidate();
 
-  return ret_code;
+  return false;
 }
 
 /**
@@ -1241,7 +1371,7 @@ static bool show_create_routine_from_dd_routine(THD *thd, enum_sp_type type,
           routine->definition().length(), &sp_chistics,
           {routine->definer_user().c_str(), routine->definer_user().length()},
           {routine->definer_host().c_str(), routine->definer_host().length()},
-          routine->sql_mode()))
+          routine->sql_mode(), false))
     return true;
 
   // Prepare sql_mode string representation.
@@ -1314,10 +1444,10 @@ static bool show_create_routine_from_dd_routine(THD *thd, enum_sp_type type,
   protocol->store(cs_info->csname, system_charset_info);
   // connection_collation
   cs_info = dd_get_mysql_charset(routine->connection_collation_id());
-  protocol->store(cs_info->name, system_charset_info);
+  protocol->store(cs_info->m_coll_name, system_charset_info);
   // database_collation
   cs_info = dd_get_mysql_charset(routine->schema_collation_id());
-  protocol->store(cs_info->name, system_charset_info);
+  protocol->store(cs_info->m_coll_name, system_charset_info);
 
   bool err_status = protocol->end_row();
 
@@ -1418,7 +1548,7 @@ sp_head *sp_find_routine(THD *thd, enum_sp_type type, sp_name *name,
   if (!cache_only) {
     if (db_find_routine(thd, type, name, &sp) == SP_OK) {
       sp_cache_insert(cp, sp);
-      DBUG_PRINT("info", ("added new: 0x%lx, level: %lu, flags %x", (ulong)sp,
+      DBUG_PRINT("info", ("added new: %p, level: %lu, flags %x", sp,
                           sp->m_recursion_level, sp->m_flags));
     }
   }
@@ -1450,15 +1580,15 @@ sp_head *sp_setup_routine(THD *thd, enum_sp_type type, sp_name *name,
   sp_head *sp = sp_cache_lookup(cp, name);
   if (sp == nullptr) return nullptr;
 
-  DBUG_PRINT("info", ("found: 0x%lx", (ulong)sp));
+  DBUG_PRINT("info", ("found: %p", sp));
 
   const ulong depth = type == enum_sp_type::PROCEDURE
                           ? thd->variables.max_sp_recursion_depth
                           : 0;
 
   if (sp->m_first_free_instance) {
-    DBUG_PRINT("info", ("first free: 0x%lx  level: %lu  flags %x",
-                        (ulong)sp->m_first_free_instance,
+    DBUG_PRINT("info", ("first free: %p  level: %lu  flags %x",
+                        sp->m_first_free_instance,
                         sp->m_first_free_instance->m_recursion_level,
                         sp->m_first_free_instance->m_flags));
     assert(!(sp->m_first_free_instance->m_flags & sp_head::IS_INVOKED));
@@ -1502,7 +1632,7 @@ sp_head *sp_setup_routine(THD *thd, enum_sp_type type, sp_name *name,
   new_sp->m_recursion_level = level;
   new_sp->m_first_instance = sp;
   sp->m_last_cached_sp = sp->m_first_free_instance = new_sp;
-  DBUG_PRINT("info", ("added level: 0x%lx, level: %lu, flags %x", (ulong)new_sp,
+  DBUG_PRINT("info", ("added level: %p, level: %lu, flags %x", new_sp,
                       new_sp->m_recursion_level, new_sp->m_flags));
   return new_sp;
 }
@@ -1520,8 +1650,8 @@ sp_head *sp_setup_routine(THD *thd, enum_sp_type type, sp_name *name,
   @retval true  Not found
 */
 
-bool sp_exist_routines(THD *thd, TABLE_LIST *routines, bool is_proc) {
-  TABLE_LIST *routine;
+bool sp_exist_routines(THD *thd, Table_ref *routines, bool is_proc) {
+  Table_ref *routine;
   bool sp_object_found;
   DBUG_TRACE;
   for (routine = routines; routine; routine = routine->next_global) {
@@ -1550,7 +1680,7 @@ bool sp_exist_routines(THD *thd, TABLE_LIST *routines, bool is_proc) {
 }
 
 /**
-  Auxilary function that adds new element to the set of stored routines
+  Auxiliary function that adds new element to the set of stored routines
   used by statement.
 
   The elements of Query_tables_list::sroutines set are accessed on prepared
@@ -1588,7 +1718,7 @@ static bool sp_add_used_routine(Query_tables_list *prelocking_ctx,
                                 Query_arena *arena, const uchar *key,
                                 size_t key_length, size_t db_length,
                                 const char *name, size_t name_length,
-                                TABLE_LIST *belong_to_view) {
+                                Table_ref *belong_to_view) {
   if (prelocking_ctx->sroutines == nullptr) {
     prelocking_ctx->sroutines.reset(
         new malloc_unordered_map<std::string, Sroutine_hash_entry *>(
@@ -1663,7 +1793,7 @@ bool sp_add_used_routine(Query_tables_list *prelocking_ctx, Query_arena *arena,
                          size_t db_length, const char *name, size_t name_length,
                          bool lowercase_db,
                          Sp_name_normalize_type name_normalize_type,
-                         bool own_routine, TABLE_LIST *belong_to_view) {
+                         bool own_routine, Table_ref *belong_to_view) {
   // Length of routine name components needs to be checked earlier.
   assert(db_length <= NAME_LEN && name_length <= NAME_LEN);
 
@@ -1717,7 +1847,7 @@ bool sp_add_used_routine(Query_tables_list *prelocking_ctx, Query_arena *arena,
 
       /*
         Stored routine names are case and accent insensitive. So for the proper
-        hash key comparision, case and accent is stripped off by replacing the
+        hash key comparison, case and accent is stripped off by replacing the
         characters with their sort weight when preparing the Sroutine_hash_entry
         key.
       */
@@ -1788,7 +1918,7 @@ void sp_remove_not_own_routines(Query_tables_list *prelocking_ctx) {
 void sp_update_stmt_used_routines(
     THD *thd, Query_tables_list *prelocking_ctx,
     malloc_unordered_map<std::string, Sroutine_hash_entry *> *src,
-    TABLE_LIST *belong_to_view) {
+    Table_ref *belong_to_view) {
   for (const auto &key_and_value : *src) {
     Sroutine_hash_entry *rt = key_and_value.second;
     (void)sp_add_used_routine(prelocking_ctx, thd->stmt_arena,
@@ -1814,7 +1944,7 @@ void sp_update_stmt_used_routines(
 
 void sp_update_stmt_used_routines(THD *thd, Query_tables_list *prelocking_ctx,
                                   SQL_I_List<Sroutine_hash_entry> *src,
-                                  TABLE_LIST *belong_to_view) {
+                                  Table_ref *belong_to_view) {
   for (Sroutine_hash_entry *rt = src->first; rt; rt = rt->next)
     (void)sp_add_used_routine(prelocking_ctx, thd->stmt_arena,
                               pointer_cast<const uchar *>(rt->m_key),
@@ -1921,7 +2051,7 @@ enum_sp_return_code sp_cache_routine(THD *thd, enum_sp_type type,
           SP allows full NAME_LEN chars thus he have to allocate enough
           size in bytes. Otherwise there is stack overrun could happen
           if multibyte sequence is `name`. `db` is still safe because the
-          rest of the server checks agains NAME_LEN bytes and not chars.
+          rest of the server checks against NAME_LEN bytes and not chars.
           Hence, the overrun happens only if the name is in length > 32 and
           uses multibyte (cyrillic, greek, etc.)
         */
@@ -1948,7 +2078,7 @@ static bool create_string(
     const char *name, size_t namelen, const char *params, size_t paramslen,
     const char *returns, size_t returnslen, const char *body, size_t bodylen,
     st_sp_chistics *chistics, const LEX_CSTRING &definer_user,
-    const LEX_CSTRING &definer_host, sql_mode_t sql_mode) {
+    const LEX_CSTRING &definer_host, sql_mode_t sql_mode, bool if_not_exists) {
   sql_mode_t old_sql_mode = thd->variables.sql_mode;
   /* Make some room to begin with */
   if (buf->alloc(100 + dblen + 1 + namelen + paramslen + returnslen + bodylen +
@@ -1963,6 +2093,7 @@ static bool create_string(
     buf->append(STRING_WITH_LEN("FUNCTION "));
   else
     buf->append(STRING_WITH_LEN("PROCEDURE "));
+  if (if_not_exists) buf->append(STRING_WITH_LEN("IF NOT EXISTS "));
   if (dblen > 0) {
     append_identifier(thd, buf, db, dblen);
     buf->append('.');
@@ -2004,90 +2135,6 @@ static bool create_string(
   return true;
 }
 
-/**
-  The function loads sp_head struct for information schema purposes
-  (used for I_S ROUTINES & PARAMETERS tables).
-
-  @param[in]      thd               thread handler
-  @param[in]      db_name           DB name.
-  @param[in]      routine           dd::Routine object.
-  @param[out]     free_sp_head      returns 1 if we need to free sp_head struct
-                                    otherwise returns 0
-
-  @return     Pointer on sp_head struct
-    @retval   NULL                  error
-*/
-
-sp_head *sp_load_for_information_schema(THD *thd, LEX_CSTRING db_name,
-                                        const dd::Routine *routine,
-                                        bool *free_sp_head) {
-  sp_head *sp;
-  enum_sp_type type = is_dd_routine_type_function(routine)
-                          ? enum_sp_type::FUNCTION
-                          : enum_sp_type::PROCEDURE;
-  *free_sp_head = false;
-  sp_cache **spc = (type == enum_sp_type::FUNCTION) ? &thd->sp_func_cache
-                                                    : &thd->sp_proc_cache;
-  sp_name sp_name_obj(
-      db_name,
-      {const_cast<char *>(routine->name().c_str()), routine->name().length()},
-      true);
-  sp_name_obj.init_qname(thd);
-
-  if ((sp = sp_cache_lookup(spc, &sp_name_obj))) {
-    return sp;
-  }
-
-  // Create stored program creation context from routine object.
-  Stored_program_creation_ctx *creation_ctx =
-      Stored_routine_creation_ctx::create_routine_creation_ctx(routine);
-  if (creation_ctx == nullptr) return nullptr;
-
-  // Prepare stored routine return type string.
-  dd::String_type return_type_str;
-  prepare_return_type_string_from_dd_routine(thd, routine, &return_type_str);
-
-  // Prepare stored routine parameter's string.
-  dd::String_type params_str;
-  prepare_params_string_from_dd_routine(thd, routine, &params_str);
-
-  // Dummy Routine body.
-  LEX_CSTRING sr_body;
-  if (type == enum_sp_type::FUNCTION)
-    sr_body = {STRING_WITH_LEN("RETURN NULL")};
-  else
-    sr_body = {STRING_WITH_LEN("BEGIN END")};
-
-  // Dummy stored routine definer.
-  const LEX_CSTRING definer_user = EMPTY_CSTR;
-  const LEX_CSTRING definer_host = EMPTY_CSTR;
-
-  // Dummy st_sp_chistics object.
-  struct st_sp_chistics sp_chistics;
-  memset(&sp_chistics, 0, sizeof(st_sp_chistics));
-
-  String defstr;
-  defstr.set_charset(creation_ctx->get_client_cs());
-  if (!create_string(thd, &defstr, type, db_name.str, db_name.length,
-                     routine->name().c_str(), routine->name().length(),
-                     params_str.c_str(), params_str.length(),
-                     return_type_str.c_str(), return_type_str.length(),
-                     sr_body.str, sr_body.length, &sp_chistics, definer_user,
-                     definer_host, routine->sql_mode()))
-    return nullptr;
-
-  LEX *old_lex = thd->lex, newlex;
-  thd->lex = &newlex;
-  newlex.thd = thd;
-  newlex.set_current_query_block(nullptr);
-  sp = sp_compile(thd, &defstr, routine->sql_mode(), creation_ctx);
-  *free_sp_head = true;
-  thd->lex->sphead = nullptr;
-  lex_end(thd->lex);
-  thd->lex = old_lex;
-  return sp;
-}
-
 /**
   Start parsing of a stored program.
 
@@ -2099,7 +2146,7 @@ sp_head *sp_load_for_information_schema(THD *thd, LEX_CSTRING db_name,
 
   @param thd      Thread context.
   @param sp_type  The stored program type
-  @param sp_name  The stored progam name
+  @param sp_name  The stored program name
 
   @return properly initialized sp_head-instance in case of success, or NULL is
   case of out-of-memory error.
@@ -2107,10 +2154,7 @@ sp_head *sp_load_for_information_schema(THD *thd, LEX_CSTRING db_name,
 sp_head *sp_start_parsing(THD *thd, enum_sp_type sp_type, sp_name *sp_name) {
   // The order is important:
   // 1. new sp_head()
-  MEM_ROOT own_root;
-
-  init_sql_alloc(key_memory_sp_head_main_root, &own_root, MEM_ROOT_BLOCK_SIZE,
-                 MEM_ROOT_PREALLOC);
+  MEM_ROOT own_root(key_memory_sp_head_main_root, MEM_ROOT_BLOCK_SIZE);
 
   void *rawmem = own_root.Alloc(sizeof(sp_head));
   if (!rawmem) return nullptr;
@@ -2215,7 +2259,7 @@ uint sp_get_flags_for_command(LEX *lex) {
         flags = 0; /* This is a SELECT with INTO clause */
         break;
       }
-      /* fallthrough */
+      [[fallthrough]];
     case SQLCOM_ANALYZE:
     case SQLCOM_OPTIMIZE:
     case SQLCOM_PRELOAD_KEYS:
@@ -2345,6 +2389,7 @@ uint sp_get_flags_for_command(LEX *lex) {
     case SQLCOM_CREATE_RESOURCE_GROUP:
     case SQLCOM_ALTER_RESOURCE_GROUP:
     case SQLCOM_DROP_RESOURCE_GROUP:
+    case SQLCOM_ALTER_TABLESPACE:
       flags = sp_head::HAS_COMMIT_OR_ROLLBACK;
       break;
     default:
@@ -2505,7 +2550,7 @@ String *sp_get_item_value(THD *thd, Item *item, String *str) {
         return item->val_str(str);
       else { /* Bit type is handled as binary string */
       }
-      // Fall through
+      [[fallthrough]];
     case STRING_RESULT: {
       String *result = item->val_str(str);
 
@@ -2524,7 +2569,7 @@ String *sp_get_item_value(THD *thd, Item *item, String *str) {
         if (cs->escape_with_backslash_is_dangerous) buf.append(' ');
         append_query_string(thd, cs, result, &buf);
         buf.append(" COLLATE '");
-        buf.append(item->collation.collation->name);
+        buf.append(item->collation.collation->m_coll_name);
         buf.append('\'');
         str->copy(buf);
 
