@@ -1,15 +1,16 @@
-/* Copyright (c) 2012, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,6 +42,7 @@
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"  // Prealloced_array
+#include "scope_guard.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"  // check_table_access
 #include "sql/binlog.h"            // mysql_bin_log
@@ -211,7 +213,7 @@ static bool subst_spvars(THD *thd, sp_instr *instr, LEX_CSTRING query_str) {
   qbuf.length(0);
   const char *cur = query_str.str;
   int prev_pos = 0;
-  bool res = 0;
+  bool res = false;
   thd->query_name_consts = 0;
 
   for (Item_splocal **splocal = sp_vars_uses.begin();
@@ -301,6 +303,83 @@ class SP_instr_error_handler : public Internal_error_handler {
   bool cts_table_exists_error = false;
 };
 
+/**
+  Execute an expression (e.g an IF) that is not a complete SQL statement.
+
+  Expressions that may be executed in this function:
+    IF, CASE, DECLARE, SET, RETURN
+
+  @param      thd    thread context
+  @param[out] nextp  next instruction pointer
+
+  @returns false if success, true if error
+*/
+bool sp_lex_instr::execute_expression(THD *thd, uint *nextp) {
+  auto execute_guard = create_scope_guard([&]() {
+    m_lex->cleanup(true);
+    if (thd->in_sub_stmt == 0) {
+      thd->get_stmt_da()->set_overwrite_status(true);
+      thd->is_error() ? trans_rollback_stmt(thd) : trans_commit_stmt(thd);
+      thd->get_stmt_da()->set_overwrite_status(false);
+    }
+    thd_proc_info(thd, "closing tables");
+    close_thread_tables(thd);
+    thd_proc_info(thd, nullptr);
+
+    if (thd->in_sub_stmt == 0) {
+      if (thd->transaction_rollback_request) {
+        trans_rollback_implicit(thd);
+        thd->mdl_context.release_transactional_locks();
+      } else if (!thd->in_multi_stmt_transaction_mode()) {
+        thd->mdl_context.release_transactional_locks();
+      } else {
+        thd->mdl_context.release_statement_locks();
+      }
+    }
+  });
+
+  /*
+    Check privileges for tables in expression, open and lock those tables,
+    bind data to expression so that it is ready for execution.
+
+    Notice that temporary tables must be opened before privilege checking.
+    This is because a session has all privileges for any temporary table that
+    it has created, however a table must be opened in order to identify it as
+    a temporary table.
+  */
+  if (m_lex->query_tables != nullptr) {
+    if (open_temporary_tables(thd, m_lex->query_tables)) {
+      return true;
+    }
+    if (check_table_access(thd, SELECT_ACL, m_lex->query_tables, false,
+                           UINT_MAX, false)) {
+      return true;
+    }
+  }
+  if (open_and_lock_tables(thd, m_lex->query_tables, 0)) {
+    return true;
+  }
+
+  if (m_arena.get_state() != Query_arena::STMT_INITIALIZED_FOR_SP) {
+    m_lex->restore_cmd_properties();
+    bind_fields(m_arena.item_list());
+  }
+  /*
+    Trace the expression. This is not an SQL statement, but pretend it is
+    a SELECT query expression.
+  */
+  Opt_trace_start ots(thd, m_lex->query_tables, SQLCOM_SELECT, &m_lex->var_list,
+                      nullptr, 0, this, thd->variables.character_set_client);
+  const Opt_trace_object trace_command(&thd->opt_trace);
+  const Opt_trace_array trace_command_steps(&thd->opt_trace, "steps");
+
+  if (exec_core(thd, nextp)) {
+    return true;
+  }
+
+  return false;
+}
+
 bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
                                            bool open_tables) {
   /*
@@ -357,7 +436,7 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
 
   /*
     In case a session state exists do not cache the SELECT stmt. If we
-    cache SELECT statment when session state information exists, then
+    cache SELECT statement when session state information exists, then
     the result sets of this SELECT are cached which contains changed
     session information. Next time when same query is executed when there
     is no change in session state, then result sets are picked from cache
@@ -376,66 +455,8 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
 
   if (!error) {
     if (open_tables) {
-      // todo: break this block out into a separate function.
-      /*
-        IF, CASE, DECLARE, SET, RETURN, have 'open_tables' true; they may
-        have a subquery in parameter and are worth tracing. They don't
-        correspond to a SQL command so we pretend that they are SQLCOM_SELECT.
-      */
-      Opt_trace_start ots(thd, m_lex->query_tables, SQLCOM_SELECT,
-                          &m_lex->var_list, nullptr, 0, this,
-                          thd->variables.character_set_client);
-      Opt_trace_object trace_command(&thd->opt_trace);
-      Opt_trace_array trace_command_steps(&thd->opt_trace, "steps");
-
-      /*
-        Check whenever we have access to tables for this statement
-        and open and lock them before executing instructions core function.
-        If we are not opening any tables, we don't need to check permissions
-        either.
-      */
-      if (m_lex->query_tables)
-        error = (open_temporary_tables(thd, m_lex->query_tables) ||
-                 check_table_access(thd, SELECT_ACL, m_lex->query_tables, false,
-                                    UINT_MAX, false));
-
-      if (!error) error = open_and_lock_tables(thd, m_lex->query_tables, 0);
-
-      if (!error) {
-        m_lex->restore_cmd_properties();
-        bind_fields(m_arena.item_list());
-
-        error = exec_core(thd, nextp);
-        DBUG_PRINT("info", ("exec_core returned: %d", error));
-      }
-
-      /*
-        Call after unit->cleanup() to close open table
-        key read.
-      */
-
-      m_lex->cleanup(thd, true);
-
-      /* Here we also commit or rollback the current statement. */
-
-      if (!thd->in_sub_stmt) {
-        thd->get_stmt_da()->set_overwrite_status(true);
-        thd->is_error() ? trans_rollback_stmt(thd) : trans_commit_stmt(thd);
-        thd->get_stmt_da()->set_overwrite_status(false);
-      }
-      thd_proc_info(thd, "closing tables");
-      close_thread_tables(thd);
-      thd_proc_info(thd, nullptr);
-
-      if (!thd->in_sub_stmt) {
-        if (thd->transaction_rollback_request) {
-          trans_rollback_implicit(thd);
-          thd->mdl_context.release_transactional_locks();
-        } else if (!thd->in_multi_stmt_transaction_mode())
-          thd->mdl_context.release_transactional_locks();
-        else
-          thd->mdl_context.release_statement_locks();
-      }
+      error = execute_expression(thd, nextp);
+      DBUG_PRINT("info", ("exec_expression returned: %d", error));
     } else {
       DEBUG_SYNC(thd, "sp_lex_instr_before_exec_core");
       error = exec_core(thd, nextp);
@@ -476,14 +497,14 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
     statement. To make sure that items are created in the statement mem_root,
     change state to STMT_INITIALIZED_FOR_SP.
 
-    When a "table exists" error occur for CREATE TABLE ... SELECT change state
+    When a "table exists" error occurs for CREATE TABLE ... SELECT change state
     to STMT_INITIALIZED_FOR_SP, as if statement must be reprepared.
 
       Why is this necessary? A useful pointer would be to note how
       PREPARE/EXECUTE uses functions like select_like_stmt_test to implement
       CREATE TABLE .... SELECT. The SELECT part of the DDL is resolved first.
       Then there is an attempt to create the table. So in the execution phase,
-      if "table exists" error occurs or flush table preceeds the execute, the
+      if "table exists" error occurs or flush table precedes the execute, the
       item tree of the select is re-created and followed by an attempt to create
       the table.
 
@@ -565,9 +586,7 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
   cleanup_before_parsing(thd);
 
   // Cleanup and re-init the lex mem_root for re-parse.
-  free_root(&m_lex_mem_root, MYF(0));
-  init_sql_alloc(PSI_NOT_INSTRUMENTED, &m_lex_mem_root, MEM_ROOT_BLOCK_SIZE,
-                 MEM_ROOT_PREALLOC);
+  m_lex_mem_root.ClearForReuse();
 
   /*
     Switch mem-roots. We store the new LEX and its Items in the
@@ -595,7 +614,7 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
   Item *execution_item_list = thd->item_list();
   thd->reset_item_list();
 
-  // Create a new LEX and intialize it.
+  // Create a new LEX and initialize it.
 
   LEX *lex_saved = thd->lex;
 
@@ -682,8 +701,6 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
 bool sp_lex_instr::validate_lex_and_execute_core(THD *thd, uint *nextp,
                                                  bool open_tables) {
   Reprepare_observer reprepare_observer;
-  int reprepare_attempt = 0;
-  const int MAX_REPREPARE_ATTEMPTS = 3;
 
   while (true) {
     DBUG_EXECUTE_IF("simulate_bug18831513", { invalidate(); });
@@ -745,17 +762,15 @@ bool sp_lex_instr::validate_lex_and_execute_core(THD *thd, uint *nextp,
           raise it to the user;
     */
     if (stmt_reprepare_observer == nullptr || thd->is_fatal_error() ||
-        thd->killed || thd->get_stmt_da()->mysql_errno() != ER_NEED_REPREPARE)
+        thd->killed || thd->get_stmt_da()->mysql_errno() != ER_NEED_REPREPARE) {
       return true;
-
+    }
     /*
-      We take only 3 attempts to reprepare the query, otherwise we might end
-      up in the endless loop.
+      Reprepare_observer ensures that the statement is retried a maximum number
+      of times, to avoid an endless loop.
     */
     assert(stmt_reprepare_observer->is_invalidated());
-    if ((reprepare_attempt++ >= MAX_REPREPARE_ATTEMPTS) ||
-        DBUG_EVALUATE_IF("simulate_max_reprepare_attempts_hit_case", true,
-                         false)) {
+    if (!stmt_reprepare_observer->can_retry()) {
       /*
         Reprepare_observer sets error status in DA but Sql_condition is not
         added. Please check Reprepare_observer::report_error(). Pushing
@@ -827,7 +842,8 @@ void sp_lex_instr::get_query(String *sql_query) const {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_stmt::psi_info = {0, "stmt", 0, PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_stmt::psi_info = {0, "stmt", 0,
+                                              "Stored Program: SQL statement"};
 #endif
 
 bool sp_instr_stmt::execute(THD *thd, uint *nextp) {
@@ -915,7 +931,7 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp) {
       and therefore pass in a null-pointer instead of a pointer to
       state at the beginning of execution.
     */
-    log_slow_do(thd, nullptr);
+    log_slow_do(thd);
   }
 
   /*
@@ -983,7 +999,9 @@ bool sp_instr_stmt::exec_core(THD *thd, uint *nextp) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_set::psi_info = {0, "set", 0, PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_set::psi_info = {
+    0, "set", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: SET statement"};
 #endif
 
 bool sp_instr_set::exec_core(THD *thd, uint *nextp) {
@@ -1026,7 +1044,8 @@ void sp_instr_set::print(const THD *thd, String *str) {
 
 #ifdef HAVE_PSI_INTERFACE
 PSI_statement_info sp_instr_set_trigger_field::psi_info = {
-    0, "set_trigger_field", 0, PSI_DOCUMENT_ME};
+    0, "set_trigger_field", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: SET NEW.<field> in TRIGGER"};
 #endif
 
 bool sp_instr_set_trigger_field::exec_core(THD *thd, uint *nextp) {
@@ -1086,7 +1105,9 @@ void sp_instr_set_trigger_field::cleanup_before_parsing(THD *thd) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_jump::psi_info = {0, "jump", 0, PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_jump::psi_info = {
+    0, "jump", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: jump microcode instruction"};
 #endif
 
 void sp_instr_jump::print(const THD *, String *str) {
@@ -1132,8 +1153,9 @@ void sp_instr_jump::opt_move(uint dst, List<sp_branch_instr> *bp) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_jump_if_not::psi_info = {0, "jump_if_not", 0,
-                                                     PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_jump_if_not::psi_info = {
+    0, "jump_if_not", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: jump if false microcode instruction"};
 #endif
 
 bool sp_instr_jump_if_not::exec_core(THD *thd, uint *nextp) {
@@ -1214,8 +1236,9 @@ void sp_lex_branch_instr::opt_move(uint dst, List<sp_branch_instr> *bp) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_jump_case_when::psi_info = {0, "jump_case_when", 0,
-                                                        PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_jump_case_when::psi_info = {
+    0, "jump_case_when", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: jump CASE WHEN microcode instruction"};
 #endif
 
 bool sp_instr_jump_case_when::exec_core(THD *thd, uint *nextp) {
@@ -1285,8 +1308,9 @@ bool sp_instr_jump_case_when::on_after_expr_parsing(THD *thd) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_freturn::psi_info = {0, "freturn", 0,
-                                                 PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_freturn::psi_info = {
+    0, "freturn", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: RETURN from STORED FUNCTION"};
 #endif
 
 bool sp_instr_freturn::exec_core(THD *thd, uint *nextp) {
@@ -1323,8 +1347,9 @@ void sp_instr_freturn::print(const THD *thd, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_hpush_jump::psi_info = {0, "hpush_jump", 0,
-                                                    PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_hpush_jump::psi_info = {
+    0, "hpush_jump", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: install a DECLARE HANDLER microcode instruction"};
 #endif
 
 sp_instr_hpush_jump::sp_instr_hpush_jump(uint ip, sp_pcontext *ctx,
@@ -1398,7 +1423,9 @@ uint sp_instr_hpush_jump::opt_mark(sp_head *sp, List<sp_instr> *leads) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_hpop::psi_info = {0, "hpop", 0, PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_hpop::psi_info = {
+    0, "hpop", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: uninstall a DECLARE HANDLER microcode instruction"};
 #endif
 
 bool sp_instr_hpop::execute(THD *thd, uint *nextp) {
@@ -1412,8 +1439,9 @@ bool sp_instr_hpop::execute(THD *thd, uint *nextp) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_hreturn::psi_info = {0, "hreturn", 0,
-                                                 PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_hreturn::psi_info = {
+    0, "hreturn", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: return from a DECLARE HANDLER microcode instruction"};
 #endif
 
 sp_instr_hreturn::sp_instr_hreturn(uint ip, sp_pcontext *ctx)
@@ -1476,7 +1504,9 @@ uint sp_instr_hreturn::opt_mark(sp_head *, List<sp_instr> *) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_cpush::psi_info = {0, "cpush", 0, PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_cpush::psi_info = {
+    0, "cpush", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: install a DECLARE CURSOR microcode instruction"};
 #endif
 
 bool sp_instr_cpush::execute(THD *thd, uint *nextp) {
@@ -1519,7 +1549,9 @@ void sp_instr_cpush::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_cpop::psi_info = {0, "cpop", 0, PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_cpop::psi_info = {
+    0, "cpop", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: uninstall a DECLARE CURSOR microcode instruction"};
 #endif
 
 bool sp_instr_cpop::execute(THD *thd, uint *nextp) {
@@ -1541,7 +1573,9 @@ void sp_instr_cpop::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_copen::psi_info = {0, "copen", 0, PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_copen::psi_info = {
+    0, "copen", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: OPEN cursor"};
 #endif
 
 bool sp_instr_copen::execute(THD *thd, uint *nextp) {
@@ -1606,8 +1640,9 @@ void sp_instr_copen::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_cclose::psi_info = {0, "cclose", 0,
-                                                PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_cclose::psi_info = {
+    0, "cclose", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: CLOSE cursor"};
 #endif
 
 bool sp_instr_cclose::execute(THD *thd, uint *nextp) {
@@ -1642,8 +1677,9 @@ void sp_instr_cclose::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_cfetch::psi_info = {0, "cfetch", 0,
-                                                PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_cfetch::psi_info = {
+    0, "cfetch", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: FETCH cursor"};
 #endif
 
 bool sp_instr_cfetch::execute(THD *thd, uint *nextp) {
@@ -1687,7 +1723,9 @@ void sp_instr_cfetch::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_error::psi_info = {0, "error", 0, PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_error::psi_info = {
+    0, "error", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: CASE WHEN not found microcode instruction"};
 #endif
 
 void sp_instr_error::print(const THD *, String *str) {
@@ -1702,8 +1740,9 @@ void sp_instr_error::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_set_case_expr::psi_info = {0, "set_case_expr", 0,
-                                                       PSI_DOCUMENT_ME};
+PSI_statement_info sp_instr_set_case_expr::psi_info = {
+    0, "set_case_expr", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
+    "Stored Program: CASE microcode instruction"};
 #endif
 
 bool sp_instr_set_case_expr::exec_core(THD *thd, uint *nextp) {
@@ -1711,16 +1750,16 @@ bool sp_instr_set_case_expr::exec_core(THD *thd, uint *nextp) {
 
   sp_rcontext *rctx = thd->sp_runtime_ctx;
 
-  if (rctx->set_case_expr(thd, m_case_expr_id, &m_expr_item) &&
-      !rctx->get_case_expr(m_case_expr_id)) {
-    // Failed to evaluate the value, the case expression is still not
-    // initialized. Set to NULL so we can continue.
+  if (rctx->set_case_expr(thd, m_case_expr_id, &m_expr_item)) {
+    if (!rctx->get_case_expr(m_case_expr_id)) {
+      // Failed to evaluate the value, the case expression is still not
+      // initialized. Set to NULL so we can continue.
+      Item *null_item = new Item_null();
 
-    Item *null_item = new Item_null();
-
-    if (!null_item || rctx->set_case_expr(thd, m_case_expr_id, &null_item)) {
-      // If this also failed, we have to abort.
-      my_error(ER_OUT_OF_RESOURCES, MYF(ME_FATALERROR));
+      if (!null_item || rctx->set_case_expr(thd, m_case_expr_id, &null_item)) {
+        // If this also failed, we have to abort.
+        my_error(ER_OUT_OF_RESOURCES, MYF(ME_FATALERROR));
+      }
     }
 
     return true;
