@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +42,7 @@
 class PT_hint_list;
 
 /**
-  Consrtuctor.
+  Constructor.
 
   @param thd_arg          The thread handler.
   @param lineno_arg       The starting line number of a hint string in a query.
@@ -114,7 +115,7 @@ int Hint_scanner::scan() {
   }
 }
 
-void HINT_PARSER_error(THD *thd MY_ATTRIBUTE((unused)), Hint_scanner *scanner,
+void HINT_PARSER_error(THD *thd [[maybe_unused]], Hint_scanner *scanner,
                        PT_hint_list **, const char *msg) {
   if (strcmp(msg, "syntax error") == 0)
     msg = ER_THD(thd, ER_WARN_OPTIMIZER_HINT_SYNTAX_ERROR);
@@ -146,7 +147,7 @@ void Hint_scanner::syntax_warning(const char *msg) const {
   Add hint tokens to main lexer's digest calculation buffer.
 */
 void Hint_scanner::add_hint_token_digest() {
-  if (digest_state == nullptr) return;  // cant add: digest buffer is full
+  if (digest_state == nullptr) return;  // can't add: digest buffer is full
 
   if (prev_token == 0 || prev_token == HINT_ERROR) return;  // nothing to add
 
