@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -44,17 +45,17 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "trx0trx.h"
 
 /** Check if SDI Index exists in a tablespace
-@param[in]	tablespace	tablespace object
-@param[in,out]	space_id	space_id from tablespace object
+@param[in]      dd_space  DD tablespace object
+@param[in,out]  space_id  space_id from tablespace object
 @return DB_SUCCESS if SDI exists, else return DB_ERROR,
 DB_TABLESPACE_NOT_FOUND */
-static dberr_t dict_sdi_exists(const dd::Tablespace &tablespace,
-                               uint32 *space_id) {
-  if (tablespace.se_private_data().get(dd_space_key_strings[DD_SPACE_ID],
-                                       space_id)) {
+static dberr_t dict_sdi_exists(const dd::Tablespace &dd_space,
+                               uint32_t *space_id) {
+  if (dd_space.se_private_data().get(dd_space_key_strings[DD_SPACE_ID],
+                                     space_id)) {
     /* error, attribute not found */
-    ut_ad(0);
-    return (DB_ERROR);
+    ut_d(ut_error);
+    ut_o(return (DB_ERROR));
   }
 
   ut_ad(check_trx_exists(current_thd) != nullptr);
@@ -68,12 +69,12 @@ static dberr_t dict_sdi_exists(const dd::Tablespace &tablespace,
 }
 
 /** Report error on failure
-@param[in]	errornum	MySQL error number (for my_error()) (Must take
+@param[in]      errornum        MySQL error number (for my_error()) (Must take
                                 4 string arguments, in the same way as
                                 ER_SDI_OPERATION_FAILED)
-@param[in]	operation	SDI set or delete
-@param[in]	table		table object for which SDI is serialized
-@param[in]	tablespace	tablespace where SDI is stored */
+@param[in]      operation       SDI set or delete
+@param[in]      table           table object for which SDI is serialized
+@param[in]      tablespace      tablespace where SDI is stored */
 static void dict_sdi_report_error(int errornum, const char *operation,
                                   const dd::Table *table,
                                   const dd::Tablespace &tablespace) {
@@ -108,9 +109,9 @@ const char *operation, const dd::Table *table, const
 dd::Tablespace &tablespace) with errornum=SDI_OPERATION_FAILED
 (for compatibility with existing code).
 
-@param[in]	operation	SDI set or delete
-@param[in]	table		table object for which SDI is serialized
-@param[in]	tablespace	tablespace where SDI is stored */
+@param[in]      operation       SDI set or delete
+@param[in]      table           table object for which SDI is serialized
+@param[in]      tablespace      tablespace where SDI is stored */
 static void dict_sdi_report_error(const char *operation, const dd::Table *table,
                                   const dd::Tablespace &tablespace) {
   dict_sdi_report_error(ER_SDI_OPERATION_FAILED, operation, table, tablespace);
@@ -118,20 +119,20 @@ static void dict_sdi_report_error(const char *operation, const dd::Table *table,
 
 /** Create SDI in a tablespace. This API should be used when
 upgrading a tablespace with no SDI.
-@param[in,out]	tablespace	tablespace object
-@retval		false		success
-@retval		true		failure */
+@param[in,out]  tablespace      tablespace object
+@retval         false           success
+@retval         true            failure */
 bool dict_sdi_create(dd::Tablespace *tablespace) {
   DBUG_EXECUTE_IF("ib_sdi", ib::info(ER_IB_MSG_213)
                                 << "SDI_CREATE: dict_sdi_create("
                                 << tablespace->name() << "," << tablespace->id()
                                 << ")";);
 
-  uint32 space_id = 0;
+  uint32_t space_id = 0;
   if (tablespace->se_private_data().get("id", &space_id)) {
     /* error, attribute not found */
-    ut_ad(0);
-    return (true);
+    ut_d(ut_error);
+    ut_o(return (true));
   }
 
   if (fsp_is_undo_tablespace(space_id) || fsp_is_system_temporary(space_id)) {
@@ -161,38 +162,38 @@ bool dict_sdi_create(dd::Tablespace *tablespace) {
 
 /** Drop SDI in a tablespace. This API should be used only
 when SDI is corrupted.
-@param[in,out]	tablespace	tablespace object
-@retval		false		success
-@retval		true		failure */
-bool dict_sdi_drop(dd::Tablespace *tablespace) {
+@param[in,out]  tablespace      tablespace object
+@retval         false           success
+@retval         true            failure */
+bool dict_sdi_drop(dd::Tablespace *) {
 #if 0  /* TODO: Enable in WL#9761 */
-	uint32	space_id;
-	if (dict_sdi_exists(tablespace, &space_id)
-	    != DB_SUCCESS) {
-		return(true);
-	}
+        uint32  space_id;
+        if (dict_sdi_exists(tablespace, &space_id)
+            != DB_SUCCESS) {
+                return(true);
+        }
 
-	dberr_t	err = ib_sdi_drop(space_id);
-	return(err != DB_SUCCESS);
+        dberr_t err = ib_sdi_drop(space_id);
+        return(err != DB_SUCCESS);
 #endif /* TODO: Enable in WL#9761 */
-  ut_ad(0);
-  return (false);
+  ut_d(ut_error);
+  ut_o(return (false));
 }
 
 /** Get the SDI keys in a tablespace into the vector provided.
-@param[in]	tablespace	tablespace object
-@param[in,out]	vector		vector to hold SDI keys
-@retval		false		success
-@retval		true		failure */
+@param[in]      tablespace      tablespace object
+@param[in,out]  vector          vector to hold SDI keys
+@retval         false           success
+@retval         true            failure */
 bool dict_sdi_get_keys(const dd::Tablespace &tablespace, sdi_vector_t &vector) {
-  uint32 space_id;
+  uint32_t space_id;
 
   if (dd_tablespace_is_discarded(&tablespace)) {
     /* sdi_get_keys shouldn't be called on discarded tablespaces.*/
-    ut_ad(false);
     my_error(ER_SDI_GET_KEYS_INVALID_TABLESPACE, MYF(0),
              tablespace.name().c_str());
-    return true;
+    ut_d(ut_error);
+    ut_o(return true);
   }
 
   if (dict_sdi_exists(tablespace, &space_id) != DB_SUCCESS) {
@@ -202,120 +203,110 @@ bool dict_sdi_get_keys(const dd::Tablespace &tablespace, sdi_vector_t &vector) {
   }
 
   if (fsp_is_undo_tablespace(space_id) || fsp_is_system_temporary(space_id)) {
-    /* There shouldn't be access for SDI on these tabelspaces as
+    /* There shouldn't be access for SDI on these tablespaces as
     SDI doesn't exist. */
-    ut_ad(false);
 
     my_error(ER_SDI_GET_KEYS_INVALID_TABLESPACE, MYF(0),
              tablespace.name().c_str());
-    return true;
+    ut_d(ut_error);
+    ut_o(return true);
   }
 
   ib_sdi_vector ib_vector;
   ib_vector.sdi_vector = &vector;
 
   trx_t *trx = check_trx_exists(current_thd);
-  trx_start_if_not_started(trx, true);
+  trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
 
   dberr_t err = ib_sdi_get_keys(space_id, &ib_vector, trx);
 
   return (err != DB_SUCCESS);
 }
 
-/** Retrieve SDI from tablespace.
-@param[in]	tablespace	tablespace object
-@param[in]	sdi_key		SDI key
-@param[in,out]	sdi		SDI retrieved from tablespace
-@param[in,out]	sdi_len		in:  size of memory allocated
-                                out: actual length of SDI
-@retval		false		success
-@retval		true		in case of failures like record not found,
-                                sdi_len is UINT64MAX_T, else sdi_len is
-                                actual length of SDI */
-bool dict_sdi_get(const dd::Tablespace &tablespace, const sdi_key_t *sdi_key,
-                  void *sdi, uint64 *sdi_len) {
+bool dict_sdi_get(const dd::Tablespace &, const sdi_key_t *, void *,
+                  uint64_t *) {
 #if 0 /* TODO: Enable in WL#9761 */
-	DBUG_EXECUTE_IF("ib_sdi",
-		ib::info(ER_IB_MSG_214) << "dict_sdi_get(" << tablespace.name()
-			<< "," << tablespace.id()
-			<< " sdi_key: type: " << sdi_key->type
-			<< " id: " << sdi_key->id
-			<< ")";
-	);
+        DBUG_EXECUTE_IF("ib_sdi",
+                ib::info(ER_IB_MSG_214) << "dict_sdi_get(" << tablespace.name()
+                        << "," << tablespace.id()
+                        << " sdi_key: type: " << sdi_key->type
+                        << " id: " << sdi_key->id
+                        << ")";
+        );
 
-	if (dd_tablespace_is_discarded(&tablespace)) {
-		/* sdi_get shouldn't be called on discarded tablespaces.*/
-		ut_ad(0);
-	}
-
-	uint32	space_id;
-	if (dict_sdi_exists(tablespace, &space_id)
-	    != DB_SUCCESS) {
-		return(true);
-	}
-
-#ifdef UNIV_DEBUG
-	if (fsp_is_undo_tablespace(space_id)
-	    || fsp_is_system_temporary(space_id)) {
-		/* There shouldn't be access for SDI on these tabelspaces as
-		SDI doesn't exist. */
-		ut_ad(0);
-	}
-#endif /* UNIV_DEBUG */
-
-	trx_t*	trx = check_trx_exists(current_thd);
-	trx_start_if_not_started(trx, true);
-
-	ib_sdi_key_t	ib_sdi_key;
-	ib_sdi_key.sdi_key = sdi_key;
-
-	ut_ad(*sdi_len < UINT32_MAX);
-	uint32_t	uncompressed_sdi_len;
-	uint32_t	compressed_sdi_len = static_cast<uint32_t>(*sdi_len);
-	byte*		compressed_sdi = static_cast<byte*>(
-		ut_malloc_nokey(compressed_sdi_len));
-
-	dberr_t	err = ib_sdi_get(
-		space_id, &ib_sdi_key,
-		compressed_sdi, &compressed_sdi_len,
-		&uncompressed_sdi_len,
-		trx);
-
-	if (err == DB_OUT_OF_MEMORY) {
-		*sdi_len = uncompressed_sdi_len;
-	} else if (err != DB_SUCCESS) {
-		*sdi_len = UINT64_MAX;
-	} else {
-		*sdi_len = uncompressed_sdi_len;
-		/* Decompress the data */
-		Sdi_Decompressor decompressor(static_cast<byte*>(sdi),
-					      uncompressed_sdi_len,
-					      compressed_sdi,
-					      compressed_sdi_len);
-		decompressor.decompress();
+        if (dd_tablespace_is_discarded(&tablespace)) {
+                /* sdi_get shouldn't be called on discarded tablespaces.*/
+                ut_d(ut_error);
         }
 
-	ut_free(compressed_sdi);
+        uint32  space_id;
+        if (dict_sdi_exists(tablespace, &space_id)
+            != DB_SUCCESS) {
+                return(true);
+        }
 
-	return(err != DB_SUCCESS);
+#ifdef UNIV_DEBUG
+        if (fsp_is_undo_tablespace(space_id)
+            || fsp_is_system_temporary(space_id)) {
+                /* There shouldn't be access for SDI on these tablespaces as
+                SDI doesn't exist. */
+                ut_d(ut_error);
+        }
+#endif /* UNIV_DEBUG */
+
+        trx_t*  trx = check_trx_exists(current_thd);
+        trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
+
+        ib_sdi_key_t    ib_sdi_key;
+        ib_sdi_key.sdi_key = sdi_key;
+
+        ut_ad(*sdi_len < UINT32_MAX);
+        uint32_t        uncompressed_sdi_len;
+        uint32_t        compressed_sdi_len = static_cast<uint32_t>(*sdi_len);
+        byte*           compressed_sdi = static_cast<byte*>(
+                ut::malloc_withkey(UT_NEW_THIS_FILE_PSI_KEY, compressed_sdi_len));
+
+        dberr_t err = ib_sdi_get(
+                space_id, &ib_sdi_key,
+                compressed_sdi, &compressed_sdi_len,
+                &uncompressed_sdi_len,
+                trx);
+
+        if (err == DB_OUT_OF_MEMORY) {
+                *sdi_len = uncompressed_sdi_len;
+        } else if (err != DB_SUCCESS) {
+                *sdi_len = UINT64_MAX;
+        } else {
+                *sdi_len = uncompressed_sdi_len;
+                /* Decompress the data */
+                Sdi_Decompressor decompressor(static_cast<byte*>(sdi),
+                                              uncompressed_sdi_len,
+                                              compressed_sdi,
+                                              compressed_sdi_len);
+                decompressor.decompress();
+        }
+
+        ut::free(compressed_sdi);
+
+        return(err != DB_SUCCESS);
 #endif /* TODO: Enable in WL#9761 */
-  ut_ad(0);
-  return (false);
+  ut_d(ut_error);
+  ut_o(return (false));
 }
 
 /** Insert/Update SDI in tablespace
-@param[in]	hton		handlerton object
-@param[in]	tablespace	tablespace object
-@param[in]	table		table object
-@param[in]	sdi_key		SDI key to uniquely identify the tablespace
+@param[in]      hton            handlerton object
+@param[in]      tablespace      tablespace object
+@param[in]      table           table object
+@param[in]      sdi_key         SDI key to uniquely identify the tablespace
 object
-@param[in]	sdi		SDI to be stored in tablespace
-@param[in]	sdi_len		SDI length
-@retval		false		success
-@retval		true		failure */
+@param[in]      sdi             SDI to be stored in tablespace
+@param[in]      sdi_len         SDI length
+@retval         false           success
+@retval         true            failure */
 bool dict_sdi_set(handlerton *hton, const dd::Tablespace &tablespace,
                   const dd::Table *table, const sdi_key_t *sdi_key,
-                  const void *sdi, uint64 sdi_len) {
+                  const void *sdi, uint64_t sdi_len) {
   const char *operation = "set";
 
   DBUG_EXECUTE_IF("ib_sdi", ib::info(ER_IB_MSG_215)
@@ -359,16 +350,16 @@ bool dict_sdi_set(handlerton *hton, const dd::Tablespace &tablespace,
     return (false);
   }
 
-  uint32 space_id;
+  uint32_t space_id;
   dberr_t err = dict_sdi_exists(tablespace, &space_id);
   if (err != DB_SUCCESS) {
     if (err == DB_TABLESPACE_NOT_FOUND) {
       /* Claim Success */
       return (false);
     } else {
-      ut_ad(0);
       dict_sdi_report_error(operation, table, tablespace);
-      return (true);
+      ut_d(ut_error);
+      ut_o(return (true));
     }
   }
 
@@ -378,7 +369,7 @@ bool dict_sdi_set(handlerton *hton, const dd::Tablespace &tablespace,
   }
 
   trx_t *trx = check_trx_exists(current_thd);
-  trx_start_if_not_started(trx, true);
+  trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
 
   innobase_register_trx(hton, current_thd, trx);
 
@@ -406,21 +397,21 @@ bool dict_sdi_set(handlerton *hton, const dd::Tablespace &tablespace,
                         << " is interrupted";);
     return (true);
   } else if (err != DB_SUCCESS) {
-    ut_ad(0);
     dict_sdi_report_error(operation, table, tablespace);
-    return (true);
+    ut_d(ut_error);
+    ut_o(return (true));
   } else {
     return (false);
   }
 }
 
 /** Delete SDI from tablespace
-@param[in]	tablespace	tablespace object
-@param[in]	table		table object
-@param[in]	sdi_key		SDI key to uniquely identify the tablespace
+@param[in]      tablespace      tablespace object
+@param[in]      table           table object
+@param[in]      sdi_key         SDI key to uniquely identify the tablespace
                                 object
-@retval		false		success
-@retval		true		failure */
+@retval         false           success
+@retval         true            failure */
 bool dict_sdi_delete(const dd::Tablespace &tablespace, const dd::Table *table,
                      const sdi_key_t *sdi_key) {
   const char *operation = "delete";
@@ -460,16 +451,16 @@ bool dict_sdi_delete(const dd::Tablespace &tablespace, const dd::Table *table,
     return (false);
   }
 
-  uint32 space_id;
+  uint32_t space_id;
   dberr_t err = dict_sdi_exists(tablespace, &space_id);
   if (err != DB_SUCCESS) {
     if (err == DB_TABLESPACE_NOT_FOUND) {
       /* Claim Success */
       return (false);
     } else {
-      ut_ad(0);
       dict_sdi_report_error(operation, table, tablespace);
-      return (true);
+      ut_d(ut_error);
+      ut_o(return (true));
     }
   }
 
@@ -479,7 +470,7 @@ bool dict_sdi_delete(const dd::Tablespace &tablespace, const dd::Table *table,
   }
 
   trx_t *trx = check_trx_exists(current_thd);
-  trx_start_if_not_started(trx, true);
+  trx_start_if_not_started(trx, true, UT_LOCATION_HERE);
 
   ib_sdi_key_t ib_sdi_key;
   ib_sdi_key.sdi_key = sdi_key;
