@@ -430,6 +430,12 @@ class TransporterRegistry {
    */
   Uint32 get_connect_count(TrpId trpId) const;
 
+  /**
+   * Update send buffer allocated and used bytes for given transporter
+   */
+  void update_send_buffer_usage(TrpId trpId, Uint64 allocBytes,
+                                Uint64 usedBytes);
+
   /**
    * Set or clear overloaded bit.
    * Query if any overloaded bit is set.
@@ -454,6 +460,14 @@ class TransporterRegistry {
    */
   Uint32 get_slowdown_count(NodeId nodeId) const;
 
+  /**
+   * Get SendBuffer alloc + usage info
+   */
+  Uint64 get_send_buffer_alloc_bytes(TrpId trpId) const;
+  Uint64 get_send_buffer_used_bytes(TrpId trpId) const;
+  Uint64 get_send_buffer_max_alloc_bytes(TrpId trpId) const;
+  Uint64 get_send_buffer_max_used_bytes(TrpId trpId) const;
+
   /**
    * prepareSend
    *
@@ -522,6 +536,7 @@ class TransporterRegistry {
   Transporter *get_node_base_transporter(NodeId nodeId) const;
   Transporter *get_node_transporter_instance(NodeId nodeId, int inst) const;
   bool is_shm_transporter(TrpId trp_id);
+  TransporterType get_transporter_type(TrpId id) const;
 
   ndb_sockaddr get_connect_address_node(NodeId nodeId) const;
   ndb_sockaddr get_connect_address(TrpId trpId) const;
