@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,6 +28,7 @@
 
 #include <functional>
 
+#include "m_string.h"                                 // is_prefix
 #include "storage/ndb/plugin/ha_ndbcluster_binlog.h"  // ndbcluster_binlog_setup_table
 #include "storage/ndb/plugin/ndb_dd.h"                // ndb_dd_fs_name_case
 #include "storage/ndb/plugin/ndb_dd_client.h"         // Ndb_dd_client
@@ -141,6 +143,17 @@ bool Ndb_dd_sync::remove_all_metadata() const {
     ndb_log_verbose(50, "Found %zu NDB tables in schema '%s'",
                     ndb_tables.size(), schema_name);
     for (const std::string &table_name : ndb_tables) {
+      // Check if the table has a trigger. Such tables are handled differently
+      // and not deleted as that would result in the trigger being deleted as
+      // well
+      const dd::Table *table_def;
+      if (!dd_client.get_table(schema_name, table_name.c_str(), &table_def)) {
+        ndb_log_error("Failed to open table '%s.%s' from DD", schema_name,
+                      table_name.c_str());
+        return false;
+      }
+      if (ndb_dd_table_has_trigger(table_def)) continue;
+
       ndb_log_info("Removing table '%s.%s'", schema_name, table_name.c_str());
       if (!remove_table(schema_name, table_name.c_str())) {
         ndb_log_error("Failed to remove table '%s.%s' from DD", schema_name,
@@ -200,12 +213,13 @@ bool Ndb_dd_sync::remove_deleted_tables() const {
     }
 
     // Fetch list of tables in NDB. The util tables are skipped since the
-    // ndb_schema, ndb_schema_result, and ndb_sql_metadata tables are handled
-    // separately during binlog setup. The index stat tables are not installed
-    // in the DD.
+    // ndb_schema, ndb_schema_result, ndb_apply_status, ndb_sql_metadata
+    // and index stat tables are handled separately during binlog setup.
     std::unordered_set<std::string> tables_in_NDB;
+    std::unordered_set<std::string> temp_tables_in_NDB;
     if (!ndb_get_table_names_in_schema(m_thd_ndb->ndb->getDictionary(),
-                                       schema_name, &tables_in_NDB)) {
+                                       schema_name, &tables_in_NDB,
+                                       &temp_tables_in_NDB)) {
       log_NDB_error(m_thd_ndb->ndb->getDictionary()->getNdbError());
       ndb_log_error(
           "Failed to get list of NDB tables in schema '%s' from "
@@ -218,6 +232,8 @@ bool Ndb_dd_sync::remove_deleted_tables() const {
                     "Found %zu NDB tables in schema '%s' in the NDB Dictionary",
                     tables_in_NDB.size(), schema_name);
 
+    remove_copying_alter_temp_tables(schema_name, temp_tables_in_NDB);
+
     // Iterate over all NDB tables found in DD. If they don't exist in NDB
     // anymore, then remove the table from DD
     for (const std::string &ndb_table_name : ndb_tables_in_DD) {
@@ -1172,6 +1188,9 @@ bool Ndb_dd_sync::synchronize_table(const char *schema_name,
                                     const char *table_name) const {
   ndb_log_verbose(1, "Synchronizing table '%s.%s'", schema_name, table_name);
 
+  // Invalidate potentially stale cached table
+  Ndb_table_guard::invalidate_table(m_thd_ndb->ndb, schema_name, table_name);
+
   Ndb_table_guard ndbtab_g(m_thd_ndb->ndb, schema_name, table_name);
   const NdbDictionary::Table *ndbtab = ndbtab_g.get_table();
   if (!ndbtab) {
@@ -1249,9 +1268,8 @@ bool Ndb_dd_sync::synchronize_table(const char *schema_name,
     return true;  // Skipped
   }
 
-  int table_id, table_version;
-  if (!ndb_dd_table_get_object_id_and_version(existing, table_id,
-                                              table_version)) {
+  Ndb_dd_handle dd_handle = ndb_dd_table_get_spi_and_version(existing);
+  if (!dd_handle.valid()) {
     ndb_log_error(
         "Failed to extract id and version from table definition "
         "for table '%s.%s'",
@@ -1260,18 +1278,22 @@ bool Ndb_dd_sync::synchronize_table(const char *schema_name,
     return false;
   }
 
-  // Check that latest version of table definition for this NDB table
-  // is installed in DD
-  if (ndbtab->getObjectId() != table_id ||
-      ndbtab->getObjectVersion() != table_version) {
-    ndb_log_info("Table '%s.%s' have different version in DD, reinstalling...",
-                 schema_name, table_name);
-    if (!install_table(schema_name, table_name, ndbtab,
-                       true /* need overwrite */)) {
-      // Failed to create table from NDB
-      ndb_log_error("Failed to install table '%s.%s' from NDB", schema_name,
-                    table_name);
-      return false;
+  {
+    // Check that latest version of table definition for this NDB table
+    // is installed in DD
+    Ndb_dd_handle curr_handle{ndbtab->getObjectId(),
+                              ndbtab->getObjectVersion()};
+    if (curr_handle != dd_handle) {
+      ndb_log_info(
+          "Table '%s.%s' have different version in DD, reinstalling...",
+          schema_name, table_name);
+      if (!install_table(schema_name, table_name, ndbtab,
+                         true /* need overwrite */)) {
+        // Failed to create table from NDB
+        ndb_log_error("Failed to install table '%s.%s' from NDB", schema_name,
+                      table_name);
+        return false;
+      }
     }
   }
 
@@ -1307,22 +1329,10 @@ bool Ndb_dd_sync::synchronize_schema(const char *schema_name) const {
   std::unordered_set<std::string> ndb_tables_in_NDB;
   NdbDictionary::Dictionary *dict = m_thd_ndb->ndb->getDictionary();
   /*
-    Fetch list of tables in NDB. The util tables are skipped since the
-    ndb_schema, ndb_schema_result, and ndb_sql_metadata tables are handled
-    separately during binlog setup. The index stat tables are not installed in
-    the DD. This is due to an issue after an initial system restart. The binlog
-    thread of the first MySQL Server connecting to the Cluster post an initial
-    restart pokes the index stat thread to create the index stat tables in NDB.
-    This happens only after the synchronization phase during binlog setup which
-    means that the tables aren't synced to the DD of the first MySQL Server
-    connecting to the Cluster. If there are multiple MySQL Servers connecting to
-    the Cluster, there's a race condition where the index stat tables could be
-    synchronized during subsequent MySQL Server connections depending on when
-    the index stat thread creates them in NDB. If the creation occurs in the
-    middle of the sync during binlog setup of a Server, it opens the door to
-    errors in the sync. The contents of these tables are incomprehensible
-    without some kind of parsing and are thus not exposed to the MySQL Server.
-    They remain visible and accessible via the ndb_select_all tool.
+    Fetch list of tables in NDB.
+    The util tables are skipped since the ndb_schema, ndb_schema_result,
+    ndb_apply_status, ndb_sql_metadata and index stat tables are handled
+    separately during binlog setup.
   */
   if (!ndb_get_table_names_in_schema(dict, schema_name, &ndb_tables_in_NDB)) {
     log_NDB_error(dict->getNdbError());
@@ -1390,3 +1400,38 @@ bool Ndb_dd_sync::synchronize() const {
   ndb_log_info("Completed metadata synchronization");
   return true;
 }
+
+void Ndb_dd_sync::remove_copying_alter_temp_tables(
+    const char *schema_name,
+    const std::unordered_set<std::string> &temp_tables_in_ndb) const {
+  for (const std::string &ndb_table_name : temp_tables_in_ndb) {
+    // if the table starts with #sql2, it's the table left behind after
+    // renaming original table to temporary one, cannot be deleted to prevent
+    // data loss
+    if (is_prefix(ndb_table_name.c_str(), "#sql2")) {
+      ndb_log_error(
+          "Found temporary table %s.%s, which is most likely left behind"
+          " by failed copying alter table",
+          schema_name, ndb_table_name.c_str());
+      continue;
+    }
+
+    // the table is temporary and does not start with prefix #sql2,
+    // so it must have been left behind before renaming orignal table,
+    // if so, it can be deleted to cleanup unfinished copying alter table
+    ndb_log_warning(
+        "Found temporary table %s.%s, wich is most likely left behind by failed"
+        " copying alter table, this table will be removed, the operation"
+        " does not affect original data",
+        schema_name, ndb_table_name.c_str());
+    Ndb_table_guard ndbtab_g(m_thd_ndb->ndb, schema_name,
+                             ndb_table_name.c_str());
+    auto ndbtab = *ndbtab_g.get_table();
+    constexpr int flag = NdbDictionary::Dictionary::DropTableCascadeConstraints;
+
+    if (m_thd_ndb->ndb->getDictionary()->dropTableGlobal(ndbtab, flag)) {
+      log_NDB_error(m_thd_ndb->ndb->getDictionary()->getNdbError());
+      ndb_log_error("Cannot drop %s.%s", schema_name, ndb_table_name.c_str());
+    }
+  }
+}
