@@ -503,7 +503,6 @@ void Trpman::execDBINFO_SCANREQ(Signal *signal) {
 
         /* Connect count, overload and slowdown states */
         row.write_uint32(globalTransporterRegistry.get_connect_count(trpId));
-
         /* FIXME: overload & slowdown is still pr NodeId */
         row.write_uint32(
             globalTransporterRegistry.get_status_overloaded().get(nodeId));
@@ -515,6 +514,19 @@ void Trpman::execDBINFO_SCANREQ(Signal *signal) {
         /* TLS, not available until 8.3 -> is_encrypted=false*/
         row.write_uint32(false);
 
+        /* Send buffer bytes/pages usage */
+        row.write_uint64(
+            globalTransporterRegistry.get_send_buffer_used_bytes(trpId));
+        row.write_uint64(
+            globalTransporterRegistry.get_send_buffer_max_used_bytes(trpId));
+        row.write_uint64(
+            globalTransporterRegistry.get_send_buffer_alloc_bytes(trpId));
+        row.write_uint64(
+            globalTransporterRegistry.get_send_buffer_max_alloc_bytes(trpId));
+
+        /* Transporter type */
+        row.write_uint32(globalTransporterRegistry.get_transporter_type(trpId));
+
         ndbinfo_send_row(signal, req, row, rl);
         trpId++;
         if (rl.need_break(req)) {
