@@ -1,31 +1,54 @@
-/* Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  it under the terms of the GNU General Public License, version 2.0,
+  as published by the Free Software Foundation.
+
+  This program is designed to work with certain software (including
+  but not limited to OpenSSL) that is licensed under separate terms,
+  as designated in a particular file or component or in included license
+  documentation.  The authors of MySQL hereby grant you an additional
+  permission to link the program and your derivative works with the
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software Foundation,
-  51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #ifndef PFS_INSTR_CLASS_H
 #define PFS_INSTR_CLASS_H
 
-#include "my_global.h"
-#include "mysql_com.h"                          /* NAME_LEN */
+#include "my_config.h"
+
+#include <assert.h>
+#include <sys/types.h>
+#include <atomic>
+
 #include "lf.h"
-#include "pfs_global.h"
-#include "pfs_atomic.h"
+#include "my_compiler.h"
+
+#include "my_inttypes.h"
+#include "mysql_com.h" /* NAME_LEN */
+#include "mysqld_error.h"
 #include "prealloced_array.h"
+#include "storage/perfschema/pfs_column_types.h"
+#include "storage/perfschema/pfs_global.h"
+#include "storage/perfschema/pfs_lock.h"
+#include "storage/perfschema/pfs_name.h"
+#include "storage/perfschema/pfs_stat.h"
+#include "storage/perfschema/terminology_use_previous_enum.h"
+
+struct TABLE_SHARE;
 
 /**
   @file storage/perfschema/pfs_instr_class.h
-  Performance schema instruments meta data (declarations).
+  Performance schema instruments metadata (declarations).
 */
 
 /**
@@ -34,6 +57,14 @@
 */
 #define PFS_MAX_INFO_NAME_LENGTH 128
 
+/**
+  Maximum length of the thread os name.
+  Must include a terminating NUL character.
+  Length is 16 because of linux pthread_setname_np(3)
+  @see my_thread_self_setname()
+*/
+#define PFS_MAX_OS_NAME_LENGTH 16
+
 /**
   Maximum length of the 'full' prefix of an instrument name.
   For example, for the instrument name 'wait/sync/mutex/sql/LOCK_open',
@@ -43,23 +74,19 @@
 */
 #define PFS_MAX_FULL_PREFIX_NAME_LENGTH 32
 
-#include <my_global.h>
-#include <my_sys.h>
-#include <mysql/psi/psi.h>
-#include "pfs_lock.h"
-#include "pfs_stat.h"
-#include "pfs_column_types.h"
-
 struct PFS_global_param;
 struct PFS_table_share;
+class PFS_opaque_container_page;
 
 /**
-  @addtogroup Performance_schema_buffers
+  @addtogroup performance_schema_buffers
   @{
 */
 
-extern my_bool pfs_enabled;
-extern enum_timer_name *class_timers[];
+extern bool pfs_enabled;
+
+/** Global ref count for plugin and component events. */
+extern std::atomic<uint32> pfs_unload_plugin_ref_count;
 
 /** Key, naming a synch instrument (mutex, rwlock, cond). */
 typedef unsigned int PFS_sync_key;
@@ -78,30 +105,30 @@ typedef unsigned int PFS_socket_key;
 /** Key, naming a memory instrument. */
 typedef unsigned int PFS_memory_key;
 
-enum PFS_class_type
-{
-  PFS_CLASS_NONE=        0,
-  PFS_CLASS_MUTEX=       1,
-  PFS_CLASS_RWLOCK=      2,
-  PFS_CLASS_COND=        3,
-  PFS_CLASS_FILE=        4,
-  PFS_CLASS_TABLE=       5,
-  PFS_CLASS_STAGE=       6,
-  PFS_CLASS_STATEMENT=   7,
-  PFS_CLASS_TRANSACTION= 8,
-  PFS_CLASS_SOCKET=      9,
-  PFS_CLASS_TABLE_IO=   10,
-  PFS_CLASS_TABLE_LOCK= 11,
-  PFS_CLASS_IDLE=       12,
-  PFS_CLASS_MEMORY=     13,
-  PFS_CLASS_METADATA=   14,
-  PFS_CLASS_LAST=       PFS_CLASS_METADATA,
-  PFS_CLASS_MAX=        PFS_CLASS_LAST + 1
+enum PFS_class_type {
+  PFS_CLASS_NONE = 0,
+  PFS_CLASS_MUTEX = 1,
+  PFS_CLASS_RWLOCK = 2,
+  PFS_CLASS_COND = 3,
+  PFS_CLASS_FILE = 4,
+  PFS_CLASS_TABLE = 5,
+  PFS_CLASS_STAGE = 6,
+  PFS_CLASS_STATEMENT = 7,
+  PFS_CLASS_TRANSACTION = 8,
+  PFS_CLASS_SOCKET = 9,
+  PFS_CLASS_TABLE_IO = 10,
+  PFS_CLASS_TABLE_LOCK = 11,
+  PFS_CLASS_IDLE = 12,
+  PFS_CLASS_MEMORY = 13,
+  PFS_CLASS_METADATA = 14,
+  PFS_CLASS_ERROR = 15,
+  PFS_CLASS_THREAD = 16,
+  PFS_CLASS_LAST = PFS_CLASS_THREAD,
+  PFS_CLASS_MAX = PFS_CLASS_LAST + 1
 };
 
 /** User-defined instrument configuration. */
-struct PFS_instr_config
-{
+struct PFS_instr_config {
   /* Instrument name. */
   char *m_name;
   /* Name length. */
@@ -112,7 +139,7 @@ struct PFS_instr_config
   bool m_timed;
 };
 
-typedef Prealloced_array<PFS_instr_config*, 10> Pfs_instr_config_array;
+typedef Prealloced_array<PFS_instr_config *, 10> Pfs_instr_config_array;
 extern Pfs_instr_config_array *pfs_instr_config_array;
 
 struct PFS_thread;
@@ -124,9 +151,58 @@ extern uint file_class_start;
 extern uint socket_class_start;
 extern uint wait_class_max;
 
+/**
+  Encapsulates the name of an instrumented entity.
+*/
+class PFS_instr_name {
+ public:
+  static constexpr uint max_length = PFS_MAX_INFO_NAME_LENGTH - 1;
+  /*
+    DO NOT ACCESS THE DATA MEMBERS DIRECTLY.  USE THE GETTERS AND
+    SETTERS INSTEAD.
+
+    The data members should really have been private, but having both
+    private and public members would make the class a non-POD.  We
+    need to call memset on PFS_instr_class (in init_instr_class), and
+    the behavior of memset is undefined on non-POD objects.  Therefore
+    we keep the data members public, with an underscore prefix, and
+    this warning text.
+  */
+ public /*private*/:
+  /** Instrument name. */
+  char m_private_name[max_length + 1];
+  /** Length in bytes of @c m_name. */
+  uint m_private_name_length;
+  /** Old instrument name, if any. */
+  const char *m_private_old_name;
+  /** Length in bytes of old instrument name len, if any. */
+  uint m_private_old_name_length;
+  /** The oldest version that uses the new name. */
+  terminology_use_previous::enum_compatibility_version m_private_version;
+
+ public:
+  /** Return the name as a string. */
+  const char *str() const;
+  /** Return the length of the string. */
+  uint length() const;
+  /**
+    Copy the specified name to this name.
+
+    @param class_type The class type of this name, i.e., whether it is
+    the name of a mutex, thread, etc.
+
+    @param name The buffer to read from.
+
+    @param max_length_arg If is given, at most that many chars are
+    copied, plus the terminating '\0'. Otherwise, up to buffer_size-1
+    characters are copied, plus the terminating '\0'.
+  */
+  void set(PFS_class_type class_type, const char *name,
+           uint max_length_arg = max_length);
+};
+
 /** Information for all instrumentation. */
-struct PFS_instr_class
-{
+struct PFS_instr_class {
   /** Class type */
   PFS_class_type m_type;
   /** True if this instrument is enabled. */
@@ -134,7 +210,11 @@ struct PFS_instr_class
   /** True if this instrument is timed. */
   bool m_timed;
   /** Instrument flags. */
-  int m_flags;
+  uint m_flags;
+  /** Instrument enforced flags. */
+  uint m_enforced_flags;
+  /** Volatility index. */
+  int m_volatility;
   /**
     Instrument name index.
     Self index in:
@@ -145,41 +225,65 @@ struct PFS_instr_class
   */
   uint m_event_name_index;
   /** Instrument name. */
-  char m_name[PFS_MAX_INFO_NAME_LENGTH];
-  /** Length in bytes of @c m_name. */
-  uint m_name_length;
-  /** Timer associated with this class. */
-  enum_timer_name *m_timer;
+  PFS_instr_name m_name;
+  /** Documentation. */
+  char *m_documentation;
 
-  bool is_singleton() const
-  {
-    return m_flags & PSI_FLAG_GLOBAL;
+  bool is_singleton() const { return m_flags & PSI_FLAG_SINGLETON; }
+
+  bool is_mutable() const { return m_flags & PSI_FLAG_MUTABLE; }
+
+  bool is_progress() const { return m_flags & PSI_FLAG_STAGE_PROGRESS; }
+
+  bool is_shared_exclusive() const { return m_flags & PSI_FLAG_RWLOCK_SX; }
+
+  bool is_priority() const { return m_flags & PSI_FLAG_RWLOCK_PR; }
+
+  bool is_transferable() const { return m_flags & PSI_FLAG_TRANSFER; }
+
+  bool is_user() const { return m_flags & PSI_FLAG_USER; }
+
+  bool is_system_thread() const { return m_flags & PSI_FLAG_THREAD_SYSTEM; }
+
+  bool is_global() const { return m_flags & PSI_FLAG_ONLY_GLOBAL_STAT; }
+
+  bool has_seqnum() const {
+    return (m_flags & (PSI_FLAG_SINGLETON | PSI_FLAG_NO_SEQNUM)) == 0;
   }
 
-  bool is_mutable() const
-  {
-    return m_flags & PSI_FLAG_MUTABLE;
+  bool has_auto_seqnum() const { return m_flags & PSI_FLAG_AUTO_SEQNUM; }
+
+  bool has_default_memory_cnt() const { return m_flags & PSI_FLAG_MEM_COLLECT; }
+
+  bool has_enforced_memory_cnt() const {
+    return m_enforced_flags & PSI_FLAG_MEM_COLLECT;
   }
 
-  bool is_progress() const
-  {
-    DBUG_ASSERT(m_type == PFS_CLASS_STAGE);
-    return m_flags & PSI_FLAG_STAGE_PROGRESS;
-  }
+  bool is_disabled() const { return m_flags & PSI_FLAG_DISABLED; }
 
-  bool is_shared_exclusive() const
-  {
-    DBUG_ASSERT(m_type == PFS_CLASS_RWLOCK);
-    return m_flags & PSI_RWLOCK_FLAG_SX;
+  bool is_untimed() const { return m_flags & PSI_FLAG_UNTIMED; }
+
+  void set_enforced_flags(uint flags) { m_enforced_flags = flags; }
+
+  void enforce_valid_flags(uint allowed_flags) {
+    /* Reserved for future use. */
+    allowed_flags |= PSI_FLAG_THREAD | PSI_FLAG_TRANSFER;
+
+    const uint valid_flags = m_flags & allowed_flags;
+    /*
+      This fails when the instrumented code is providing
+      flags that are not supported for this instrument.
+      To fix it, clean up the instrumented code.
+    */
+    assert(valid_flags == m_flags);
+    m_flags = valid_flags;
   }
 
   static void set_enabled(PFS_instr_class *pfs, bool enabled);
   static void set_timed(PFS_instr_class *pfs, bool timed);
 
-  bool is_deferred() const
-  {
-    switch(m_type)
-    {
+  bool is_deferred() const {
+    switch (m_type) {
       case PFS_CLASS_SOCKET:
         return true;
         break;
@@ -188,13 +292,34 @@ struct PFS_instr_class
         break;
     };
   }
+
+  bool can_be_timed() const {
+    switch (m_type) {
+      case PFS_CLASS_MEMORY:
+      case PFS_CLASS_ERROR:
+      case PFS_CLASS_THREAD:
+        return false;
+      default:
+        return true;
+    };
+  }
+
+  bool can_be_enforced() const {
+    switch (m_type) {
+      case PFS_CLASS_MEMORY:
+        return true;
+      default:
+        return false;
+    };
+  }
 };
 
 struct PFS_mutex;
 
-/** Instrumentation metadata for a MUTEX. */
-struct PFS_ALIGNED PFS_mutex_class : public PFS_instr_class
-{
+#define PFS_MUTEX_PARTITIONS 2
+
+/** Instrumentation metadata for a mutex. */
+struct PFS_ALIGNED PFS_mutex_class : public PFS_instr_class {
   /** Mutex usage statistics. */
   PFS_mutex_stat m_mutex_stat;
   /** Singleton instance. */
@@ -203,9 +328,8 @@ struct PFS_ALIGNED PFS_mutex_class : public PFS_instr_class
 
 struct PFS_rwlock;
 
-/** Instrumentation metadata for a RWLOCK. */
-struct PFS_ALIGNED PFS_rwlock_class : public PFS_instr_class
-{
+/** Instrumentation metadata for a read write lock. */
+struct PFS_ALIGNED PFS_rwlock_class : public PFS_instr_class {
   /** Rwlock usage statistics. */
   PFS_rwlock_stat m_rwlock_stat;
   /** Singleton instance. */
@@ -214,9 +338,8 @@ struct PFS_ALIGNED PFS_rwlock_class : public PFS_instr_class
 
 struct PFS_cond;
 
-/** Instrumentation metadata for a COND. */
-struct PFS_ALIGNED PFS_cond_class : public PFS_instr_class
-{
+/** Instrumentation metadata for a condition. */
+struct PFS_ALIGNED PFS_cond_class : public PFS_instr_class {
   /**
     Condition usage statistics.
     This statistic is not exposed in user visible tables yet.
@@ -227,35 +350,31 @@ struct PFS_ALIGNED PFS_cond_class : public PFS_instr_class
 };
 
 /** Instrumentation metadata of a thread. */
-struct PFS_ALIGNED PFS_thread_class
-{
-  /** True if this thread instrument is enabled. */
-  bool m_enabled;
+struct PFS_ALIGNED PFS_thread_class : public PFS_instr_class {
   /** Singleton instance. */
   PFS_thread *m_singleton;
-  /** Thread instrument name. */
-  char m_name[PFS_MAX_INFO_NAME_LENGTH];
-  /** Length in bytes of @c m_name. */
-  uint m_name_length;
+  /** Thread history instrumentation flag. */
+  bool m_history{false};
+  /** Thread os name. */
+  char m_os_name[PFS_MAX_OS_NAME_LENGTH];
+  /** Thread instance sequence number counter. */
+  std::atomic<unsigned int> m_seqnum;
 };
 
 /** Key identifying a table share. */
-struct PFS_table_share_key
-{
-  /**
-    Hash search key.
-    This has to be a string for LF_HASH,
-    the format is "<enum_object_type><schema_name><0x00><object_name><0x00>"
-    @see create_table_def_key
-  */
-  char m_hash_key[1 + NAME_LEN + 1 + NAME_LEN + 1];
-  /** Length in bytes of @c m_hash_key. */
-  uint m_key_length;
+struct PFS_table_share_key {
+  /** Object type. */
+  enum_object_type m_type;
+
+  /** Table schema. */
+  PFS_schema_name m_schema_name;
+
+  /** Table name. */
+  PFS_table_name m_table_name;
 };
 
 /** Table index or 'key' */
-struct PFS_table_key
-{
+struct PFS_table_key {
   /** Index name */
   char m_name[NAME_LEN];
   /** Length in bytes of @c m_name. */
@@ -263,71 +382,55 @@ struct PFS_table_key
 };
 
 /** Index statistics of a table.*/
-struct PFS_table_share_index
-{
+struct PFS_table_share_index {
   pfs_lock m_lock;
   /** The index name */
   PFS_table_key m_key;
   /** The index stat */
   PFS_table_io_stat m_stat;
   /** Owner table share. To be used later. */
-  PFS_table_share* m_owner;
+  PFS_table_share *m_owner;
+  /** Container page. */
+  PFS_opaque_container_page *m_page;
 };
 
 /** Lock statistics of a table.*/
-struct PFS_table_share_lock
-{
+struct PFS_table_share_lock {
   pfs_lock m_lock;
   /** Lock stats. */
   PFS_table_lock_stat m_stat;
   /** Owner table share. To be used later. */
-  PFS_table_share* m_owner;
+  PFS_table_share *m_owner;
+  /** Container page. */
+  PFS_opaque_container_page *m_page;
 };
 
 /** Instrumentation metadata for a table share. */
-struct PFS_ALIGNED PFS_table_share
-{
-public:
-  uint32 get_version()
-  { return m_lock.get_version(); }
+struct PFS_ALIGNED PFS_table_share {
+ public:
+  uint32 get_version() { return m_lock.get_version(); }
 
-  enum_object_type get_object_type()
-  {
-    return (enum_object_type) m_key.m_hash_key[0];
-  }
+  enum_object_type get_object_type() const { return m_key.m_type; }
 
-  void aggregate_io(void);
-  void aggregate_lock(void);
+  void aggregate_io();
+  void aggregate_lock();
 
   void sum_io(PFS_single_stat *result, uint key_count);
   void sum_lock(PFS_single_stat *result);
   void sum(PFS_single_stat *result, uint key_count);
 
-  inline void aggregate(void)
-  {
+  inline void aggregate() {
     aggregate_io();
     aggregate_lock();
   }
 
-  inline void init_refcount(void)
-  {
-    PFS_atomic::store_32(& m_refcount, 1);
-  }
+  inline void init_refcount() { m_refcount.store(1); }
 
-  inline int get_refcount(void)
-  {
-    return PFS_atomic::load_32(& m_refcount);
-  }
+  inline int get_refcount() { return m_refcount.load(); }
 
-  inline void inc_refcount(void)
-  {
-    PFS_atomic::add_32(& m_refcount, 1);
-  }
+  inline void inc_refcount() { ++m_refcount; }
 
-  inline void dec_refcount(void)
-  {
-    PFS_atomic::add_32(& m_refcount, -1);
-  }
+  inline void dec_refcount() { --m_refcount; }
 
   void refresh_setup_object_flags(PFS_thread *thread);
 
@@ -346,37 +449,33 @@ public:
 
   /** Search key. */
   PFS_table_share_key m_key;
-  /** Schema name. */
-  const char *m_schema_name;
-  /** Length in bytes of @c m_schema_name. */
-  uint m_schema_name_length;
-  /** Table name. */
-  const char *m_table_name;
-  /** Length in bytes of @c m_table_name. */
-  uint m_table_name_length;
+
   /** Number of indexes. */
   uint m_key_count;
+  /** Container page. */
+  PFS_opaque_container_page *m_page;
 
   PFS_table_share_lock *find_lock_stat() const;
   PFS_table_share_lock *find_or_create_lock_stat();
   void destroy_lock_stat();
 
   PFS_table_share_index *find_index_stat(uint index) const;
-  PFS_table_share_index *find_or_create_index_stat(const TABLE_SHARE *server_share, uint index);
+  PFS_table_share_index *find_or_create_index_stat(
+      const TABLE_SHARE *server_share, uint index);
   void destroy_index_stats();
 
-private:
+ private:
   /** Number of opened table handles. */
-  int m_refcount;
+  std::atomic<int> m_refcount;
   /** Table locks statistics. */
-  PFS_table_share_lock *m_race_lock_stat;
-  /** Table indexes' stats. */
-  PFS_table_share_index *m_race_index_stat[MAX_INDEXES + 1];
+  std::atomic<PFS_table_share_lock *> m_race_lock_stat;
+  /** Table indexes stats. */
+  std::atomic<PFS_table_share_index *> m_race_index_stat[MAX_INDEXES + 1];
 };
 
 /** Statistics for the IDLE instrument. */
 extern PFS_single_stat global_idle_stat;
-/** Statistics for dropped table io. */
+/** Statistics for dropped table I/O. */
 extern PFS_table_io_stat global_table_io_stat;
 /** Statistics for dropped table lock. */
 extern PFS_table_lock_stat global_table_lock_stat;
@@ -384,11 +483,13 @@ extern PFS_table_lock_stat global_table_lock_stat;
 extern PFS_single_stat global_metadata_stat;
 /** Statistics for the transaction instrument. */
 extern PFS_transaction_stat global_transaction_stat;
+/** Statistics for the error instrument. */
+extern PFS_error_stat global_error_stat;
 
-inline uint sanitize_index_count(uint count)
-{
-  if (likely(count <= MAX_INDEXES))
+inline uint sanitize_index_count(uint count) {
+  if (likely(count <= MAX_INDEXES)) {
     return count;
+  }
   return 0;
 }
 
@@ -402,8 +503,10 @@ inline uint sanitize_index_count(uint count)
 /** Transaction events are not wait events .*/
 #define GLOBAL_TRANSACTION_INDEX 0
 
+#define GLOBAL_ERROR_INDEX 0
+
 /**
-  Instrument controlling all table io.
+  Instrument controlling all table I/O.
   This instrument is used with table SETUP_OBJECTS.
 */
 extern PFS_instr_class global_table_io_class;
@@ -419,13 +522,22 @@ extern PFS_instr_class global_table_lock_class;
 */
 extern PFS_instr_class global_idle_class;
 
+/**
+  Instrument controlling all metadata locks.
+*/
 extern PFS_instr_class global_metadata_class;
 
+/** Instrumentation metadata for an error. */
+struct PFS_ALIGNED PFS_error_class : public PFS_instr_class {};
+/**
+  Instrument controlling all server errors.
+*/
+extern PFS_error_class global_error_class;
+
 struct PFS_file;
 
 /** Instrumentation metadata for a file. */
-struct PFS_ALIGNED PFS_file_class : public PFS_instr_class
-{
+struct PFS_ALIGNED PFS_file_class : public PFS_instr_class {
   /** File usage statistics. */
   PFS_file_stat m_file_stat;
   /** Singleton instance. */
@@ -433,11 +545,10 @@ struct PFS_ALIGNED PFS_file_class : public PFS_instr_class
 };
 
 /** Instrumentation metadata for a stage. */
-struct PFS_ALIGNED PFS_stage_class : public PFS_instr_class
-{
+struct PFS_ALIGNED PFS_stage_class : public PFS_instr_class {
   /**
-    Length of the 'stage/<component>/' prefix.
-    This is to extract 'foo' from 'stage/sql/foo'.
+    Length of the @c "stage/\<component\>/" prefix.
+    This is to extract @c "foo" from @c "stage/sql/foo".
   */
   uint m_prefix_length;
   /** Stage usage statistics. */
@@ -445,22 +556,17 @@ struct PFS_ALIGNED PFS_stage_class : public PFS_instr_class
 };
 
 /** Instrumentation metadata for a statement. */
-struct PFS_ALIGNED PFS_statement_class : public PFS_instr_class
-{
-};
+struct PFS_ALIGNED PFS_statement_class : public PFS_instr_class {};
 
 /** Instrumentation metadata for a transaction. */
-struct PFS_ALIGNED PFS_transaction_class : public PFS_instr_class
-{
-};
+struct PFS_ALIGNED PFS_transaction_class : public PFS_instr_class {};
 
 extern PFS_transaction_class global_transaction_class;
 
 struct PFS_socket;
 
 /** Instrumentation metadata for a socket. */
-struct PFS_ALIGNED PFS_socket_class : public PFS_instr_class
-{
+struct PFS_ALIGNED PFS_socket_class : public PFS_instr_class {
   /** Socket usage statistics. */
   PFS_socket_stat m_socket_stat;
   /** Singleton instance. */
@@ -468,16 +574,13 @@ struct PFS_ALIGNED PFS_socket_class : public PFS_instr_class
 };
 
 /** Instrumentation metadata for a memory. */
-struct PFS_ALIGNED PFS_memory_class : public PFS_instr_class
-{
-};
+struct PFS_ALIGNED PFS_memory_class : public PFS_instr_class {};
 
 void init_event_name_sizing(const PFS_global_param *param);
 
 void register_global_classes();
 
-int init_sync_class(uint mutex_class_sizing,
-                    uint rwlock_class_sizing,
+int init_sync_class(uint mutex_class_sizing, uint rwlock_class_sizing,
                     uint cond_class_sizing);
 
 void cleanup_sync_class();
@@ -488,12 +591,13 @@ void cleanup_table_share();
 
 int init_table_share_lock_stat(uint table_stat_sizing);
 void cleanup_table_share_lock_stat();
-PFS_table_share_lock* create_table_share_lock_stat();
+PFS_table_share_lock *create_table_share_lock_stat();
 void release_table_share_lock_stat(PFS_table_share_lock *pfs);
 
 int init_table_share_index_stat(uint index_stat_sizing);
 void cleanup_table_share_index_stat();
-PFS_table_share_index* create_table_share_index_stat(const TABLE_SHARE *share, uint index);
+PFS_table_share_index *create_table_share_index_stat(const TABLE_SHARE *share,
+                                                     uint index);
 void release_table_share_index_stat(PFS_table_share_index *pfs);
 
 int init_table_share_hash(const PFS_global_param *param);
@@ -510,33 +614,31 @@ int init_memory_class(uint memory_class_sizing);
 void cleanup_memory_class();
 
 PFS_sync_key register_mutex_class(const char *name, uint name_length,
-                                  int flags);
+                                  PSI_mutex_info *info);
 
 PFS_sync_key register_rwlock_class(const char *name, uint name_length,
-                                   int flags);
+                                   PSI_rwlock_info *info);
 
 PFS_sync_key register_cond_class(const char *name, uint name_length,
-                                 int flags);
+                                 PSI_cond_info *info);
 
 PFS_thread_key register_thread_class(const char *name, uint name_length,
-                                     int flags);
+                                     PSI_thread_info *info);
 
 PFS_file_key register_file_class(const char *name, uint name_length,
-                                 int flags);
+                                 PSI_file_info *info);
 
-PFS_stage_key register_stage_class(const char *name,
-                                   uint prefix_length,
-                                   uint name_length,
-                                   int flags);
+PFS_stage_key register_stage_class(const char *name, uint prefix_length,
+                                   uint name_length, PSI_stage_info *info);
 
 PFS_statement_key register_statement_class(const char *name, uint name_length,
-                                           int flags);
+                                           PSI_statement_info *info);
 
 PFS_socket_key register_socket_class(const char *name, uint name_length,
-                                     int flags);
+                                     PSI_socket_info *info);
 
 PFS_memory_key register_memory_class(const char *name, uint name_length,
-                                     int flags);
+                                     PSI_memory_info *info);
 
 PFS_mutex_class *find_mutex_class(PSI_mutex_key key);
 PFS_mutex_class *sanitize_mutex_class(PFS_mutex_class *unsafe);
@@ -562,15 +664,16 @@ PFS_instr_class *find_idle_class(uint index);
 PFS_instr_class *sanitize_idle_class(PFS_instr_class *unsafe);
 PFS_instr_class *find_metadata_class(uint index);
 PFS_instr_class *sanitize_metadata_class(PFS_instr_class *unsafe);
+PFS_error_class *find_error_class(uint index);
+PFS_error_class *sanitize_error_class(PFS_instr_class *unsafe);
 PFS_transaction_class *find_transaction_class(uint index);
-PFS_transaction_class *sanitize_transaction_class(PFS_transaction_class *unsafe);
+PFS_transaction_class *sanitize_transaction_class(
+    PFS_transaction_class *unsafe);
 
-PFS_table_share *find_or_create_table_share(PFS_thread *thread,
-                                            bool temporary,
+PFS_table_share *find_or_create_table_share(PFS_thread *thread, bool temporary,
                                             const TABLE_SHARE *share);
 void release_table_share(PFS_table_share *pfs);
-void drop_table_share(PFS_thread *thread,
-                      bool temporary,
+void drop_table_share(PFS_thread *thread, bool temporary,
                       const char *schema_name, uint schema_name_length,
                       const char *table_name, uint table_name_length);
 
@@ -595,6 +698,7 @@ extern ulong socket_class_max;
 extern ulong socket_class_lost;
 extern ulong memory_class_max;
 extern ulong memory_class_lost;
+extern ulong error_class_max;
 
 /* Exposing the data directly, for iterators. */
 
@@ -615,6 +719,13 @@ void update_program_share_derived_flags(PFS_thread *thread);
 
 extern LF_HASH table_share_hash;
 
+/**
+  Get current time for GTID monitoring.
+
+  @return my_getsystime() when PFS monitoring is enabled.
+  @return 0 when PFS monitoring is disabled.
+*/
+ulonglong gtid_monitoring_getsystime();
+
 /** @} */
 #endif
-
